Package: projoint
Type: Package
Title: Conjoint Analysis with Reliability Correction and Visualization
Version: 1.1.4
Date: 2026-08-28
Maintainer: Yusaku Horiuchi <yusaku.horiuchi@gmail.com>
Authors@R: 
    c(
      person("Yusaku", "Horiuchi", email = "yusaku.horiuchi@gmail.com",
             role = c("aut", "cre"),
             comment = c(ORCID = "0000-0003-0295-4089")),
      person("Aaron", "Kaufman", email = "aaronkaufman@nyu.edu",
             role = "aut",
             comment = c(ORCID = "0000-0003-3688-0428")),
      person("Gary", "King", email = "king@harvard.edu",
             role = "aut",
             comment = c(ORCID = "0000-0002-5327-7631"))
    )
Author: Yusaku Horiuchi [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0295-4089>),
  Aaron Kaufman [aut] (ORCID: <https://orcid.org/0000-0003-3688-0428>),
  Gary King [aut] (ORCID: <https://orcid.org/0000-0002-5327-7631>)
Description: Provides tools for analyzing data generated from conjoint survey experiments, a method widely used in the social sciences for studying multidimensional preferences. The package implements estimation of marginal means (MMs) and average marginal component effects (AMCEs), with corrections for measurement error. Methods include profile-level and choice-level estimators, bias correction using intra-respondent reliability (IRR), and visualization utilities. For details on the methodology, see Clayton, Horiuchi, Kaufman, King, and Komisarchik (2025) <https://gking.harvard.edu/conjointE>.
Imports: dplyr (>= 1.1.2), readr, rlang, tidyr, stringr, tidyselect,
        estimatr, ggthemes, ggplot2, stats, tibble, methods, MASS,
        forcats, scales
Suggests: tidyverse, knitr, rmarkdown, downloadthis, patchwork,
        testthat (>= 3.0.0), DiagrammeR
VignetteBuilder: knitr
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Config/testthat/edition: 3
URL: https://yhoriuchi.github.io/projoint/
Depends: R (>= 4.1.0)
LazyData: false
BugReports: https://github.com/yhoriuchi/projoint/issues
NeedsCompilation: no
Packaged: 2026-08-28 19:00:57 UTC; yh25m
Repository: CRAN
Date/Publication: 2026-08-28 19:30:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(plot,projoint_results)
S3method(plot,projoint_tau)
S3method(print,projoint_data)
S3method(print,projoint_results)
S3method(print,projoint_tau)
S3method(summary,projoint_data)
S3method(summary,projoint_results)
S3method(summary,projoint_tau)
export(make_projoint_data)
export(predict_tau)
export(projoint)
export(read_Qualtrics)
export(read_labels)
export(reshape_projoint)
export(save_labels)
export(set_qoi)
import(dplyr)
import(estimatr)
import(forcats)
import(ggplot2)
import(ggthemes)
import(readr)
import(rlang)
import(stringr)
import(tidyr)
import(tidyselect)
importFrom(MASS,mvrnorm)
importFrom(ggplot2,"%+replace%")
importFrom(methods,is)
importFrom(methods,new)
importFrom(readr,parse_number)
importFrom(stats,cov)
importFrom(stats,sd)
importFrom(stats,setNames)

<div align="center">

<img src="man/figures/projoint.png" width="150" height="150" />

[![R-CMD-check](https://github.com/yhoriuchi/projoint/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/yhoriuchi/projoint/actions/workflows/R-CMD-check.yaml)
[![CRAN Status](https://www.r-pkg.org/badges/version/projoint)](https://CRAN.R-project.org/package=projoint)
[![Downloads](https://cranlogs.r-pkg.org/badges/grand-total/projoint)](https://cran.r-project.org/package=projoint)


### 🛠️ *Projoint*: The One-Stop Conjoint Shop

</div>

---

Conjoint survey designs are spreading across the social sciences due to their unusual capacity to estimate many quantities of interest from a single randomized experiment. **Projoint is general-purpose software for the design, implementation, and analysis of conjoint surveys.** Its easy-to-use components include a drag-and-drop web-based application for survey design and an R package for analysis.

We created this software for our <a href="https://gking.harvard.edu/conjointE" target="_blank" class="external-link"><em>AJPS</em> article</a> ("Correcting Measurement Error Bias in Conjoint Survey Experiments"), which offers easier and more powerful ways of designing and analyzing conjoint experiments -- including everything from writing Qualtrics survey code for you to preparing figures for your paper or report. It also includes easy measurement error corrections, which are essential because conjoint’s tendency to mirror complicated real-world choices generates substantial measurement error and, without corrections, considerable bias.

We intend for Projoint software to be the easiest and fastest way to design, field, and analyze conjoint surveys.<sup><a href="https://yhoriuchi.github.io/projoint/articles/faq.html#what-is-the-history-of-conjoint-analysis-what-is-the-difference-between-profile-level-and-choice-level-data" target="_blank" style="text-decoration: none;">&#9432;</a></sup> Take the 6 steps below and you should be all set.

---

## 🚀 Six Easy Steps

<!-- =========================
     1. Install
     ========================= -->

<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;"><b>1. Install</b> the software</summary>
Open R (or <a href="https://www.r-project.org/" target="_blank">install R</a> if you do not have it), and run the following command in your coding environment.
```r
install.packages("projoint")
```
</details>

<!-- =========================
     2. Design
     ========================= -->

<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;"><b>2. Design</b> your survey</summary>
Online surveys are frequently written with an online software called Qualtrics. Using our web tool, called the <a href="https://projoint.aaronrkaufman.com/" target="_blank" class="external-link">Projoint Survey Designer</a>, you don't need to learn how to write a survey in Qualtrics.  

* Use the <a href="https://projoint.aaronrkaufman.com/" target="_blank" class="external-link">Projoint Survey Designer</a> and export surveys formatted for Qualtrics.  
* Follow the <a href="https://yhoriuchi.github.io/projoint/articles/design.html" target="_blank">step-by-step guide</a> to learn how to set up your Qualtrics survey.  
</details>

<!-- =========================
     3. Field
     ========================= -->

<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;"><b>3. Field</b> your survey</summary>

* Using the .QSF file export from the Projoint Survey Designer, load your survey into Qualtrics.
  * Log into your Qualtrics account.
  * Click "Create a new project"
  * Under "From scratch" select "Survey" and then "Get started"
  * Enter a name and under "How do you want to start your survey" select "Import a QSF file"
  * Click "Choose file" and select your .QSF file.
  * Click "Create project"
* You are free to field your Qualtrics survey through online vendors.
* When you are done fielding your survey, you will now need to export your data from Qualtrics to R.
  * Click “Download Data”.
  * Choose CSV format.
  * Critically, select “Use choice text” rather than coded values.
</details>

<!-- =========================
     4. Read
     ========================= -->

<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;"><b>4. Read</b> the results into R and <b>wrangle</b> your data into structured form</summary>

* Load your <strong>survey responses</strong> into R:
```r
library(projoint)
dat <- read_Qualtrics("your_file.csv")
```

* Prepare the data for analysis:
```r
dat <- reshape_projoint(
  .dataframe = dat,
  .outcomes = c(paste0("choice", 1:8), "choice1_repeated_flipped"),
  .choice_map = c("Community A" = 1, "Community B" = 2)
)
```

The names in `.choice_map` should be the exact response values stored in the
outcome columns; the values `1` and `2` identify the corresponding Qualtrics
profile positions. Inspect the exported values with
`unique(unlist(dat[c("choice1", "choice2")]))`, then verify which response
denotes profile 1 and which denotes profile 2 against the survey instrument or
QSF file. Neither the CSV nor **projoint** can infer this mapping. Invalid,
ambiguous, or missing choices stop before estimation; missing choices can be
retained only with the explicit `.allow_missing_choices = TRUE` option.

* Follow the <a href="https://yhoriuchi.github.io/projoint/articles/read.html" target="_blank">step-by-step guide</a> to learn how to read and reshape data for conjoint analysis.
</details>

<!-- =========================
     5. Analyze
     ========================= -->

<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;"><b>5. Analyze</b> and <b>visualize</b> important <b>Quantities of Interest</b></summary>

* Estimate Marginal Means (MMs) or Average Marginal Component Effects (AMCEs) with correction for measurement error:

> **Note:** The following example illustrates a `profile_level` analysis.  This approach is common in social science and useful as an initial diagnostic, but we encourage researchers to consider `choice_level` analysis (the default for `.structure`). See <a href="https://yhoriuchi.github.io/projoint/articles/structure.html" target="_blank">Choice-Level Analysis</a>. Detailed steps of analysis appear in the <a href="https://yhoriuchi.github.io/projoint/articles/analyze.html" target="_blank">step-by-step guide</a>.

```r
output <- projoint(out1_arranged, .structure = "profile_level")
print(output)
summary(output)
```

* Visualize your results easily:
```r
plot(output)
```
* Estimate additional quantities of interest and explore subgroup comparisons using choice-level analysis.
* Follow the <a href="https://yhoriuchi.github.io/projoint/articles/analyze.html" target="_blank">step-by-step guide</a> to learn how to:
  * Estimate and correct marginal means (MMs) or average marginal component effects (AMCEs), including predicting IRR if necessary.
  * Visualize the marginal means (MMs) or average marginal component effects (AMCEs).

</details>

<!-- =========================
     6. Explore
     ========================= -->

<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;"><b>6. Explore</b> additional quantities and compare subgroups</summary>

* Go beyond standard profile-level summaries.  
* Use **choice-level analysis** to:
  - Directly compare trade-offs (e.g., low housing cost vs. low crime).  
  - Collapse multiple levels (e.g., city vs. suburban preferences).  
  - Estimate subgroup differences (e.g., Democrats vs. Republicans).  
* See detailed examples in the <a href="https://yhoriuchi.github.io/projoint/articles/explore.html" target="_blank">Explore and Compare Further</a> vignette.

</details>


---

## Cite projoint

Please cite **projoint** whenever you use it. Run `citation("projoint")` in R for ready-to-use text and BibTeX, or use the [package citation page](https://yhoriuchi.github.io/projoint/authors.html#citation). The canonical package record is available from [CRAN](https://CRAN.R-project.org/package=projoint) and its DOI is [10.32614/CRAN.package.projoint](https://doi.org/10.32614/CRAN.package.projoint).

If your analysis uses the package’s measurement-error correction method, please cite the accompanying [methodological article](https://gking.harvard.edu/conjointE) as well. GitHub’s **Cite this repository** panel uses the same package metadata from [CITATION.cff](https://github.com/yhoriuchi/projoint/blob/master/CITATION.cff).

---

## 📦 Additional Info

<details style="margin-left: 25px; margin-bottom: 5px">
    <summary>Upcoming Features</summary>
    <ul>
    <li>Weighted estimation for features and respondents</li>
    <li>Support for non-binary outcomes (ratings, rankings)</li>
    </ul>
</details>

<details style="margin-left: 25px; margin-bottom: 5px">
    <summary>Comments, questions, or suggestions?</summary>
    Please check issues already filed at our <a href="https://github.com/yhoriuchi/projoint/issues" target="_blank">GitHub repository</a>. If you don't see an answer to your question, click on the green "new issue" button.
</details>

---

<p align="center" style="font-size: 18px;">✨ Thank you for using projoint! ✨</p>

## ----setup, include=FALSE-----------------------------------------------------
library(projoint)
library(dplyr)
library(ggplot2)
library(patchwork)

## ----echo = FALSE-------------------------------------------------------------
data("exampleData1")
outcomes <- paste0("choice", 1:8)
outcomes <- c(outcomes, "choice1_repeated_flipped")
out1 <- reshape_projoint(exampleData1, outcomes)

## -----------------------------------------------------------------------------
mm0 <- projoint(out1, .structure = "profile_level", .estimand = "mm")
print(mm0)
summary(mm0)

## -----------------------------------------------------------------------------
qoi_1 <- set_qoi(
  .structure = "profile_level",
  .estimand = "mm",
  .att_choose = "att1",
  .lev_choose = "level1"
)

mm1 <- projoint(out1, .qoi = qoi_1)
print(mm1)
summary(mm1)

## -----------------------------------------------------------------------------
mm1b <- projoint(out1, .qoi = qoi_1, .irr = 0.75)
print(mm1b)
summary(mm1b)

## -----------------------------------------------------------------------------
amce0 <- projoint(out1, .structure = "profile_level", .estimand = "amce")
print(amce0)
summary(amce0)

## -----------------------------------------------------------------------------
qoi_3 <- set_qoi(
  .structure = "profile_level",
  .estimand = "amce",
  .att_choose = "att1",
  .lev_choose = "level3",
  .att_choose_b = "att1",
  .lev_choose_b = "level1"
)

amce1 <- projoint(out1, .qoi = qoi_3)
print(amce1)
summary(amce1)

## -----------------------------------------------------------------------------
amce1b <- projoint(out1, .qoi = qoi_3, .irr = 0.75)
print(amce1b)
summary(amce1b)

## -----------------------------------------------------------------------------
data("out1_arranged")
mm   <- projoint(out1_arranged, .structure = "profile_level") 
amce <- projoint(out1_arranged, .structure = "profile_level", .estimand = "amce")

## ----fig.width = 6, fig.height = 6--------------------------------------------
plot(mm)

## ----fig.width = 6, fig.height = 6--------------------------------------------
plot(amce)

## ----fig.height = 6.5, fig.width = 7------------------------------------------
outcomes <- c(paste0("choice", 1:8), "choice1_repeated_flipped")

df <- exampleData1 |> mutate(white = ifelse(race == "White", 1, 0))

df_0 <- df |> filter(white == 0) |> reshape_projoint(outcomes)
df_1 <- df |> filter(white == 1) |> reshape_projoint(outcomes)
df_d <- df |> reshape_projoint(outcomes, .covariates = "white")

data_file <- system.file("extdata", "labels_arranged.csv", package = "projoint")
if (data_file == "") stop("File not found!")

df_0 <- read_labels(df_0, data_file)
df_1 <- read_labels(df_1, data_file)
df_d <- read_labels(df_d, data_file)

out_0 <- projoint(df_0, .structure = "profile_level")
out_1 <- projoint(df_1, .structure = "profile_level")
out_d <- projoint(df_d, .structure = "profile_level", .by_var = "white")

plot_0 <- plot(out_0)
plot_1 <- plot(out_1)
plot_d <- plot(out_d, .by_var = TRUE)

plot_0 +
  coord_cartesian(xlim = c(0.2, 0.8)) +
  labs(title = "Non-white", x = "AMCE") +
  theme(plot.title = element_text(hjust = 0.5)) +

plot_1 +
  coord_cartesian(xlim = c(0.2, 0.8)) +
  labs(title = "White", x = "AMCE") +
  theme(axis.text.y = element_blank(), plot.title = element_text(hjust = 0.5)) +

plot_d +
  coord_cartesian(xlim = c(-0.4, 0.4)) +
  labs(title = "Difference", x = "Difference") +
  theme(axis.text.y = element_blank(), plot.title = element_text(hjust = 0.5))

## ----echo=FALSE, warning = F--------------------------------------------------
library(downloadthis)
download_link(
  link = "https://raw.githubusercontent.com/yhoriuchi/projoint/master/data-raw/CHKKK_Mummolo_and_Nall_full_replication_W1_Lucid_-_August_2021.qsf",
  button_label = "Download QSF file",
  button_type = "danger",
  has_icon = TRUE,
  icon = "fa fa-save",
  self_contained = FALSE
)

## ----setup, include=FALSE-----------------------------------------------------
library(projoint)

## ----warning=FALSE, message=FALSE---------------------------------------------
library(tidyverse)
library(projoint)

## ----eval=TRUE, echo=FALSE----------------------------------------------------
data <- read_Qualtrics(
  system.file("extdata", "mummolo_nall_replication.csv", package = "projoint")
)

## -----------------------------------------------------------------------------
# Inspect the imported data:
data

## ----fig-setup, include=FALSE-------------------------------------------------
# Global default settings for all figures
knitr::opts_chunk$set(
  fig.width = 7,
  fig.height = 5,
  fig.align = "center",
  dpi = 300  # Optional: high-resolution plots
)

# Helper functions for special figure sizes
narrow_fig <- function() list(fig.width = 5, fig.height = 4)
wide_fig <- function() list(fig.width = 8, fig.height = 5)
tall_fig <- function() list(fig.width = 6, fig.height = 7)

# Load libraries
library(projoint)
data(exampleData1, package = "projoint")
data(exampleData2, package = "projoint")
data(exampleData3, package = "projoint")
data(exampleData1_labelled_tibble, package = "projoint")
data(out1_arranged, package = "projoint")


## ----error=TRUE---------------------------------------------------------------
try({
outcomes <- paste0("choice", 1:8)
outcomes1 <- c(outcomes, "choice1_repeated_flipped")

out1 <- reshape_projoint(
  .dataframe = exampleData1,
  .outcomes = outcomes1,
  .choice_map = c("A" = 1, "B" = 2),
  .alphabet = "K",
  .idvar = "ResponseId",
  .repeated = TRUE,
  .flipped = TRUE
)
})

## -----------------------------------------------------------------------------
outcomes <- paste0("choice", 1:8)
outcomes2 <- c(outcomes, "choice1_repeated_notflipped")
out2 <- reshape_projoint(
  .dataframe = exampleData2,
  .outcomes = outcomes2,
  .repeated = TRUE,
  .flipped = FALSE
)

## -----------------------------------------------------------------------------
outcomes <- paste0("choice", 1:8)
out3 <- reshape_projoint(
  .dataframe = exampleData3,
  .outcomes = outcomes,
  .repeated = FALSE
)

## -----------------------------------------------------------------------------
fill_FALSE <- reshape_projoint(
  .dataframe = exampleData1,
  .outcomes = outcomes1,
  .fill = FALSE
)

fill_TRUE <- reshape_projoint(
  .dataframe = exampleData1,
  .outcomes = outcomes1,
  .fill = TRUE
)

## -----------------------------------------------------------------------------
selected_vars <- c("id", "task", "profile", "selected", "selected_repeated", "agree")
fill_FALSE$data[selected_vars]
fill_TRUE$data[selected_vars]

## -----------------------------------------------------------------------------
out4 <- make_projoint_data(
  .dataframe = exampleData1_labelled_tibble,
  .attribute_vars = c(
    "School Quality", "Violent Crime Rate (Vs National Rate)",
    "Racial Composition", "Housing Cost",
    "Presidential Vote (2020)", "Total Daily Driving Time for Commuting and Errands",
    "Type of Place"
  ),
  .id_var = "id",
  .task_var = "task",
  .profile_var = "profile",
  .selected_var = "selected",
  .selected_repeated_var = "selected_repeated",
  .fill = TRUE
)

## -----------------------------------------------------------------------------
out4

## ----eval=FALSE---------------------------------------------------------------
# save_labels(out1, "temp/labels_original.csv")

## ----eval=FALSE---------------------------------------------------------------
# out1_arranged <- read_labels(out1, "temp/labels_arranged.csv")

## ----eval=TRUE, eco=FALSE-----------------------------------------------------
data(out1_arranged, package = "projoint")

## -----------------------------------------------------------------------------
mm <- projoint(out1, .structure = "profile_level", .estimand = "mm")
plot(mm)

## -----------------------------------------------------------------------------
mm <- projoint(out1_arranged, .structure = "profile_level", .estimand = "mm")
plot(mm)

## ----setup, include=FALSE-----------------------------------------------------
library(projoint)
library(dplyr)
library(ggplot2)
library(patchwork)

## -----------------------------------------------------------------------------
projoint_data <- function(labels, data) {
  structure(
    list(labels = labels, data = data),
    class = "projoint_data"
  )
}

## -----------------------------------------------------------------------------
data("out1_arranged")
out1_arranged$labels

## ----message=FALSE, warning=FALSE---------------------------------------------
# 1) Data: keep only the two joint profiles of interest
data("out1_arranged")
d1 <- out1_arranged$data
d2 <- d1 |>
  mutate(y1 = case_when(
    # Low housing cost, high crime
    att1 == "att1:level1" & att6 == "att6:level2" ~ 1,
    TRUE ~ 0
  ),
  y0 = case_when(
    # High housing cost, low crime
    att1 == "att1:level3" & att6 == "att6:level1" ~ 1,
    TRUE ~ 0
  )) |>
  filter(y1 == 1 | y0 == 1)

# 2) Labels: rename only the two att1 levels to reflect the joint trade-offs
labels1 <- out1_arranged$labels
labels2 <- labels1 |> 
  mutate(level = case_when(level_id == "att1:level1" ~ "Housing Cost (Low)\nCrime Rate(High)", 
                           level_id == "att1:level3" ~ "Housing Cost (High)\nCrime Rate(Low)", 
                           TRUE ~ level_id))

## -----------------------------------------------------------------------------
d1 |> count(att1, att6)
d2 |> count(att1, att6) # only the two joint profiles remain
labels1 |> filter(attribute_id == "att1")
labels2 |> filter(attribute_id == "att1" & level_id %in% c("att1:level1", "att1:level3"))

## ----fig.width = 6, fig.height = 3--------------------------------------------
# 3) Build a new projoint_data object
pj_data_wrangled <- projoint_data("labels" = labels2, 
                                  "data" = d2)

# 4) Quantity of interest: Low vs High housing cost under the specified crime conditions (choice-level MM)
qoi <- set_qoi(
  .att_choose = "att1", 
  .lev_choose = "level1", # Low housing cost (with high crime in this subset)
  .att_notchoose = "att1", 
  .lev_notchoose = "level3" # High housing cost (with low crime in this subset)
)

# 5) Estimate and plot (horizontal layout)
out <- projoint(pj_data_wrangled, qoi)
plot(out)

## ----message=FALSE, warning=FALSE---------------------------------------------
# 1) Data: collapse levels for att7
d1 <- out1_arranged$data
d2 <- d1 |>
  mutate(
    att7 = case_when(
      att7 %in% c("att7:level1", "att7:level2") ~ "att7:level7",  # City
      att7 %in% c("att7:level5", "att7:level6") ~ "att7:level8",  # Suburban
      TRUE ~ att7
    )
  )

# 2) Labels: create matching level IDs and readable names
labels1 <- out1_arranged$labels
labels2 <- labels1 |>
  mutate(
    level_id = case_when(
      level_id %in% c("att7:level1", "att7:level2") ~ "att7:level7",
      level_id %in% c("att7:level5", "att7:level6") ~ "att7:level8",
      TRUE ~ level_id
    ),
    level = case_when(
      level_id == "att7:level7" ~ "City",
      level_id == "att7:level8" ~ "Suburban",
      TRUE ~ level
    )
  ) |>
  distinct()

## -----------------------------------------------------------------------------
d1 |> count(att7)
d2 |> count(att7)
labels1 |> filter(attribute_id == "att7")
labels2 |> filter(attribute_id == "att7")

## ----fig.width = 6, fig.height = 3--------------------------------------------
# 3) Build a new projoint_data object
pj_data_wrangled <- projoint_data("labels" = labels2, 
                                  "data" = d2)

# 4) Quantity of interest: City vs. Suburban (choice-level MM)
qoi <- set_qoi(
  .structure     = "choice_level",
  .att_choose    = "att7",
  .lev_choose    = "level7",  # City
  .att_notchoose = "att7",
  .lev_notchoose = "level8"   # Suburban
)

# 5) Estimate and plot (horizontal layout)
out <- projoint(pj_data_wrangled, qoi)
plot(out)

## ----fig.width = 6, fig.height = 3--------------------------------------------
data("exampleData1")

outcomes <- c(paste0("choice", 1:8), "choice1_repeated_flipped")

df_D <- exampleData1 |> filter(party_1 == "Democrat") |> reshape_projoint(outcomes)
df_R <- exampleData1 |> filter(party_1 == "Republican") |> reshape_projoint(outcomes)
df_0 <- exampleData1 |> filter(party_1 %in% c("Something else", "Independent")) |> reshape_projoint(outcomes)

qoi <- set_qoi(
  .structure = "choice_level",
  .estimand = "mm",
  .att_choose = "att2",
  .lev_choose = "level3",
  .att_notchoose = "att2",
  .lev_notchoose = "level1"
)

out_D <- projoint(df_D, qoi)
out_R <- projoint(df_R, qoi)
out_0 <- projoint(df_0, qoi)

out_merged <- bind_rows(
  out_D$estimates |> mutate(party = "Democrat"),
  out_R$estimates |> mutate(party = "Republican"),
  out_0$estimates |> mutate(party = "Independent")
) |> filter(estimand == "mm_corrected")

# Plot

ggplot(out_merged, aes(y = party, x = estimate)) +
  geom_vline(xintercept = 0.5, linetype = "dashed", color = "gray") +
  geom_pointrange(aes(xmin = conf.low, xmax = conf.high)) +
  geom_text(aes(label = format(round(estimate, 2), nsmall = 2)), vjust = -1) +
  labs(y = NULL, x = "Choice-level Marginal Mean",
       title = "Preference for Democratic-majority areas") +
  theme_classic()

#' Calculates the intra-respondent reliability (IRR), average marginal component effect (AMCE), marginal mean (MM), and correcte AMCEs and MMs for a given conjoint data set, as well as bootstrapped confidence intervals.
#'
#' @param .data A conjoint data set.
#' @return A summarized data set describing the results of a conjoint analysis, corrected for IRR measurement error.




pj_bs <- function(.data, .id = "id", .n_boot = 500, .diff = NULL){
  
  # Unique IDs
  ids <- .data %>%
    dplyr::select(dplyr::all_of(.id)) %>%
    dplyr::distinct() %>%
    pull()
  
  # Bootstrapped IDs
  bs_ids <- sample(x = ids, size = length(ids) * .n_boot, replace = TRUE)
  
  # Bootstrapped sample IDs
  sample <- rep(1:.n_boot, length(ids))
  
  # Bootstrapped data frames
  bs_data <- data.frame(sample = sample,
                        id = bs_ids) %>%
    dplyr::left_join(.data, by = "id")
  
  # Return
  purrr::map_dfr(1:.n_boot, pj_bs_sample, bs_data, .diff)
  
}

pj_merge <- function(.estimates_withoutBS, .estimates_withBS, .pvalue = 0.05){
  
  .lower <- .pvalue / 2
  .upper <- (1 - .lower)
  
  tau <- NULL
  mm <- NULL
  mm_corrected <- NULL
  amce <- NULL
  amce_corrected <- NULL
  att_level <- NULL
  
  out1 <- .estimates_withoutBS %>% 
    tidyr::pivot_longer(cols = c(tau, mm, mm_corrected, amce, amce_corrected), 
                        names_to = "estimand",
                        values_to = "mean") 
  
  out2 <- .estimates_withBS %>% 
    tidyr::pivot_longer(cols = c(tau, mm, mm_corrected, amce, amce_corrected), 
                        names_to = "estimand",
                        values_to = "value") %>% 
    dplyr::group_by(att_level, estimand) %>% 
    dplyr::summarise(bs_mean = mean(value, na.rm = TRUE),
                     se = sd(value, na.rm = TRUE),
                     lower = stats::quantile(value, .lower, na.rm = TRUE),
                     upper = stats::quantile(value, .upper, na.rm = TRUE), .groups = "drop") %>%
    dplyr::ungroup() %>% 
    dplyr::mutate(bs_mean = ifelse(is.nan(bs_mean), NA, bs_mean))
  
  dplyr::left_join(out1, out2, 
                   by = c("att_level", "estimand"))
  
}

#' Calculates the intra-respondent reliability (IRR), average marginal component effect (AMCE), marginal mean (MM), and correcte AMCEs and MMs for a given conjoint data set.
#'
#' @param .data A conjoint data set.
#' @param .tau An optional numeric argument allowing the researcher to specify their own value of \texttt{tau}
#' @return A summarized data set describing the results of a conjoint analysis, corrected for IRR measurement error.
#' @export
#' @examples
#' library(projoint)
#' 
#' data("exampleData1")
#' head(exampleData1)
#' reshaped_data = reshape_conjoint(exampleData1, .idvar = "ResponseId",
#'                                  .outcomes = c("Q4.1", "Q5.1", "Q6.1", "Q7.1", "Q8.1","Q9.1"),
#'                                  .alphabet ="A")
#' results = pj(reshaped_data)
#'
#' library(projoint)
#' 
#' data("exampleData2")
#' head(exampleData2)
#' reshaped_data = reshape_conjoint(exampleData1, .idvar = "ResponseId",
#'                                  .outcomes = c("Q4.1", "Q5.1", "Q6.1", "Q7.1", "Q8.1","Q9.1"),
#'                                  .alphabet ="A")
#' results = pj(reshaped_data, .tau = 0.75)



pj <- function(.data, .tau = NULL){
  
  att_level <- NULL

  if(is.NULL(.tau)){  
    .tau <- calculate_tau(.data)
  }else if(!is.NULL(.tau)){
    if(.tau >= 1 | .tau <= .5){
      stop(".tau must be between 0.5 and 1, not inclusive")
    }
  }

  mm_estimates <- calculate_mm(.data) %>% 
    tidyr::separate(att_level, sep = ":", into = c("attribute", "level"))
  
  mm_baselines <- mm_estimates %>% 
    dplyr::filter(str_detect(level, "level1$")) %>% 
    dplyr::select(-level) %>% 
    dplyr::rename("baseline" = mm)
  
  mm_estimates %>% 
    dplyr::left_join(mm_baselines, by = "attribute") %>% 
    dplyr::mutate(tau = .tau,
                  mm_corrected = (mm - tau) / (1 - 2 * tau),
                  amce = ifelse(level != "level1", mm - baseline, NA),
                  amce_corrected = amce / (1 - 2 * tau),
                  att_level = str_c(attribute, ":", level)) %>% 
    dplyr::select(tau, att_level, mm, mm_corrected, amce, amce_corrected) 
  
}

#' Print a projoint_data object
#'
#' Custom print method for objects of class \code{projoint_data}.
#'
#' @param x A \code{projoint_data} object.
#' @param ... Additional arguments (currently unused).
#'
#' @return No return value, called for its side effect of printing a summary of
#'   the \code{projoint_data} object to the console.
#'
#' @export
#'
#' @examples
#' \donttest{
#' data(exampleData1)
#' dat <- reshape_projoint(
#'   exampleData1,
#'   .outcomes = c("choice1", "choice2")
#' )
#' print(dat)
#' }
print.projoint_data <- function(x, ...) {
  cat("<projoint_data>\n")
  
  if (!is.null(x$data)) {
    cat("- data:    ", nrow(x$data), "rows,", ncol(x$data), "columns\n")
  } else {
    cat("- data:    NULL\n")
  }
  
  if (!is.null(x$labels)) {
    cat("- labels:  ", nrow(x$labels), "levels,", ncol(x$labels), "columns\n")
  } else {
    cat("- labels:  NULL\n")
  }
  
  invisible(x)
}

#' Create a projoint_data Object
#'
#' Internal constructor for \code{projoint_data} objects.  
#' Used by \code{\link{reshape_projoint}} and \code{\link{make_projoint_data}} to
#' bundle conjoint survey labels and response data into a consistent structure.
#'
#' @param labels A data frame of conjoint attribute–level metadata. Must include
#'   attribute names, attribute IDs (e.g., \code{"att1"}), and level IDs
#'   (e.g., \code{"att1:lev1"}).
#' @param data A data frame (typically a tibble) containing the reshaped conjoint
#'   survey responses, one row per respondent–task–profile, with attribute columns,
#'   \code{selected}, \code{agree}, and any covariates.
#'
#' @return A list of length two with class \code{"projoint_data"}, containing:
#'   \itemize{
#'     \item \code{$labels}: attribute–level mapping
#'     \item \code{$data}: respondent–task–profile dataset
#'   }
#'
#' @keywords internal
projoint_data <- function(labels, data) {
  structure(
    list(
      labels = labels,
      data = data
    ),
    class = "projoint_data"
  )
}

#' Set the quantities of interest (QoIs)
#'
#' Constructs a quantities-of-interest (QoI) specification for \pkg{projoint}.
#' Use this to request specific estimands—marginal means (MMs) or average
#' marginal component effects (AMCEs)—at either the choice- or profile-level,
#' and to declare which attribute levels are compared (including baselines).
#'
#' @param .structure Either \code{"choice_level"} (default) or \code{"profile_level"}.
#' @param .estimand Either \code{"mm"} for marginal means or \code{"amce"} for
#'   average marginal component effects.
#' @param .att_choose Character scalar: the attribute (column) for the level(s)
#'   that are \emph{chosen}.
#' @param .lev_choose Character vector: the level id(s) for the \emph{chosen}
#'   side. Length 1 for profile-level, \eqn{\ge}1 for choice-level.
#' @param .att_notchoose Character scalar: the attribute (column) for the level(s)
#'   that are \emph{not chosen}. Only used for \code{.structure == "choice_level"}.
#' @param .lev_notchoose Character vector: the level id(s) for the \emph{not chosen}
#'   side. Length 1 for profile-level, \eqn{\ge}1 for choice-level. Only used for
#'   \code{.structure == "choice_level"}.
#' @param .att_choose_b Character scalar: \emph{baseline} attribute for the
#'   chosen side when computing AMCEs.
#' @param .lev_choose_b Character vector: \emph{baseline} level id(s) for the
#'   chosen side when computing AMCEs. Length 1 for profile-level, \eqn{\ge}1 for
#'   choice-level.
#' @param .att_notchoose_b Character scalar: \emph{baseline} attribute for the
#'   not-chosen side (choice-level only) when computing AMCEs.
#' @param .lev_notchoose_b Character vector: \emph{baseline} level id(s) for the
#'   not-chosen side (choice-level only) when computing AMCEs.
#'
#' @return A \code{projoint_qoi} object (list-like) containing fields such as:
#'   \code{structure}, \code{estimand}, \code{attribute_of_interest},
#'   \code{levels_of_interest}, and their baseline counterparts. This object
#'   can be supplied to downstream estimation helpers that accept a QoI spec.
#'
#' @export
#'
#' @examples
#' # Specify a simple choice-level MM comparison for att1 levels:
#' q_mm <- set_qoi(
#'   .structure  = "choice_level",
#'   .estimand   = "mm",
#'   .att_choose = "att1",
#'   .lev_choose = c("att1:lev2"),
#'   .att_notchoose = "att1",
#'   .lev_notchoose = c("att1:lev1")
#' )
#' str(q_mm)
#'
#' # Example AMCE with explicit baselines (profile-level):
#' q_amce <- set_qoi(
#'   .structure   = "profile_level",
#'   .estimand    = "amce",
#'   .att_choose  = "att2",
#'   .lev_choose  = "att2:lev3",
#'   .att_choose_b = "att2",
#'   .lev_choose_b = "att2:lev1"
#' )
#' str(q_amce)
set_qoi <- function(
    .structure = "choice_level",
    .estimand = "mm",
    .att_choose, 
    .lev_choose,
    .att_notchoose = NULL, 
    .lev_notchoose = NULL,
    .att_choose_b = NULL, 
    .lev_choose_b = NULL,
    .att_notchoose_b = NULL, 
    .lev_notchoose_b = NULL
){
  
  
  # check -------------------------------------------------------------------
  
  structure  <- rlang::arg_match0(.structure, c("choice_level", "profile_level"))
  estimand  <- rlang::arg_match0(.estimand, c("mm", "amce"))
  
  # return ------------------------------------------------------------------
  
  projoint_qoi("structure" = structure,
               "estimand" = estimand,
               "attribute_of_interest" = .att_choose,
               "levels_of_interest" = .lev_choose,
               "attribute_of_interest_0" = .att_notchoose,
               "levels_of_interest_0" = .lev_notchoose,
               "attribute_of_interest_baseline" = .att_choose_b,
               "levels_of_interest_baseline" = .lev_choose_b,
               "attribute_of_interest_0_baseline" = .att_notchoose_b,
               "levels_of_interest_0_baseline" = .lev_notchoose_b)
  
}

set_qoi <- function(
    .structure = "choice_level",
    .estimand = "mm",
    .att_choose, 
    .lev_choose,
    .att_notchoose = NULL, 
    .lev_notchoose = NULL,
    .att_choose_b = NULL, 
    .lev_choose_b = NULL,
    .att_notchoose_b = NULL, 
    .lev_notchoose_b = NULL
){
  
  
  # check -------------------------------------------------------------------
  
  structure  <- rlang::arg_match0(.structure, c("choice_level", "profile_level"))
  estimand  <- rlang::arg_match0(.estimand, c("mm", "amce"))
  
  # return ------------------------------------------------------------------
  
  projoint_qoi("structure" = structure,
               "estimand" = estimand,
               "attribute_of_interest" = .att_choose,
               "levels_of_interest" = .lev_choose,
               "attribute_of_interest_0" = .att_notchoose,
               "levels_of_interest_0" = .lev_notchoose,
               "attribute_of_interest_baseline" = .att_choose_b,
               "levels_of_interest_baseline" = .lev_choose_b,
               "attribute_of_interest_0_baseline" = .att_notchoose_b,
               "levels_of_interest_0_baseline" = .lev_notchoose_b)
  
}

test_that(".weights_2 works for choice-level MMs (same-attribute and cross-attribute QOIs)", {
  # Load package data
  data("exampleData1", package = "projoint", envir = environment())
  expect_true(exists("exampleData1"))

  # Add a synthetic covariate to be used as weights
  set.seed(123)
  exampleData1$weight <- stats::runif(nrow(exampleData1))

  # Outcomes as used in the reported issue
  outcomes <- c(paste0("choice", 1:8), "choice1_repeated_flipped")

  out1 <- reshape_projoint(
    exampleData1,
    outcomes,
    .covariates = "weight"
  )

  # Sanity: weight must survive reshape into out1$data
  expect_true("weight" %in% names(out1$data))

  # ------------------------------------------------------------
  # 1) Same-attribute QOI (att1 level3 vs att1 level1)
  # ------------------------------------------------------------
  qoi_same <- set_qoi(
    .structure     = "choice_level",
    .att_choose    = "att1",
    .lev_choose    = "level3",
    .att_notchoose = "att1",
    .lev_notchoose = "level1"
  )

  expect_error(
    projoint(out1, qoi_same, .weights_2 = weight),
    NA
  )

  expect_error(
    projoint(out1, qoi_same, .weights_1 = weight),
    NA
  )

  expect_error(
    projoint(out1, qoi_same, .weights_1 = weight, .weights_2 = weight),
    NA
  )

  # ------------------------------------------------------------
  # 2) Cross-attribute QOI (att1 level1 vs att3 level1)
  # ------------------------------------------------------------
  qoi_cross <- set_qoi(
    .structure     = "choice_level",
    .att_choose    = "att1",
    .lev_choose    = "level1",
    .att_notchoose = "att3",
    .lev_notchoose = "level1"
  )

  expect_error(
    projoint(out1, qoi_cross, .weights_2 = weight),
    NA
  )
})

#' Predicts \texttt{tau} from a conjoint data set by extrapolation from similar but non-identical tasks.
#'
#' @param .data A conjoint data set.
#' @return An tibble with estimated \texttt{tau} with confidence interval.
#' @export
#' @examples
#' library(projoint)
#' 
#' 
#' data("exampleData2")
#' head(exampleData2)
#' reshaped_data = reshape_conjoint(exampleData1, .idvar = "ResponseId",
#'                                  .outcomes = c("Q4.1", "Q5.1", "Q6.1", "Q7.1", "Q8.1","Q9.1"),
#'                                  .alphabet ="A")
#' tau_est = predict_tau(reshaped_data)


predict_tau <- function(.data){
  
  library(tidyverse)
  
  id <- NULL
  task <- profile <- NULL
  selected <- selected_repeated <- NULL
  
  # The number of attributes
  
  n_attributes <- names(.data) %>% str_detect(., "att") %>% sum()
  
  # All combinations of tasks (within respondents) 
  
  task_combinations <- .data %>% 
    select(id, "task1" = task, "task2" = task) %>% 
    distinct() %>% 
    group_by(id) %>% 
    complete(task1, task2) %>% 
    ungroup() %>% 
    filter(task1 != task2) %>% 
    mutate(t1 = ifelse(task1 < task2, task2, task1),
           t2 = ifelse(task1 > task2, task2, task1)) %>% 
    select(-task1, -task2) %>% 
    distinct()
  
  # Initial cleaning: add y (a profile chosen)
  
  d <- .data %>% 
    mutate(y = case_when(selected == 1 & profile == 1 ~ 1,
                         selected == 1 & profile == 2 ~ 2,
                         selected == 0 & profile == 1 ~ 2,
                         selected == 0 & profile == 2 ~ 1)) 
  
  # Cleaning for each attribute
  
  d2 <- d %>% 
    select(id, task, y) %>% 
    distinct()
  
  for (i in 1:n_attributes){
    
    att_quo <- rlang::as_name(str_c("att", i))
    var1 <- str_c("att", i, "_comb") %>% rlang::quo_name()
    var2 <- str_c("att", i, "_same") %>% rlang::quo_name()
    
    temp <- d %>% 
      select(id, task, profile, "att" = !!att_quo) %>% 
      group_by(id, task) %>% 
      pivot_wider(names_from = profile, values_from = att) %>% 
      ungroup() %>% 
      mutate(att_comb = str_c(`1`, ", ", `2`)) %>% 
      select(id, task,
             !!var1 := att_comb) 
    
    d2 <- d2 %>% 
      left_join(temp, by = c("id", "task"))
    
  }
  
  task1 <- d2 %>% set_names(str_c("task1_", names(d2)))
  task2 <- d2 %>% set_names(str_c("task2_", names(d2)))
  
  d3 <- task_combinations %>% 
    left_join(task1, by = c("id" = "task1_id", "t1" = "task1_task")) %>% 
    left_join(task2, by = c("id" = "task2_id", "t2" = "task2_task")) %>% 
    mutate(y_same = ifelse(task1_y == task2_y, 1, 0))
  
  for (i in 1:n_attributes){
    
    var1 <- sym(str_c("task1_att", i, "_comb"))
    var2 <- sym(str_c("task2_att", i, "_comb"))
    var3 <- str_c("att", i, "_comb_same") %>% rlang::quo_name()
    
    d3 <- d3 %>% 
      # Logical: whether two level-combinations are the same between tasks
      mutate(!!var3 := ifelse(!!var1 == !!var2, 0, 1)) 
    
  }
  
  y <- d3 %>% 
    select(id, t1, t2, y_same) %>% 
    distinct()
  
  x <- d3 %>% 
    select(id, t1, t2, contains("comb_same")) %>% 
    pivot_longer(cols = 4:ncol(.)) %>% 
    group_by(id, t1, t2) %>% 
    summarise(x = sum(value), .groups = "drop")
  
  out <- y %>% 
    left_join(x, by = c("id", "t1", "t2"))
  
  # Calculate the percentage agreement by the number of attributes with levels that differ between two tasks
  pct_agreement <- out %>% 
    group_by(x) %>% 
    summarise(tidy(estimatr::lm_robust(y_same ~ 1, data = cur_data(), clusters = id)), .groups = "drop") %>% 
    mutate(n_attributes = n_attributes)
  
  # Finally, predict tau
  pct_agreement %>% 
    filter(!(estimate %in% c(0, 1))) %>% 
    summarise(tidy(estimatr::lm_robust(estimate ~ x, data = ., weights = 1/std.error^2), se_type = "classical")) %>% 
    filter(term == "(Intercept)") %>% 
    select(estimate, conf.low, conf.high) 
​
}

#' Calculate marginal means given a data set
#'
#' @param .data A conjoint data set.
#' @return A summarized data set of marginal means for each attribute-level



calculate_mm <- function(.data){

  selected <- NULL
  id <- NULL
  att_level <- NULL

  .data %>%
    dplyr::filter(!is.na(selected)) %>%
    dplyr::select(id, selected, dplyr::contains("att")) %>%
    tidyr::pivot_longer(3:ncol(.), names_to = "attribute", values_to = "att_level") %>%
    dplyr::group_by(att_level) %>%
    dplyr::summarize(mm = mean(selected), .groups = "drop") %>%
    dplyr::ungroup()

}

#' Projoint Example Data Set 2: Building Conjoint with a Repeated, Non-Flipped Task
#'
#' A cleaned Qualtrics export from a conjoint study that compares two potential
#' new building developments. Each respondent completed 8 standard tasks as well
#' as a repeated version of the first task, where the profiles were presented in
#' the same left-right positions (not flipped).
#'
#' @docType data
#' @usage data(exampleData2)
#'
#' @format A data frame with 400 rows and 185 columns. Contains survey responses
#'   including demographic information, outcome choices, and conjoint attribute
#'   values identified by \code{K-*-*} variable names.
#'
#' @keywords datasets
#'
#' @examples
#' # Load the dataset
#' data(exampleData2)
#'
#' # Inspect the first few rows
#' head(exampleData2)
#'
#' # Number of rows and columns
#' dim(exampleData2)
#'
#' # Display first 10 column names
#' names(exampleData2)[1:10]
#'
#' @name exampleData2
NULL

#' Calculates the intra-respondent reliability (IRR), average marginal component effect (AMCE), marginal mean (MM), and correcte AMCEs and MMs for a given conjoint data set specified across a given subgroup.
#'
#' @param .data A conjoint data set.
#' @param .diff A dichotomous variable to calculate difference IRR, AMCEs, and MMs across.
#' @param .tau An optional numeric argument allowing the researcher to specify their own value of \texttt{tau}
#' @return A summarized data set describing the results of a conjoint analysis subgroup comparison, corrected for IRR measurement error.
#' @export
#' @examples
#' library(projoint)
#' 
#' data("exampleData1")
#' head(exampleData1)
#' reshaped_data = reshape_conjoint(exampleData1, .idvar = "ResponseId",
#'                                  .outcomes = c("Q4.1", "Q5.1", "Q6.1", "Q7.1", "Q8.1","Q9.1"),
#'                                  .alphabet ="A")
#' reshaped_data$age2 = cut(reshaped_data$age, 2) # Dichotomize age
#' results = pj_diff(reshaped_data, .diff = "age2")



# In the current version, .group_by must be a dichotomous variable with {1, 2}
pj_diff <- function(.data, .diff, .tau = NULL){
  
  data <- .data %>% 
    rename("group_by_var" = !!rlang::sym(.diff)) 
  
  group1 <- data %>% filter(group_by_var == 1 & !is.na(group_by_var))
  group2 <- data %>% filter(group_by_var == 2 & !is.na(group_by_var))
  

  if(is.NULL(.tau)){
    out1 <- pj(group1)
    out2 <- pj(group2)
  }else if(is.numeric(.tau)){
    if(.tau < 1 | .tau > .5){
      out1 <- pj(group1, .tau = .tau)
      out2 <- pj(group2, .tau = .tau)
    }else{
      stop("tau must be between 0.5 and 1, not inclusive")
    }
  }else{
    stop("tau must be numeric between 0.5 and 1, not inclusive")
  }
  
  names(out1) <- str_c("group1:", names(out1))
  names(out2) <- str_c("group2:", names(out2))
  
  out1b <- out1 %>% rename("att_level" = `group1:att_level`)
  out2b <- out2 %>% rename("att_level" = `group2:att_level`)
  
  full_join(out1b, out2b, by = "att_level") %>% 
    mutate(tau = `group2:tau` - `group1:tau`,
           mm = `group2:mm` - `group1:mm`,
           mm_corrected = `group2:mm_corrected` - `group1:mm_corrected`,
           amce = `group2:amce` - `group1:amce`,
           amce_corrected = `group2:amce_corrected` - `group1:amce_corrected`) %>% 
    select(tau, att_level, mm, mm_corrected, amce, amce_corrected)

}

# Used only within pj_bs()
pj_bs_sample <- function(.sample, .bs_data, .diff = NULL){
  
  sample <- NULL
  
  d <- .bs_data %>% 
    dplyr::filter(sample == .sample)
  
  if (is.null(.diff)){
    
    pj(d) %>% 
      dplyr::mutate(sample = .sample)
    
  } else{
    
    pj_diff(d, .diff) %>% 
      dplyr::mutate(sample = .sample)
    
    
  }
  
  
}

#' Projoint Example Data Set 1: "Labelled Tibble"
#'
#' A cleaned tibble where each attribute corresponds to a separate column with a
#' descriptive attribute name. The unit of observation is each of two profiles in
#' each task for each respondent.
#'
#' @format A tibble with 6,400 rows and 14 columns. Contains survey responses
#'   including outcome choices and conjoint attribute values (columns typically
#'   named with a `K-*-*` convention).
#'
#' @source Qualtrics survey on Prolific; see Clayton et al. replication materials.
#'
#' @usage data(exampleData1_labelled_tibble)
#'
#' @details This dataset is intended for illustrating reading, reshaping, and
#'   analysis workflows in \pkg{projoint}. Column names are compatible with
#'   \code{reshape_projoint()}.
#'
#' @examples
#' # Load the data
#' data(exampleData1_labelled_tibble)
#'
#' # Basic inspection (fast and always runnable)
#' head(exampleData1_labelled_tibble)
#' dim(exampleData1_labelled_tibble)
#'
#' # Optional: quick structure peek (names only)
#' names(exampleData1_labelled_tibble)
#'
#' @name exampleData1_labelled_tibble
#' @docType data
#' @keywords datasets
NULL

#' Read and apply a reordered attribute/level mapping
#'
#' Reads a CSV containing a revised ordering of attributes and levels and applies it
#' to an existing \code{\link{projoint_data}} object. Typical workflow: first save the
#' current labels to CSV (e.g., with \code{\link{save_labels}}), manually reorder rows
#' (and/or the attribute grouping) in the CSV, then call \code{read_labels()} to apply.
#'
#' @param .data A \code{\link{projoint_data}} object whose labels/data should be reordered.
#' @param .filename Path to the revised labels CSV (originally produced from the package's labels).
#'
#' @return A \code{\link{projoint_data}} object with the same content as \code{.data} but with
#'   attributes and levels reordered to match the CSV. The returned object contains:
#'   \itemize{
#'     \item \code{$labels}: a tibble with new \code{attribute_id} and \code{level_id} reflecting the chosen order
#'     \item \code{$data}: a tibble whose \code{att*} columns have been remapped to the new \code{level_id}s
#'   }
#'
#' @seealso \code{\link{save_labels}}, \code{\link{reshape_projoint}}
#'
#' @import readr
#' @import dplyr
#' @import forcats
#' @import tidyr
#' @import stringr
#' @import tidyselect
#'
#' @examples
#' \donttest{
#' # Create a projoint_data object from the example dataset
#' data(exampleData1)
#' outcomes <- c(paste0("choice", 1:8), "choice1_repeated_flipped")
#' pj <- reshape_projoint(exampleData1, outcomes)
#'
#' # Write current labels to a temporary CSV, adding an 'order' column
#' tmp <- tempfile(fileext = ".csv")
#' pj$labels |>
#'   dplyr::mutate(order = dplyr::row_number()) |>
#'   readr::write_csv(tmp)
#'
#' # (User would reorder rows in 'tmp' manually; we just read it back)
#' pj_reordered <- read_labels(pj, tmp)
#'
#' # Inspect the updated label order
#' head(pj_reordered$labels)
#' }
#'
#' @export
read_labels <- function(
    .data,
    .filename
){
  
  labels_arranged <- readr::read_csv(.filename, 
                                     show_col_types = FALSE) |> 
    dplyr::mutate(attribute_id = stringr::str_extract(level_id, "^.+(?=\\:)"),
                  attribute_id_arranged = forcats::fct_reorder(attribute_id, order),
                  attribute_id_arranged = stringr::str_c("att", as.numeric(attribute_id_arranged))) |> 
    dplyr::arrange(order) |> 
    dplyr::group_by(attribute_id_arranged) |>
    dplyr::mutate(level_id_arranged = dplyr::row_number(),
                  level_id_arranged = stringr::str_c(attribute_id_arranged, ":level", level_id_arranged)) |>
    dplyr::ungroup() 
  
  data_arranged <- .data$data |> 
    tidyr::pivot_longer(cols = tidyselect::contains("att")) |> 
    dplyr::mutate(value = as.character(value)) |> 
    dplyr::left_join(labels_arranged, 
                     by = c("value" = "level_id")) |> 
    dplyr::select(-name, -value, -attribute, -attribute_id, -level, -order) |> 
    tidyr::pivot_wider(names_from = attribute_id_arranged, 
                       values_from = level_id_arranged)
  
  labels_arranged <- labels_arranged |> 
    dplyr::select(-order, -attribute_id, -level_id) |> 
    dplyr::rename(attribute_id = attribute_id_arranged,
                  level_id = level_id_arranged)
  
  # return new projoint_data object
  out <- projoint_data("labels" = labels_arranged, 
                       "data" = data_arranged)
  
  return(out)
  
}

#' Plot choice-level marginal means (MMs) (helper)
#'
#' Internal helper used by \code{\link{plot.projoint_results}} to render
#' choice-level marginal means (MMs). Supports a bar chart or a horizontal
#' pointrange layout and optional custom level labels.
#'
#' @keywords internal
#'
#' @param x A \code{\link{projoint_results}} object produced by
#'   \code{\link{projoint}} with \code{structure = "choice_level"} and
#'   \code{estimand = "mm"}.
#' @param .type Character. Either \code{"bar"} (two bars with CIs) or
#'   \code{"pointrange"} (horizontal estimate with CI and level labels at the
#'   extremes). Default \code{"pointrange"}.
#' @param .estimates Character. Which estimate version to plot:
#'   \code{"corrected"} (default) or \code{"uncorrected"}.
#' @param .labels Optional character vector of length 2 for custom level labels
#'   (left/right). If \code{NULL}, labels are derived from \code{x$labels}.
#' @param .show_attribute Logical; if \code{TRUE} (default), add the attribute
#'   name as the title when both levels are from the same attribute.
#' @param .remove_xaxis Logical; if \code{TRUE}, remove x-axis line, ticks, and
#'   labels (useful when embedding). Default \code{FALSE}.
#' @param .xlim Numeric length-2 giving the x-axis limits. Default \code{c(0, 1)}.
#' @param .plot.margin Numeric vector of plot margins in cm,
#'   \code{c(top, left, bottom, right)}. Default \code{c(1, 2, 1, 2)}.
#' @param ... Currently unused (reserved for future extensions).
#'
#' @details
#' This helper expects that the \code{projoint_results} object \emph{already}
#' contains a single pair of choice-level MMs (i.e., one \code{att_level_choose}
#' and one \code{att_level_notchoose}). It is called internally by
#' \code{\link{plot.projoint_results}} when \code{x$structure == "choice_level"}
#' and \code{x$estimand == "mm"}.
#'
#' When \code{.type = "pointrange"}, the level labels are placed just outside
#' \code{.xlim} to avoid overlap with the confidence interval. If the two levels
#' correspond to different attributes, the attribute title is omitted and a
#' message is emitted.
#'
#' @return A \code{ggplot2} object.
#'
#' @seealso \code{\link{plot.projoint_results}} for the user-facing plot method.
#'
#' @examples
#' data(exampleData1)
#' dat <- reshape_projoint(exampleData1,
#'   .outcomes = c("choice1", "choice2", "choice1_repeated_flipped")
#' )
#' att <- unique(dat$labels$attribute_id)[1]
#' levs <- subset(dat$labels, attribute_id == att)$level_id
#' lev_names <- sub(".*:", "", levs)
#' q <- set_qoi("choice_level", "mm",
#'   .att_choose = att, .lev_choose = lev_names[2],
#'   .att_notchoose = att, .lev_notchoose = lev_names[1]
#' )
#' fit <- projoint(dat, .qoi = q)
#' plot(fit)
plot_projoint_choice_level_mm <- function(
    x,
    .type = "pointrange",
    .estimates = "corrected",
    .labels = NULL,
    .show_attribute = TRUE, 
    .remove_xaxis = FALSE, 
    .xlim = c(0, 1),
    .plot.margin = c(top = 1, left = 2, bottom = 1, right = 2),
    ...
) {
  if (!inherits(x, "projoint_results")) {
    stop("Input `x` must be a `projoint_results` object.")
  }
  
  .type <- match.arg(.type, choices = c("bar", "pointrange"))
  .estimates <- match.arg(.estimates, choices = c("corrected", "uncorrected"))
  
  # Attribute label logic
  label_att1 <- stringr::str_extract(x$estimates$att_level_choose, "att\\d+")[1]
  label_att0 <- stringr::str_extract(x$estimates$att_level_notchoose, "att\\d+")[1]
  
  if (label_att1 != label_att0) {
    message("The attributes are different between the two levels. Check your setting carefully. The attribute label is not added to the figure.")
    label_att <- NULL
  } else {
    label_att <- data.frame(attribute_id = label_att1) |>
      dplyr::left_join(x$labels, by = "attribute_id") |>
      dplyr::distinct(attribute) |>
      dplyr::pull()
  }
  
  estimand <- ifelse(x$estimand == "mm", "Marginal Mean", "Average Marginal Component Effect")
  
  # Resolve labels
  if (!is.null(.labels)) {
    if (length(.labels) != 2L) stop("`.labels` must be a character vector of length 2.")
  } else {
    labels_df <- x$labels
    label0 <- labels_df$level[match(x$estimates$att_level_notchoose[1], labels_df$level_id)]
    label1 <- labels_df$level[match(x$estimates$att_level_choose[1],    labels_df$level_id)]
    .labels <- c(label0, label1)
  }
  
  label0 <- .labels[1]
  label1 <- .labels[2]
  
  if (.type == "bar") {
    suffix <- if (.estimates == "corrected") "_corrected" else "_uncorrected"
    
    output_reshaped <- dplyr::bind_rows(
      x$estimates |>
        dplyr::select(estimand, estimate, se, conf.low, conf.high, "att_level" = att_level_choose),
      x$estimates |>
        dplyr::select(estimand, estimate, se, conf.low, conf.high, "att_level" = att_level_notchoose) |>
        dplyr::mutate(estimate = 1 - estimate,
                      conf.low = 1 - conf.low,
                      conf.high = 1 - conf.high)
    ) |>
      dplyr::filter(stringr::str_detect(estimand, suffix)) |>
      dplyr::left_join(x$labels, by = c("att_level" = "level_id"))
    
    if (!is.null(.labels)) {
      output_reshaped <- output_reshaped |>
        dplyr::mutate(att_level = factor(level, levels = .labels))
    }
    
    g <- ggplot2::ggplot(output_reshaped) +
      ggthemes::theme_few() +
      ggplot2::theme(plot.margin = grid::unit(.plot.margin, "cm")) +
      ggplot2::geom_col(ggplot2::aes(x = att_level, y = estimate), width = 0.5, fill = "gray") +
      ggplot2::geom_errorbar(ggplot2::aes(x = att_level, ymin = conf.low, ymax = conf.high), width = 0.2) +
      ggplot2::labs(title = label_att, y = estimand, x = NULL)
    
    return(g)
  }
  
  # pointrange
  target_estimand <- paste0("mm_", .estimates)
  
  out <- x$estimates |>
    dplyr::filter(.data$estimand == target_estimand) |>
    dplyr::select(-estimand) |>
    dplyr::left_join(x$labels |>
                       dplyr::select(level_id, level, attribute),
                     by = c("att_level_choose" = "level_id")) |>
    dplyr::rename(level1 = level, attribute1 = attribute) |>
    dplyr::left_join(x$labels |>
                       dplyr::select(level_id, level, attribute),
                     by = c("att_level_notchoose" = "level_id")) |>
    dplyr::rename(level0 = level, attribute0 = attribute)
  
  if (nrow(out) != 1) {
    stop("This plotting function expects only one attribute-level pair at a time.")
  }
  
  g <- ggplot2::ggplot(out, ggplot2::aes(x = estimate, y = 1)) +
    ggplot2::geom_vline(xintercept = 0.5, linetype = "dashed", color = "gray") +
    ggplot2::geom_pointrange(ggplot2::aes(xmin = conf.low, xmax = conf.high)) +
    ggplot2::geom_text(ggplot2::aes(label = format(round(estimate, 2), nsmall = 2)), vjust = -1) +
    ggplot2::annotate("text", x = .xlim[1] - 0.08 * diff(.xlim), y = 1, label = label0, hjust = 1) +
    ggplot2::annotate("text", x = .xlim[2] + 0.08 * diff(.xlim), y = 1, label = label1, hjust = 0) +
    ggplot2::scale_x_continuous(breaks = scales::pretty_breaks(n = 4)(.xlim)) +
    ggplot2::labs(y = NULL, x = NULL) +
    ggplot2::coord_cartesian(xlim = .xlim, clip = "off") +
    ggthemes::theme_few() +
    ggplot2::theme(
      axis.text.x  = ggplot2::element_text(size = 11, color = "black", hjust = .5, vjust = 1),
      axis.text.y  = ggplot2::element_blank(),
      axis.ticks.y = ggplot2::element_blank(),
      plot.margin  = grid::unit(.plot.margin, "cm")
    )
  
  if (.show_attribute) {
    g <- g +
      ggplot2::labs(title = label_att) +
      ggplot2::theme(plot.title = ggplot2::element_text(hjust = 0.5, size = 11, face = "bold", vjust = -0.5))
  }
  
  if (.remove_xaxis) {
    g <- g +
      ggplot2::theme(axis.ticks.x = ggplot2::element_blank(),
                     axis.line.x  = ggplot2::element_blank()) +
      ggplot2::labs(x = NULL)
  } else {
    g <- g + ggplot2::labs(x = estimand)
  }
  
  g
}

#' Estimate intra-respondent reliability (tau) without a repeated task
#'
#' Uses the extrapolation method to estimate intra-respondent reliability (IRR, \eqn{\tau})
#' when your conjoint design does not include an explicit repeated task. The input is a
#' \code{\link{projoint_data}} object (e.g., produced by \code{\link{reshape_projoint}}).
#'
#' @details
#' The procedure constructs pairs of base tasks within respondent, computes the proportion
#' of identical choices as a function of how many attributes differ between the two tasks,
#' fits a weighted regression of agreement on the number of differing attributes, and
#' extrapolates to zero differences to obtain \eqn{\hat{\tau}}.
#'
#' @param .data A \code{\link{projoint_data}} object (from \code{\link{reshape_projoint}}).
#' @param .title Optional character string used as the plot title prefix.
#'
#' @return A \code{projoint_tau} object (list-like) with components:
#' \itemize{
#'   \item \code{$irr}: a tibble with columns \code{x} (number of differing attributes) and
#'         \code{predicted} (fitted agreement), including \code{x = 0} which is the estimate
#'         of \eqn{\tau}.
#'   \item \code{$figure}: a \code{ggplot2} object visualizing observed agreement by \code{x}
#'         and the fitted line with the extrapolated point at \code{x = 0}.
#' }
#'
#' @seealso \code{\link{plot.projoint_tau}}, \code{\link{summary.projoint_tau}},
#'   \code{\link{reshape_projoint}}
#'
#' @import dplyr
#' @import tidyr
#' @import stringr
#' @import rlang
#' @import tidyselect
#' @import estimatr
#' @import ggplot2
#' @import ggthemes
#' @importFrom stats setNames
#' @importFrom methods new
#'
#' @examples
#' \donttest{
#' # Example workflow:
#' data(exampleData1)
#' outcomes <- c(paste0("choice", 1:8), "choice1_repeated_flipped")
#'
#' # Even if your real study lacks a repeated task, this shows the API:
#' pj <- reshape_projoint(exampleData1, outcomes, .repeated = TRUE)
#'
#' tau_fit <- predict_tau(pj, .title = "IRR (tau): ")
#' # Inspect the extrapolated tau (row where x == 0)
#' tau_fit$irr[tau_fit$irr$x == 0, ]
#'
#' # Plot (also available via plot(tau_fit))
#' print(tau_fit$figure)
#' }
#'
#' @export
predict_tau <- function(
    .data,
    .title = NULL
){

  if(!is(.data, "projoint_data")){
    stop("The .data argument must be of class `projoint_data` from the `reshape_projoint` function.")
  }
  
  # data frame
  
  .dataframe <- .data$data

  # the number of attributes
  
  n_attributes <- names(.dataframe) %>% str_detect(., "att") %>% sum()
  
  # all combinations of tasks (within respondents)
  # There are MANY combinations of tasks for each respondent.
  
  task_combinations <- .dataframe %>% 
    dplyr::select(id, "task1" = task, "task2" = task) %>% 
    dplyr::distinct() %>% 
    dplyr::group_by(id) %>% 
    tidyr::complete(task1, task2) %>% 
    dplyr::ungroup() %>% 
    # remove the same tasks
    dplyr::filter(task1 != task2) %>% 
    # make unique pairs of tasks
    dplyr::mutate(t1 = ifelse(task1 < task2, task2, task1),
                  t2 = ifelse(task1 > task2, task2, task1)) %>% 
    dplyr::select(-task1, -task2) %>% 
    dplyr::distinct()
  
  # initial cleaning: add y (a profile chosen)
  
  d <- .dataframe %>% 
    dplyr::mutate(y = dplyr::case_when(selected == 1 & profile == 1 ~ 1,
                                       selected == 1 & profile == 2 ~ 2,
                                       selected == 0 & profile == 1 ~ 2,
                                       selected == 0 & profile == 2 ~ 1)) %>% 
    dplyr::select(-selected)
  
  # cleaning for each attribute
  
  d2 <- d %>% 
    dplyr::select(id, task, y) %>% 
    dplyr::distinct()
  
  for (i in 1:n_attributes){
    
    att_quo <- rlang::as_name(str_c("att", i))
    var1 <- str_c("att", i, "_comb") %>% rlang::quo_name()
    var2 <- str_c("att", i, "_same") %>% rlang::quo_name()
    
    temp <- d %>% 
      dplyr::select(id, task, profile, "att" = !!att_quo) %>% 
      dplyr::group_by(id, task) %>% 
      tidyr::pivot_wider(names_from = profile, values_from = att) %>% 
      dplyr::ungroup() %>% 
      dplyr::mutate(att_comb = stringr::str_c(`1`, ", ", `2`)) %>% 
      dplyr::select(id, task,
                    !!var1 := att_comb) 
    
    d2 <- d2 %>% 
      left_join(temp, by = c("id", "task"))
    
  }
  
  # rename attributes for each task
  
  task1 <- d2 %>% setNames(str_c("task1_", names(d2)))
  task2 <- d2 %>% setNames(str_c("task2_", names(d2)))
  
  # all combinations of tasks within respondents after cleaning the variable names
  
  d3 <- task_combinations %>% 
    dplyr::left_join(task1, by = c("id" = "task1_id", "t1" = "task1_task")) %>% 
    dplyr::left_join(task2, by = c("id" = "task2_id", "t2" = "task2_task")) %>% 
    dplyr::mutate(y_same = ifelse(task1_y == task2_y, 1, 0))
  
  for (i in 1:n_attributes){
    
    var1 <- sym(str_c("task1_att", i, "_comb"))
    var2 <- sym(str_c("task2_att", i, "_comb"))
    var3 <- str_c("att", i, "_comb_same") %>% rlang::quo_name()
    
    d3 <- d3 %>% 
      mutate(!!var3 := ifelse(!!var1 == !!var2, 0, 1)) # Logical: whether two level-combinations are the same between tasks
    
  }
  
  # dependent variable: the same profile was selected for the same set of tasks
  
  y <- d3 %>% 
    dplyr::select(id, t1, t2, y_same) %>% 
    dplyr::distinct()
  
  # independent variables: the number of same attributes within task combinations
  
  x <- d3 %>% 
    dplyr::select(id, t1, t2, tidyselect::contains("comb_same")) %>% 
    tidyr::pivot_longer(cols = 4:ncol(.)) %>% 
    dplyr::group_by(id, t1, t2) %>% 
    dplyr::summarise(x = sum(value), .groups = "drop")
  
  # merge y and x and run regression to estimate tau for each x (the number of same attributes)
  
  out1 <- y %>% 
    dplyr::left_join(x, by = c("id", "t1", "t2")) %>% 
    dplyr::group_by(x) %>% 
    # Cluster-robust variances require between-cluster variation.
    dplyr::filter(dplyr::n_distinct(id) > 1) %>%
    dplyr::reframe(estimatr::tidy(estimatr::lm_robust(y_same ~ 1, 
                                                      data = dplyr::pick(everything()), 
                                                      clusters = id))) %>% 
    
    # Remove boundary cases where estimate of reliability is 0 or 1.
    # Due to floating-point arithmetic in `estimatr` / its RcppEigen C++ routines,
    # sometimes the returned estimate is 1-.Machine$double.eps, which is not exactly == 1.
    # We must use `dplyr::near()` to robustly handle near-equality by adding a tiny tolerance.
    dplyr::filter(!dplyr::near(estimate, 0, tol = .Machine$double.eps^0.5)) %>%
    dplyr::filter(!dplyr::near(estimate, 1, tol = .Machine$double.eps^0.5)) %>%
    
    # remove if all attributes are the same within task combinations
    dplyr::filter(x != 0) %>% 
    
    # remove if the standard error is extremely small.
    dplyr::filter(!(std.error <= .Machine$double.eps))
  
  # then run a weighted least square regression
  
  out_reg <-  estimatr::lm_robust(estimate ~ x, data = out1, weights = 1/std.error^2, se_type = "classical") 
  
  # data frame to add predicted values
  
  newdata <- data.frame(x = 0:n_attributes)
  
  # a vector of predicted values
  predicted <- newdata %>% 
    mutate(predicted = predict(out_reg, newdata)) %>% 
    as_tibble()
  
  # prediction of x == 0
  
  prediction <- predicted %>% 
    dplyr::filter(x == 0) %>% 
    dplyr::pull(predicted) 
  
  g <- ggplot2::ggplot() +
    ggplot2::geom_hline(yintercept = 0.5, linetype = "dashed", color = "gray") +
    ggplot2::geom_pointrange(data = out1,
                             ggplot2::aes(x = x,
                                          y = estimate,
                                          ymin = conf.low,
                                          ymax = conf.high)) +
    ggplot2::geom_line(data = predicted, 
                       ggplot2::aes(x = x, 
                                    y = predicted),
                       linetype = "dotted",
                       color = "red") +
    ggplot2::geom_point(data = predicted, 
                        ggplot2::aes(x = x, 
                                     y = predicted),
                        size = 4,
                        shape = 5,
                        color = "red") +
    ggplot2::coord_cartesian(ylim = c(0.40, 1.0), 
                             xlim = c(0, n_attributes)) +
    ggplot2::scale_x_continuous(breaks = seq(0, n_attributes, 1)) +
    ggthemes::theme_few() +
    ggplot2::labs(y = "Percent agreement", 
                  x = "Number of attributes with different levels between two tasks", 
                  title = stringr::str_c(.title, "Prediction = ", format(round(prediction, digits = 2), nsmall = 2)))
  
  
  # return
  out <- projoint_tau(irr = predicted, figure = g)
  
  return(out)
  
}

#' Projoint Example Data Set 3: Building Conjoint with No Repeated Task
#'
#' A cleaned Qualtrics export from a conjoint study that compares two potential
#' new building developments. Each respondent completed 8 standard tasks only;
#' no repeated tasks are included in this dataset.
#'
#' @docType data
#' @usage data(exampleData3)
#'
#' @format A data frame with 400 rows and 184 columns. Contains survey responses
#'   including demographic information, outcome choices, and conjoint attribute
#'   values identified by \code{K-*-*} variable names.
#'
#' @keywords datasets
#'
#' @examples
#' # Load the dataset
#' data(exampleData3)
#'
#' # Inspect the first few rows
#' head(exampleData3)
#'
#' # Number of rows and columns
#' dim(exampleData3)
#'
#' # Display first 10 column names
#' names(exampleData3)[1:10]
#'
#' @name exampleData3
NULL

#' Create a projoint_results Object
#'
#' Internal constructor for \code{projoint_results} objects.  
#' A \code{projoint_results} stores the outputs of a conjoint analysis,
#' including estimated effects and metadata about the analysis settings.
#' It is normally generated by \code{\link{projoint}} and not called directly.
#'
#' @param ... Named elements, typically including:
#'   \itemize{
#'     \item \code{estimates}: A data frame of estimated effects (point estimates,
#'       standard errors, confidence intervals).
#'     \item \code{labels}: Attribute/level label mappings from the design.
#'     \item \code{structure}: Either \code{"profile_level"} or \code{"choice_level"}.
#'     \item \code{estimand}: Either \code{"mm"} (marginal mean) or
#'       \code{"amce"} (average marginal component effect).
#'     \item \code{irr}: Intra-respondent reliability value used.
#'     \item \code{tau}: Estimated \eqn{\tau} value from IRR correction.
#'     \item \code{se_method}: Method used for standard errors.
#'   }
#'
#' @return A list with class \code{"projoint_results"} containing
#'   estimates and associated metadata. This object is the standard
#'   output of \code{\link{projoint}} and is accepted by plotting
#'   and summary methods.
#'
#' @keywords internal
projoint_results <- function(...) {
  structure(
    list(...),
    class = "projoint_results"
  )
}

#' Plot method for \code{projoint_tau}
#'
#' Visualizes the estimated intra-respondent reliability (\eqn{\tau}) produced
#' by the extrapolation method and stored in a \code{projoint_tau} object.
#'
#' @param x A \code{projoint_tau} object.
#' @param ... Optional arguments (currently unused).
#'
#' @return A \code{ggplot2} object representing the IRR (\eqn{\tau}) visualization.
#'   The plot is drawn for its side effect and also returned (invisibly).
#'
#' @export
#'
#' @examples
#' \donttest{
#' # Estimate tau, then plot:
#' # dat <- reshape_projoint(exampleData1, .outcomes = c("choice1","choice2"))
#' # tau_fit <- projoint_tau(dat)  # or predict_tau(dat)
#' # p <- plot(tau_fit)            # also returns the ggplot object (invisibly)
#' }
plot.projoint_tau <- function(x, ...) {
  p <- x$figure
  print(p)
  invisible(p)
}

#' Make a \code{projoint_data} object from a labelled tibble
#'
#' Converts a labelled tibble/data frame (one column per attribute) into an object
#' of class \code{projoint_data} that downstream functions (e.g., \code{\link{projoint}})
#' can consume. The unit of observation should be each of two profiles in each task
#' for each respondent.
#'
#' @param .dataframe A data frame (or tibble). One row per profile per task per respondent.
#' @param .attribute_vars Character vector of attribute column names.
#' @param .id_var Column name (character) with respondent IDs. Default \code{"id"}.
#' @param .task_var Column name (character) with task numbers. Default \code{"task"}.
#' @param .profile_var Column name (character) with profile numbers. Default \code{"profile"}.
#' @param .selected_var Column name (character) with the binary choice for each task
#'   (values in \code{\{0,1\}}). Default \code{"selected"}.
#' @param .selected_repeated_var Optional column name (character) with the binary choice
#'   for the repeated task. Default \code{NULL}.
#' @param .fill Logical. If \code{TRUE}, uses repeated-task agreement to \emph{fill}
#'   missing agreement values for non-repeated tasks (assumes IRR is independent of
#'   table content). If unsure, prefer the default \code{FALSE}.
#'
#' @return A \code{projoint_data} object (a list-like object containing a \code{labels}
#'   tibble and a \code{data} tibble) ready to pass to \code{\link{projoint}} and related
#'   functions.
#'
#' @export
#'
#' @examples
#' \donttest{
#' # Example: build a projoint_data object from the labelled-tibble example
#' data(exampleData1_labelled_tibble)
#'
#' att_cols <- c("School Quality", "Violent Crime Rate (Vs National Rate)",
#'               "Racial Composition", "Housing Cost",
#'               "Presidential Vote (2020)", 
#'               "Total Daily Driving Time for Commuting and Errands",
#'               "Type of Place")
#'
#' pj_dat <- make_projoint_data(
#'   .dataframe             = exampleData1_labelled_tibble,
#'   .attribute_vars        = att_cols,
#'   .id_var                = "id",
#'   .task_var              = "task",
#'   .profile_var           = "profile",
#'   .selected_var          = "selected",
#'   .selected_repeated_var = "selected_repeated",
#'   .fill                  = FALSE
#' )
#'
#' class(pj_dat)
#' # [1] "projoint_data"
#' }
make_projoint_data <- function(
    .dataframe, 
    .attribute_vars,
    .id_var = "id",
    .task_var = "task",
    .profile_var = "profile",
    .selected_var = "selected", 
    .selected_repeated_var = NULL, 
    .fill = FALSE){
  
  # check
  
  if (!is.data.frame(.dataframe)){
    stop("The .dataframe argument must be a data frame (or a tibble).")
  }
  if (!is.character(.attribute_vars)){
    stop("The .attribute_vars argument must be a vector of characters with the length of 1 or more.")
  } 
  if (length(.attribute_vars) == 0L) {
    stop("`.attribute_vars` is empty. Supply the names of your attribute columns.")
  }
  if (!is.character(.id_var)){
    stop("The .id_var argument must be a character")
  } 
  if (!is.character(.task_var)){
    stop("The .task_var argument must be a character.")
  } 
  if (!is.character(.profile_var)){
    stop("The .profile_var argument must be a character.")
  } 
  if (!is.character(.selected_var)){
    stop("The .attribute_vars argument must be a character.")
  } 
  if (!is.null(.selected_repeated_var) & !is.character(.selected_repeated_var)){
    stop("The .selected_repeated_var argument must be NULL or a character.")
  } 
  
  # assign attribute_id (in an alphabetical order)
  attributes <- data.frame(attribute = .attribute_vars) |> 
    dplyr::mutate(attribute_id = factor(attribute) |> as.numeric(),
                  attribute_id = stringr::str_c("att", attribute_id))
  
  # rename variables
  variable_names <- data.frame("original" = c("id", "task", "profile"), 
                               "renamed" = c("id", "task", "profile")) |>
    dplyr::bind_rows(attributes |> 
                       rlang::set_names(c("original", "renamed"))) |> 
    dplyr::add_row(original = "selected", 
                   renamed = "selected")
  
  # rename and reorder variables
  if (is.null(.selected_repeated_var)){
    
    data <- .dataframe |> 
      dplyr::select("id" = all_of(.id_var),
                    "task" = all_of(.task_var), 
                    "profile" = all_of(.profile_var),
                    any_of(.attribute_vars),
                    "selected" = all_of(.selected_var)) |> 
      rlang::set_names(pull(variable_names)) |> 
      mutate(selected_repeated = NA,
             agree = NA)
    
  } else{
    
    variable_names <- variable_names |> 
      dplyr::add_row(original = "selected_repeated", 
                     renamed = "selected_repeated")
    
    data <- .dataframe |> 
      dplyr::select("id" = all_of(.id_var),
                    "task" = all_of(.task_var), 
                    "profile" = all_of(.profile_var),
                    any_of(.attribute_vars),
                    "selected" = all_of(.selected_var), 
                    "selected_repeated" = all_of(.selected_repeated_var)) |> 
      rlang::set_names(pull(variable_names)) |> 
      mutate(agree = ifelse(selected == selected_repeated, 1, 0))
    
  }
  
  # make "labels" data frame
  labels <- data |> 
    dplyr::select(dplyr::all_of(attributes$attribute_id)) |>
    pivot_longer(names_to = "attribute_id", values_to = "level", cols = dplyr::everything()) |> 
    distinct() |> 
    dplyr::arrange(attribute_id, level) |> 
    dplyr::group_by(attribute_id) |> 
    dplyr::mutate(level_id = dplyr::row_number()) |> 
    dplyr::ungroup() |> 
    dplyr::mutate(level_id = stringr::str_c(attribute_id, ":level", level_id)) |> 
    dplyr::left_join(attributes, by = "attribute_id") |> 
    dplyr::arrange(attribute, level, attribute_id, level_id)
  
  for (i in 1:nrow(attributes)){
    
    .attribute_id <- attributes$attribute_id[i]
    var_quo <- rlang::sym(.attribute_id)
    
    temp <- labels |> 
      dplyr::filter(attribute_id == .attribute_id) |> 
      dplyr::select(level, level_id) |> 
      rlang::set_names(c(.attribute_id, "level_id"))
    
    suppressMessages(
      data <- data |> 
        
        dplyr::left_join(temp) |> 
        dplyr::select(-all_of(.attribute_id)) |> 
        dplyr::rename({{var_quo}} := level_id)
    )
    
    
  }
  
  if (.fill == TRUE){
    
    data_final <- data |> 
      dplyr::arrange(id, task, agree) |> 
      tidyr::fill(agree)
    
  } else{
    
    data_final <- data |> 
      dplyr::arrange(id, task, agree)
  }
  
  
  # a class generator  
  out <- projoint_data("labels" = labels,
                       "data" = data_final)
  
  # return
  return(out)
  
}

#' Estimate subgroup differences (internal)
#'
#' Worker used by \code{\link{projoint}} to compute subgroup differences
#' (group \code{== 1} minus group \code{== 0}) in marginal means (MMs) or
#' average marginal component effects (AMCEs). Supported only for
#' \code{.structure = "profile_level"}.
#'
#' @param .data A \code{\link{projoint_data}} object.
#' @param .qoi Optional \code{\link{projoint_qoi}}; if \code{NULL}, estimates all MMs/AMCEs.
#' @param .by_var Column name in \code{.data$data} defining subgroups; must be logical,
#'   numeric/integer coded as 0/1, or factor with levels \code{"0"}/\code{"1"}.
#' @param .structure Must be \code{"profile_level"}.
#' @param .estimand Either \code{"mm"} or \code{"amce"}.
#' @param .se_method One of \code{"analytical"}, \code{"simulation"}, or \code{"bootstrap"}.
#' @param .irr \code{NULL} to estimate IRR from repeated tasks, or numeric to fix IRR.
#' @param .remove_ties Logical; drop ties before estimation? Default \code{TRUE}.
#' @param .ignore_position Ignored (subgroup analysis is profile-level only).
#' @param .n_sims Integer; required when \code{.se_method = "simulation"}.
#' @param .n_boot Integer; required when \code{.se_method = "bootstrap"}.
#' @param .weights_1,.clusters_1,.se_type_1 Passed to \code{\link[estimatr]{lm_robust}} for IRR estimation.
#' @param .weights_2,.clusters_2,.se_type_2 Passed to \code{\link[estimatr]{lm_robust}} for MM/AMCE estimation.
#' @param .auto_cluster Logical; if \code{TRUE} (default), auto-cluster on \code{id} when suitable
#'   and no clusters are provided (applies only if the corresponding \code{.se_type_*} is \code{NULL}).
#' @param .seed Optional integer; sets a temporary RNG seed and restores prior state on exit.
#'
#' @return A \code{\link{projoint_results}} object containing subgroup differences with fields:
#' \itemize{
#'   \item \code{estimates}: tibble with one row per attribute/level and columns
#'         like \code{estimate} (group1 minus group0), \code{se}, \code{conf.low},
#'         \code{conf.high}, plus internal columns (\code{estimate_1}, \code{estimate_0},
#'         \code{se_1}, \code{se_0}) used in the diff.
#'   \item \code{estimand}: \code{"mm"} or \code{"amce"}.
#'   \item \code{structure}: \code{"profile_level"}.
#'   \item \code{se_method}: SE method used.
#'   \item \code{irr}: \code{"Estimated"} or \code{"Assumed (<value> )"}.
#'   \item \code{tau}: numeric reliability used (average of subgroup taus when estimated).
#'   \item \code{remove_ties}, \code{ignore_position}: flags echoed from inputs.
#'   \item \code{se_type_used}, \code{cluster_by}: details propagated from fitting calls.
#'   \item \code{labels}, \code{data}: design labels and the analysis data (passed through).
#' }
#'
#' @keywords internal
#' @seealso \code{\link{projoint}}, \code{\link{projoint_level}}, \code{\link{projoint_results}}
projoint_diff <- function(
    .data,
    .qoi,
    .by_var,
    .structure,
    .estimand,
    .se_method,
    .irr,
    .remove_ties,
    .ignore_position,
    .n_sims,
    .n_boot,
    .weights_1,
    .clusters_1,
    .se_type_1,
    .weights_2,
    .clusters_2,
    .se_type_2,
    .auto_cluster = TRUE,
    .seed = NULL
){
  
  # ---- Early guards --------------------------------------------------------
  
  .structure <- rlang::arg_match0(.structure, c("profile_level","choice_level"))
  
  # require .by_var and validate it
  if (missing(.by_var) || is.null(.by_var)) {
    stop("`.by_var` must be provided for subgroup analysis.")
  }
  if (.structure != "profile_level") {
    stop("Subgroup analysis via `.by_var` is only supported for `.structure = 'profile_level'`.")
  }
  if (!.by_var %in% names(.data$data)) {
    stop("`.by_var` must be a column in `.data$data`.")
  }
  vals <- unique(stats::na.omit(.data$data[[.by_var]]))
  is_ok <- is.logical(.data$data[[.by_var]]) ||
    (is.numeric(vals) && all(vals %in% c(0, 1))) ||
    (is.factor(.data$data[[.by_var]]) && all(levels(.data$data[[.by_var]]) %in% c("0","1")))
  if (!is_ok) {
    stop("`.by_var` must be logical (TRUE/FALSE) or numeric/integer coded as 0/1 (factor levels '0'/'1' allowed).")
  }

  # ---- Optional reproducible RNG (wrapper) -----------------------------------
  if (!is.null(.seed)) {
    if (exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) {
      old_seed <- .Random.seed
      on.exit({
        if (exists("old_seed", inherits = FALSE)) .Random.seed <<- old_seed
      }, add = TRUE)
    }
    set.seed(.seed)
  }
  
  # ---- Split data -----------------------------------------------------------
  
  by_col <- .data$data[[.by_var]]
  # allow factor "0"/"1"
  if (is.factor(by_col) && all(levels(by_col) %in% c("0","1"))) {
    by_col <- as.integer(as.character(by_col))
  }
  
  # drop NAs in .by_var (otherwise they go to neither subgroup)
  keep <- !is.na(by_col)
  
  if (is.logical(by_col)) {
    subgroup1 <- .data$data[ keep &  by_col, , drop = FALSE]
    subgroup0 <- .data$data[ keep & !by_col, , drop = FALSE]
  } else {
    # assumes 0/1 per guard above
    subgroup1 <- .data$data[ keep & by_col == 1L, , drop = FALSE]
    subgroup0 <- .data$data[ keep & by_col == 0L, , drop = FALSE]
  }
  
  if (nrow(subgroup1) == 0L || nrow(subgroup0) == 0L) {
    stop("Both subgroups defined by `.by_var` must have at least one observation.")
  }

  data1 <-  projoint_data("labels" = .data$labels, "data" = subgroup1)
  data0 <-  projoint_data("labels" = .data$labels, "data" = subgroup0)
  
  # ---- Run per subgroup -----------------------------------------------------
  
  out1 <- projoint_level(.data = data1,
                         .qoi = .qoi,
                         .structure = .structure,
                         .estimand = .estimand,
                         .se_method = .se_method,
                         .irr = .irr,
                         .remove_ties = .remove_ties,
                         .ignore_position = .ignore_position,
                         .n_sims = .n_sims,
                         .n_boot = .n_boot,
                         .weights_1 = .weights_1,
                         .clusters_1 = .clusters_1,
                         .se_type_1 = .se_type_1,
                         .weights_2 = .weights_2,
                         .clusters_2 = .clusters_2,
                         .se_type_2 = .se_type_2,
                         .auto_cluster = .auto_cluster)
  
  out0 <- projoint_level(.data = data0,
                         .qoi = .qoi,
                         .structure = .structure,
                         .estimand = .estimand,
                         .se_method = .se_method,
                         .irr = .irr,
                         .remove_ties = .remove_ties,
                         .ignore_position = .ignore_position,
                         .n_sims = .n_sims,
                         .n_boot = .n_boot,
                         .weights_1 = .weights_1,
                         .clusters_1 = .clusters_1,
                         .se_type_1 = .se_type_1,
                         .weights_2 = .weights_2,
                         .clusters_2 = .clusters_2,
                         .se_type_2 = .se_type_2,
                         .auto_cluster = .auto_cluster)
  
  # ---- Combine --------------------------------------------------------------
  
  estimate1 <- out1$estimates |> 
    dplyr::select(estimand, att_level_choose,
                  "estimate_1" = estimate,
                  "se_1" = se) |> 
    dplyr::mutate(tau = out1$tau)
  
  estimate0 <- out0$estimates |> 
    dplyr::select(estimand, att_level_choose,
                  "estimate_0" = estimate,
                  "se_0" = se)
  
  estimates <- estimate1 |> 
    dplyr::left_join(estimate0, by = c("estimand", "att_level_choose")) |> 
    dplyr::mutate(estimate = estimate_1 - estimate_0,
           se = sqrt(se_1^2 + se_0^2), 
           conf.low = estimate - 1.96 * se,
           conf.high = estimate + 1.96 * se) 
  
  tau <- mean(c(out1$tau, out0$tau), na.rm = TRUE)

  se_type_used <- out1$se_type_used
  cluster_by   <- out1$cluster_by

  # ---- Labels & return ------------------------------------------------------
  
  if (is.null(.irr)){
    irr <- "Estimated"
  } else {
    irr <- stringr::str_c("Assumed (", .irr, ")")
  }

  if (.estimand == "mm"){
    
    if(is.null(.qoi)){
      return(projoint_results("estimand" = .estimand,
                       "structure" = .structure,
                       "estimates" = estimates, 
                       "se_method" = .se_method,
                       "irr" = irr,
                       "tau" = tau,
                       "remove_ties" = .remove_ties,
                       "ignore_position" = .ignore_position,
                       "attribute_of_interest" = "all",
                       "levels_of_interest" = "all",
                       "attribute_of_interest_0" = NULL,
                       "levels_of_interest_0" = NULL,
                       "attribute_of_interest_baseline" = NULL,
                       "levels_of_interest_baseline" = NULL,
                       "attribute_of_interest_0_baseline" = NULL,
                       "levels_of_interest_0_baseline" = NULL,
                       "se_type_used" = se_type_used,
                       "cluster_by"   = cluster_by,
                       labels = .data$labels,
                       data = .data$data))
    } else {
      return(projoint_results("estimand" = .estimand,
                       "structure" = .structure,
                       "estimates" = estimates, 
                       "se_method" = .se_method,
                       "irr" = irr,
                       "tau" = tau,
                       "remove_ties" = .remove_ties,
                       "ignore_position" = .ignore_position,
                       "attribute_of_interest" = .qoi$attribute_of_interest,
                       "levels_of_interest" = .qoi$levels_of_interest,
                       "attribute_of_interest_0" = .qoi$attribute_of_interest_0,
                       "levels_of_interest_0" = .qoi$levels_of_interest_0,
                       "attribute_of_interest_baseline" = NULL,
                       "levels_of_interest_baseline" = NULL,
                       "attribute_of_interest_0_baseline" = NULL,
                       "levels_of_interest_0_baseline" = NULL,
                       "se_type_used" = se_type_used,
                       "cluster_by"   = cluster_by,
                       labels = .data$labels,
                       data = .data$data))
    }
    
  } else {
    
    if(is.null(.qoi)){
      return(projoint_results("estimand" = .estimand,
                       "structure" = .structure,
                       "estimates" = estimates, 
                       "se_method" = .se_method,
                       "irr" = irr,
                       "tau" = tau,
                       "remove_ties" = .remove_ties,
                       "ignore_position" = .ignore_position,
                       "attribute_of_interest" = "all",
                       "levels_of_interest" = "all except level1",
                       "attribute_of_interest_0" = NULL,
                       "levels_of_interest_0" = NULL,
                       "attribute_of_interest_baseline" = "all",
                       "levels_of_interest_baseline" = "level1",
                       "attribute_of_interest_0_baseline" = NULL,
                       "levels_of_interest_0_baseline" = NULL,
                       "se_type_used" = se_type_used,
                       "cluster_by"   = cluster_by,
                       labels = .data$labels,
                       data = .data$data))

    } else {
      return(projoint_results("estimand" = .estimand,
                       "structure" = .structure,
                       "estimates" = estimates, 
                       "se_method" = .se_method,
                       "irr" = irr,
                       "tau" = tau,
                       "remove_ties" = .remove_ties,
                       "ignore_position" = .ignore_position,
                       "attribute_of_interest" = .qoi$attribute_of_interest,
                       "levels_of_interest" = .qoi$levels_of_interest,
                       "attribute_of_interest_0" = .qoi$attribute_of_interest_0,
                       "levels_of_interest_0" = .qoi$levels_of_interest_0,
                       "attribute_of_interest_baseline" = .qoi$attribute_of_interest_baseline,
                       "levels_of_interest_baseline" = .qoi$levels_of_interest_baseline,
                       "attribute_of_interest_0_baseline" = .qoi$attribute_of_interest_0_baseline,
                       "levels_of_interest_0_baseline" = .qoi$levels_of_interest_0_baseline,
                       "se_type_used" = se_type_used,
                       "cluster_by"   = cluster_by,
                       labels = .data$labels,
                       data = .data$data))
    }
    
  }
}

#' Create a projoint_tau Object
#'
#' Internal constructor for \code{projoint_tau} objects.
#' A \code{projoint_tau} stores intra-respondent reliability (IRR) estimates
#' obtained via the extrapolation method, along with an optional diagnostic figure.
#'
#' @param irr A data frame (or tibble) with predicted IRR values as a function
#'   of the number of differing attributes between task pairs. Must include at
#'   least column \code{predicted}. If available, include \code{x} (number of differing attributes).
#' @param figure Optional. A \code{ggplot2} object showing the extrapolation plot
#'   of IRR versus number of differing attributes, or \code{NULL}.
#'
#' @return A list of class \code{"projoint_tau"} with elements:
#'   \itemize{
#'     \item \code{irr}: Data frame of IRR estimates and predictions.
#'     \item \code{figure}: A \code{ggplot2} object with the diagnostic plot, or \code{NULL}.
#'   }
#'
#' @keywords internal
#' @noRd
projoint_tau <- function(irr, figure = NULL) {
  if (!is.data.frame(irr)) {
    stop("`irr` must be a data frame (or tibble).", call. = FALSE)
  }
  if (!("predicted" %in% names(irr))) {
    stop("`irr` must contain a `predicted` column.", call. = FALSE)
  }
  if (!is.null(figure) && !inherits(figure, "ggplot")) {
    stop("`figure` must be a ggplot object or NULL.", call. = FALSE)
  }
  
  structure(
    list(
      irr = irr,
      figure = figure
    ),
    class = "projoint_tau"
  )
}

#' Plot method for \code{projoint_results}
#'
#' Creates publication-ready plots from a \code{projoint_results} object produced by
#' \code{\link{projoint}}. Supports both profile-level and choice-level analyses,
#' with plotting options tailored to each structure.
#'
#' @param x A \code{projoint_results} object (typically from \code{\link{projoint}}).
#' @param .estimates Character: which estimates to plot. One of
#'   \code{"corrected"}, \code{"uncorrected"}, or \code{"both"} (for profile-level),
#'   and \code{"corrected"} or \code{"uncorrected"} (for choice-level). Default \code{"corrected"}.
#' @param .by_var Logical (profile-level only). Whether to plot subgroup differences.
#'   Default \code{FALSE}.
#' @param .labels Character vector of length 2 (choice-level only). Custom x-axis
#'   labels for bar/pointrange plots. If \code{NULL}, labels are taken from \code{x$labels}.
#' @param .base_size Numeric. Base font size. Default \code{12}.
#' @param .base_family Character. Base font family. Default \code{""} (system default).
#' @param .type Character (choice-level only). One of \code{"bar"} or \code{"pointrange"}.
#'   Default \code{"bar"}.
#' @param .show_attribute Logical (choice-level only). Show the attribute name as the
#'   title when both levels belong to the same attribute. Default \code{TRUE}.
#' @param .remove_xaxis Logical (choice-level only). Remove x-axis line, ticks, and labels.
#'   Default \code{FALSE}.
#' @param .xlim Numeric length-2 vector (choice-level only). X-axis limits. Default \code{c(0, 1)}.
#' @param .plot.margin Numeric length-4 vector (choice-level only). Plot margins in cm:
#'   \code{c(top, left, bottom, right)}. Default \code{c(0, 3, 0, 3)}.
#' @param ... Additional arguments passed to downstream plotting helpers.
#'
#' @details
#' For profile-level results, only \code{.by_var}, \code{.base_size}, and \code{.base_family}
#' are relevant. For choice-level results, only \code{.type}, \code{.labels},
#' \code{.show_attribute}, \code{.remove_xaxis}, \code{.xlim}, and \code{.plot.margin}
#' are relevant. Irrelevant arguments are ignored with a warning.
#'
#' @return A \code{ggplot2} object.
#'
#' @seealso \code{\link{projoint}}, \code{plot_projoint_profile_level},
#'   \code{\link{plot_projoint_choice_level_mm}}
#'
#' @export
#'
#' @examples
#' \donttest{
#' data(exampleData1)
#'
#' # Two base tasks (1 & 2) + repeated of task 1 (last)
#' dat <- reshape_projoint(
#'   exampleData1,
#'   .outcomes = c("choice1", "choice2", "choice1_repeated_flipped")
#' )
#'
#' # Build a valid QOI from the labels
#' att <- unique(dat$labels$attribute_id)[1]
#' levs <- subset(dat$labels, attribute_id == att)$level_id
#' lev_names <- sub(".*:", "", levs)
#'
#' q <- set_qoi(
#'   .structure     = "choice_level",
#'   .estimand      = "mm",
#'   .att_choose    = att,
#'   .lev_choose    = lev_names[2],
#'   .att_notchoose = att,
#'   .lev_notchoose = lev_names[1]
#' )
#'
#' fit <- projoint(dat, .qoi = q)
#'
#' # Plot method
#' plot(fit)
#' }
plot.projoint_results <- function(
    x, 
    .estimates = "corrected",
    
    .by_var = FALSE,
    .labels = NULL,
    .base_size = 12,
    .base_family = "",
    
    .type = c("bar", "pointrange"),
    .show_attribute = TRUE, 
    .remove_xaxis = FALSE, 
    .xlim = c(0, 1),
    .plot.margin = c(0, 3, 0, 3),
    ...
) {
  
  if (!inherits(x, "projoint_results")) {
    stop("The `x` argument must be a `projoint_results` object, output from `projoint()`.")
  }
  
  .estimand <- x$estimand
  .structure <- x$structure
  
  .structure <- match.arg(.structure, choices = c("profile_level", "choice_level"))
  .estimand <- match.arg(.estimand, choices = c("mm", "amce"))
  
  if (.structure == "profile_level") {
  
    .estimates <- match.arg(.estimates, choices = c("corrected", "uncorrected", "both"))
    
    irrelevant_args <- c()
    
    if (!missing(.show_attribute) && .show_attribute != FALSE) irrelevant_args <- c(irrelevant_args, ".show_attribute")
    if (!missing(.remove_xaxis) && .remove_xaxis != FALSE) irrelevant_args <- c(irrelevant_args, ".remove_xaxis")
    if (!missing(.xlim) && !identical(.xlim, c(0, 1))) irrelevant_args <- c(irrelevant_args, ".xlim")
    if (!missing(.plot.margin) && !identical(.plot.margin, c(0, 3, 0, 3))) irrelevant_args <- c(irrelevant_args, ".plot.margin")
    
    if (length(irrelevant_args) > 0) {
      warning("The following arguments are ignored for profile-level plots: ", paste(irrelevant_args, collapse = ", "))
    }
    
    out <- plot_projoint_profile_level(
      x = x,
      .estimates = .estimates,
      .by_var = .by_var,
      .base_size = .base_size,
      .base_family = .base_family,
      ...
    )
    
  } else {
    
    if (.estimand != "mm") {
      stop("Currently, plotting is only supported for `choice_level` structure and `mm` estimand. Stay tuned!")
    }
    
    .estimates <- match.arg(.estimates, choices = c("corrected", "uncorrected"))
    
    .type <- match.arg(.type)
    
    irrelevant_args <- c()
    
    if (!missing(.by_var) && .by_var != FALSE) irrelevant_args <- c(irrelevant_args, ".by_var")
    
    if (length(irrelevant_args) > 0) {
      warning("The following arguments are ignored for choice-level plots: ", paste(irrelevant_args, collapse = ", "))
    }

    out <- plot_projoint_choice_level_mm(
      x = x,
      .type = .type,
      .estimates = .estimates,
      .labels = .labels,
      .show_attribute = .show_attribute, 
      .remove_xaxis = .remove_xaxis, 
      .xlim = .xlim,
      .plot.margin = .plot.margin,
      ...
    )
    
  } 
  
  return(out)
}

#' Print method for projoint_results
#'
#' Custom print method for objects of class \code{projoint_results}.
#'
#' @param x An object of class \code{projoint_results}.
#' @param ... Additional arguments (ignored).
#'
#' @return No return value, called for its side effect of printing a summary of
#'   the \code{projoint_results} object to the console.
#'
#' @export
#'
#' @examples
#' \donttest{
#' data(exampleData1)
#' dat <- reshape_projoint(
#'   exampleData1,
#'   .outcomes = c("choice1", "choice2", "choice1_repeated_flipped")
#' )
#' att <- unique(dat$labels$attribute_id)[1]
#' levs <- subset(dat$labels, attribute_id == att)$level_id
#' lev_names <- sub(".*:", "", levs)
#' q <- set_qoi(
#'   .structure     = "choice_level",
#'   .estimand      = "mm",
#'   .att_choose    = att,
#'   .lev_choose    = lev_names[2],
#'   .att_notchoose = att,
#'   .lev_notchoose = lev_names[1]
#' )
#' fit <- projoint(dat, .qoi = q)
#' print(fit)
#' }
print.projoint_results <- function(x, ...) {
  
  cat("\nProjoint results object\n")
  cat("-------------------------\n")
  cat("Estimand: ", x$estimand, "\n")
  cat("Structure: ", x$structure, "\n")
  cat("Standard error method: ", x$se_method, "\n")
  cat("IRR: ", x$irr, "\n")
  cat("Tau: ", round(x$tau, 3), "\n")
  cat("Number of estimates: ", nrow(x$estimates), "\n")
  invisible(x)
}

#' Plot all profile-level MMs or AMCEs (helper)
#'
#' Internal helper used by \code{\link{plot.projoint_results}} to render
#' profile-level marginal means (MMs) or AMCEs.
#'
#' @keywords internal
#' @param x A \code{\link{projoint_results}} object (profile-level).
#' @param .estimates \code{"corrected"}, \code{"uncorrected"}, or \code{"both"}.
#' @param .by_var Logical; if \code{TRUE}, plot subgroup differences.
#' @param .base_size Base font size (pts).
#' @param .base_family Base font family.
#' @param ... Additional options passed internally.
#'
#' @return A \code{ggplot2} object.
#'
#' @keywords internal
#' @noRd
plot_projoint_profile_level <- function(
    x, 
    .estimates = "corrected",
    .by_var = FALSE,
    .base_size = 12,
    .base_family = "",
    ...
){
  
  # check -------------------------------------------------------------------
  
  if (!inherits(x, "projoint_results")) {
    stop("`x` must be a 'projoint_results' object returned by `projoint()`.", call. = FALSE)
  }
  
  .estimates <- match.arg(.estimates, c("corrected", "uncorrected", "both"))

  if (!is.logical(.by_var) || length(.by_var) != 1) stop("`.by_var` must be TRUE or FALSE.", call. = FALSE)
  
  
  .estimand = x$estimand
  
  if (.by_var == FALSE){
    
    if (.estimand == "mm"){
      
      .xintercept = 0.5
      .xlabel = "Marginal Mean"
      
    } else if (.estimand == "amce"){
      
      .xintercept = 0
      .xlabel = "Average Marginal Component Effect"
      
    } else{
      
      stop("The .estimand argument should be either mm or amce.")
      
    }
    
  } else if (.by_var == TRUE){
    
    .xintercept = 0
    .xlabel = "Difference"
    
  } else {
    
    stop("The .by_var argument should be logical.")
    
  }
  
  
  
  # initial data wrangling --------------------------------------------------
  
  out1 <- dplyr::left_join(
    x$estimates |> 
      dplyr::mutate(level_id = att_level_choose,
                    estimates = dplyr::case_when(stringr::str_detect(estimand, "uncorrected") ~ "uncorrected",
                                                 stringr::str_detect(estimand, "corrected") ~ "corrected")) |>
      dplyr::select(-estimand),
    x$labels |> 
      dplyr::select(attribute, level, level_id),
    by = dplyr::join_by(level_id)
  )
  
  attributes <-  x$labels |> 
    dplyr::select(attribute, level_id) |>
    dplyr::mutate(level_id = stringr::str_replace_all(level_id, "\\d+$", "0")) |> 
    dplyr::distinct()
  
  if (.estimand == "mm"){
    
    out2 <- dplyr::bind_rows(
      out1,
      attributes |> dplyr::mutate(estimates = "corrected"),
      attributes |> dplyr::mutate(estimates = "uncorrected")
    ) 
    
  } else if (.estimand == "amce"){
    
    levels1 <- x$labels |> 
      dplyr::select(attribute, level, level_id) |> 
      dplyr::filter(stringr::str_detect(level_id, "level1$"))
    
    out2 <- dplyr::bind_rows(
      out1,
      levels1 |> dplyr::mutate(estimates = "corrected"),
      levels1 |> dplyr::mutate(estimates = "uncorrected"),
      attributes |> dplyr::mutate(estimates = "corrected"),
      attributes |> dplyr::mutate(estimates = "uncorrected")
    ) |> 
      dplyr::arrange(level_id) |> 
      dplyr::mutate(estimate = ifelse(stringr::str_detect(level_id, "level1$"), 0, estimate),
                    se = ifelse(stringr::str_detect(level_id, "level1$"), 0, se)) 
    
    
  } 
  
  out3 <- dplyr::bind_rows(
    out2 |> 
      dplyr::filter(estimates == "corrected") |> 
      dplyr::arrange(dplyr::desc(level_id)) |> 
      dplyr::mutate(order = dplyr::row_number()),
    out2 |> 
      dplyr::filter(estimates == "uncorrected") |> 
      dplyr::arrange(dplyr::desc(level_id)) |> 
      dplyr::mutate(order = dplyr::row_number())
  )
  
  out4 <- if (.estimates == "both") out3 else out3 |> dplyr::filter(estimates == .estimates)
  
  if (!nrow(out4)) {
    stop("No data to plot for the requested options.", call. = FALSE)
  }
  
  # make labels for the vertical axis ---------------------------------------
  
  labels <- out4 |> 
    dplyr::mutate(att_level_labels = ifelse(is.na(level), 
                                            stringr::str_c(attribute, ":"), 
                                            stringr::str_c("     ", level))) |> 
    dplyr::select(order, att_level_labels) |> 
    dplyr::distinct() |> 
    dplyr::pull(att_level_labels)
  
  # make a figure -----------------------------------------------------------
  
  g <- ggplot2::ggplot(out4) +
    ggplot2::geom_vline(xintercept = .xintercept,
                        linetype = "dashed", 
                        color = "gray") +
    ggplot2::scale_y_continuous(breaks = seq_along(labels),
                                labels = labels) +
    ggplot2::labs(y = NULL,
                  x = .xlabel) +
    ggthemes::theme_few(base_size = .base_size, 
                        base_family = .base_family) %+replace%
    ggplot2::theme(axis.text.x =  ggplot2::element_text(size = .base_size, 
                                                        color = "black", 
                                                        hjust = .5 , 
                                                        vjust = 1),
                   axis.text.y =  ggplot2::element_text(size = .base_size, 
                                                        color = "black", 
                                                        hjust = 0  , 
                                                        vjust = 0.5),
                   axis.ticks =   ggplot2::element_line(colour = "grey50"),
                   axis.title.y = ggplot2::element_text(size = .base_size, 
                                                        angle = 90, 
                                                        vjust = .01, 
                                                        hjust = .1))
  
  
  if (.estimates != "both"){
    
    if (.estimand == "mm"){
      
      g + ggplot2::geom_pointrange(ggplot2::aes(x = estimate,
                                       xmin = conf.low,
                                       xmax = conf.high, 
                                       y = order), 
                                   na.rm = TRUE)
      
    } else if (.estimand == "amce"){
      
      g + ggplot2::geom_pointrange(data = out4 |> 
                                     dplyr::filter(!stringr::str_detect(level_id, "level1$")),
                                   ggplot2::aes(x = estimate,
                                       xmin = conf.low,
                                       xmax = conf.high, 
                                       y = order), 
                                   na.rm = TRUE) +
        ggplot2::geom_pointrange(data = out4 |> 
                                   dplyr::filter(stringr::str_detect(level_id, "level1$")),
                                 ggplot2::aes(x = estimate,
                                     xmin = estimate,
                                     xmax = estimate,
                                     y = order), 
                                 color = "gray",
                                 shape = 1) 
    }
    
    
  } else{
    
    if (.estimand == "mm"){
      
      g + ggplot2::geom_pointrange(data = out4,
                                   ggplot2::aes(x = estimate,
                                       xmin = conf.low,
                                       xmax = conf.high, 
                                       y = order,
                                       color = estimates), 
                                   position = ggplot2::position_dodge(width = 0.5),
                                   na.rm = TRUE) +
        ggplot2::theme(legend.position = "top") +
        ggplot2::labs(color = NULL)
      
    } else if (.estimand == "amce"){
      
      g + ggplot2::geom_pointrange(data = out4 |> 
                                     dplyr::filter(!stringr::str_detect(level_id, "level1$")),
                                   ggplot2::aes(x = estimate,
                                       xmin = conf.low,
                                       xmax = conf.high, 
                                       y = order,
                                       color = estimates), 
                                   position = ggplot2::position_dodge(width = 0.5),
                                   na.rm = TRUE) +
        ggplot2::geom_pointrange(data = out4 |> 
                                   dplyr::filter(stringr::str_detect(level_id, "level1$")),
                                 ggplot2::aes(x = estimate,
                                     xmin = estimate,
                                     xmax = estimate,
                                     y = order), 
                                 color = "gray",
                                 shape = 1) +
        ggplot2::theme(legend.position = "top") +
        ggplot2::labs(color = NULL)
      
      
    }
    
    
  }
  
  
}

#' Create a projoint_qoi Object
#'
#' Internal constructor for \code{projoint_qoi} objects.  
#' A \code{projoint_qoi} stores the specification of custom
#' quantities of interest (QOIs) for conjoint estimation.
#' Typically called by \code{\link{set_qoi}} rather than directly.
#'
#' @param ... Named elements specifying QOI details, such as:
#'   \itemize{
#'     \item \code{structure}: Analysis level (\code{"profile_level"} or \code{"choice_level"}).
#'     \item \code{estimand}: Quantity of interest (\code{"mm"} or \code{"amce"}).
#'     \item \code{attribute_of_interest}, \code{levels_of_interest}, etc.
#'   }
#'
#' @return A list of QOI specifications with class \code{"projoint_qoi"}.
#'
#' @keywords internal
projoint_qoi <- function(...) {
  structure(
    list(...),
    class = "projoint_qoi"
  )
}

#' Summary method for projoint_tau objects
#'
#' Custom summary method for objects of class \code{projoint_tau}, typically created
#' by \code{\link{projoint}} or related functions. Summarizes intra-respondent
#' reliability (IRR) estimates.
#'
#' @param object An object of class \code{projoint_tau}.
#' @param ... Additional arguments (currently unused).
#'
#' @return A tibble (data frame) showing IRR estimates, typically by the number
#'   of differing attributes, as stored in \code{object$irr}.
#'
#' @export
#'
#' @examples
#' toy_tau <- structure(
#'   list(irr = data.frame(predicted = 0.413, se = 0.02, n = 200)),
#'   class = "projoint_tau"
#' )
#' summary(toy_tau)
summary.projoint_tau <- function(object, ...) {
  irr <- object$irr
  if (is.null(irr) || !is.data.frame(irr)) {
    stop("Invalid 'projoint_tau' object: missing or invalid `irr`.", call. = FALSE)
  }
  irr
}

#' Organize data for estimation (internal helper)
#'
#' Prepares tidy inputs for MM/AMCE estimation and IRR handling.
#' Called inside \code{pj_estimate()} after reshaping to respondent–task–profile.
#'
#' @param .dataframe A tibble/data frame from \code{\link{reshape_projoint}()},
#'   containing columns like \code{id}, \code{task}, \code{profile}, \code{selected},
#'   \code{agree} (if repeated), and attribute columns named \code{att1}, \code{att2}, ...
#'   that store \code{level_id}s (e.g., \code{"att1:level2"}).
#' @param .structure Either \code{"profile_level"} or \code{"choice_level"}.
#' @param .estimand Either \code{"mm"} or \code{"amce"}.
#' @param .remove_ties Logical; if \code{TRUE} (default) remove tied responses
#'   in profile-level setups (keeps tasks where exactly one profile is selected).
#' @param .att_choose Attribute ID for the “chosen” side (e.g., \code{"att3"}).
#' @param .lev_choose Level ID(s) for the chosen side (e.g., \code{"level2"} for
#'   profile-level; vector of level IDs for choice-level).
#' @param .att_notchoose Attribute ID for the “not chosen” side (choice-level only).
#' @param .lev_notchoose Level ID(s) for the not-chosen side (choice-level only).
#'
#' @return A named list with two tibbles:
#' \itemize{
#'   \item \code{$data_for_estimand}: rows restricted and reshaped to the
#'         requested estimand/QOI. For profile-level, one row per respondent–task–profile
#'         where the target level is present (ties optionally removed). For choice-level,
#'         one row per respondent–task with paired information for profiles 1 and 2,
#'         restricted to the requested attribute-level pair(s).
#'   \item \code{$data_for_irr}: one row per respondent with columns \code{id},
#'         \code{agree} (if available), and any detected weight/cluster hints
#'         (columns matching \code{"my_weight|my_cluster|^weights?$|^clusters?$"}).
#' }
#'
#' @keywords internal
organize_data <- function(
  .dataframe,
  .structure,
  .estimand,
  .remove_ties,
  .att_choose,
  .lev_choose,
  .att_notchoose,
  .lev_notchoose
) {
  # check various settings --------------------------------------------------

  structure <- rlang::arg_match0(.structure, c("choice_level", "profile_level"))

  if ((is.null(.remove_ties) | !is.logical(.remove_ties))) {
    stop("The .remove_ties argument must be logical.")
  }

  if (structure == "profile_level") {
    if (is.null(.att_choose) | length(.att_choose) != 1) {
      stop("The .att_choose argument must be a character with the length of 1.")
    }
    if (is.null(.lev_choose) | length(.lev_choose) != 1) {
      stop("The .lev_choose argument must be a character with the length of 1.")
    }
    if (!is.null(.att_notchoose)) {
      stop("The .att_notchoose argument must be null.")
    }
    if (!is.null(.lev_notchoose)) {
      stop("The .lev_notchoose argument must be null.")
    }
  } else if (structure == "choice_level") {
    if (is.null(.att_choose) | length(.att_choose) != 1) {
      stop("The .att_choose argument must be a character with the length of 1.")
    }
    if (is.null(.lev_choose) | length(.lev_choose) < 1) {
      stop("The .lev_choose argument must be a character with the length of 1+.")
    }
    if (is.null(.att_notchoose) | length(.att_notchoose) != 1) {
      stop("The .att_notchoose argument must be a character with the length of 1.")
    }
    if (is.null(.lev_notchoose) | length(.lev_notchoose) < 1) {
      stop("The .lev_notchoose argument must be a character with the length of 1+.")
    }
  } else {
    stop("The .structure argument should be either profile_level or choice_level.")
  }


  if (structure == "profile_level") {
    # Organize data to estimate profile-level MM ------------------------------

    # specify the attributes and levels of interest
    attlev_choose <- stringr::str_c(.att_choose, ":", .lev_choose)
    att_choose <- rlang::sym(.att_choose)

    # keep relevant rows only
    out2 <- out1 <- .dataframe |>
      dplyr::mutate(qoi_choose = !!att_choose) |>
      dplyr::filter(qoi_choose %in% attlev_choose) |>
      dplyr::select(-matches("^att\\d+$"))

    if (.remove_ties == TRUE) {
      out2 <- out1 |>
        dplyr::group_by(id, task) |>
        dplyr::mutate(ties = dplyr::n() - 1) |>
        dplyr::ungroup() |>
        dplyr::filter(ties == 0) |>
        dplyr::select(-ties)
    }

    out <- out2
  } else {
    # Organize data to estimate profile-level MM ------------------------------

    # specify the attributes and levels of interest
    attlev_choose <- stringr::str_c(.att_choose, ":", .lev_choose)
    attlev_notchoose <- stringr::str_c(.att_notchoose, ":", .lev_notchoose)

    # Helper: keep only columns that are constant within (id, task) across profiles
    keep_task_constant_cols <- function(df, id_cols) {
      # If id/task missing, fall back to the input
      if (!all(c("id", "task", "profile") %in% names(df))) {
        return(id_cols)
      }

      # Candidates other than the core keys
      core <- intersect(c("id", "task", "agree"), id_cols)
      cand <- setdiff(id_cols, core)
      if (length(cand) == 0L) {
        return(core)
      }

      # A column is "safe" iff it never varies across profiles within any (id, task)
      max_ndist <- df |>
        dplyr::group_by(id, task) |>
        dplyr::summarise(
          dplyr::across(dplyr::all_of(cand), ~ dplyr::n_distinct(.x, na.rm = FALSE)),
          .groups = "drop"
        ) |>
        dplyr::summarise(dplyr::across(dplyr::everything(), ~ max(.x, na.rm = TRUE))) |>
        unlist(use.names = TRUE)

      safe <- names(max_ndist)[max_ndist <= 1]
      unique(c(core, safe))
    }


    if (.att_choose == .att_notchoose) {
      out1 <- .dataframe |>
        dplyr::mutate(qoi = !!rlang::sym(.att_choose)) |>
        dplyr::select(-matches("^att\\d+$"))

      id_cols <- setdiff(names(out1), c("profile", "selected", "selected_repeated", "qoi"))
      id_cols <- keep_task_constant_cols(out1, id_cols)

      out2 <- out1 |>
        tidyr::pivot_wider(
          # id_cols = dplyr::all_of(setdiff(names(out1), c("profile", "selected", "selected_repeated", "qoi"))),
          id_cols = dplyr::all_of(id_cols),
          names_from = profile,
          values_from = c(selected, qoi)
        )

      stopifnot(all(out2 |> dplyr::count(id, task) |> dplyr::pull(n) == 1L))

      out3 <- out2 |>
        dplyr::filter(
          (qoi_1 %in% attlev_choose & qoi_2 %in% attlev_notchoose) |
            (qoi_1 %in% attlev_notchoose & qoi_2 %in% attlev_choose)
        )
    } else {
      out1 <- .dataframe |>
        dplyr::mutate(
          qoi_choose = !!rlang::sym(.att_choose),
          qoi_notchoose = !!rlang::sym(.att_notchoose)
        ) |>
        dplyr::select(-matches("^att\\d+$"))
      # dplyr::filter(qoi_choose %in% attlev_choose & qoi_notchoose %in% attlev_notchoose)

      id_cols <- setdiff(names(out1), c("profile", "selected", "selected_repeated", "qoi_choose", "qoi_notchoose"))
      id_cols <- keep_task_constant_cols(out1, id_cols)

      out2 <- out1 |>
        tidyr::pivot_wider(
          # id_cols = dplyr::all_of(setdiff(names(out1), c("profile", "selected", "selected_repeated", "qoi_choose", "qoi_notchoose"))),
          id_cols = dplyr::all_of(id_cols),
          names_from = profile,
          values_from = c(selected, qoi_choose, qoi_notchoose)
        )

      stopifnot(all(out2 |> dplyr::count(id, task) |> dplyr::pull(n) == 1L))

      # out3 <- out2 |>
      #   dplyr::filter(
      #       (qoi_choose_1 %in% attlev_choose & qoi_notchoose_2 %in% attlev_notchoose) |
      #       (qoi_choose_1 %in% attlev_notchoose & qoi_notchoose_2 %in% attlev_choose)
      #   )

      out3 <- out2 |>
        dplyr::filter(
          (qoi_choose_1 %in% attlev_choose & qoi_notchoose_2 %in% attlev_notchoose) |
            (qoi_choose_2 %in% attlev_choose & qoi_notchoose_1 %in% attlev_notchoose)
        )
    }

    out <- out3
  }

  # Keep necessary variables only and return --------------------------------

  # data frame to estimate IRR
  # data1 <- .dataframe |>
  #   dplyr::select(id, agree) |>
  #   dplyr::filter(!is.na(agree)) |>
  #   dplyr::distinct() |>
  #   tibble::as_tibble()

  # data1 <- .dataframe |>
  #   dplyr::filter(!is.na(agree)) |>
  #   dplyr::group_by(id) |>
  #   dplyr::slice(1) |> # take first occurrence per respondent
  #   dplyr::ungroup() |>
  #   # dplyr::select(id, agree, matches("my_weight|my_cluster|^weights?$|^clusters?$")) |>
  #   dplyr::select(id, agree, matches("my_weight|my_cluster|^weight(s)?$|^cluster(s)?$|^weights?$|^clusters?$")) |>
  #   tibble::as_tibble()

  data1 <- .dataframe |>
    dplyr::filter(!is.na(agree)) |>
    dplyr::group_by(id) |>
    dplyr::slice(1) |>
    dplyr::ungroup() |>
    dplyr::select(-matches("^att\\d+$")) |>
    tibble::as_tibble()

  # data frame to estimate MM or AMCE
  data2 <- out

  return(
    list(
      "data_for_irr" = data1,
      "data_for_estimand" = data2
    )
  )
}

#' Example Output: Manually Rearranged Labels
#'
#' A toy \code{\link{projoint_data}} object showing what happens after labels
#' have been manually rearranged (e.g., via \code{\link{save_labels}} /
#' \code{\link{read_labels}}).
#'
#' Useful for illustrating workflows without requiring users to re-run the
#' reordering themselves.
#'
#' @docType data
#' @usage data(out1_arranged)
#' @format An object of class \code{projoint_data}.
#'   Contains two elements:
#'   \itemize{
#'     \item \code{$labels}: reordered attribute–level mapping
#'     \item \code{$data}: the corresponding profile-level dataset
#'   }
#' @keywords datasets
#' @name out1_arranged
NULL

#' Read and re-format a Qualtrics CSV (choice text)
#'
#' Reads a CSV file exported from Qualtrics (with "Use choice text" enabled) and
#' returns a data frame formatted for downstream processing with
#' \code{\link{reshape_projoint}}.
#'
#' @param .file A character string giving the path to a Qualtrics CSV file.
#' @param .metadata_rows Number of metadata rows after the CSV header. The
#'   default, \code{NULL}, detects current Qualtrics exports (question text plus
#'   an \code{ImportId} row) and legacy exports (question text only). Supply a
#'   non-negative integer to override detection.
#'
#' @return A tibble containing response rows only. Column names and physical
#'   column order are preserved from the Qualtrics export.
#'
#' @seealso \code{\link{reshape_projoint}}
#'
#' @examples
#' \donttest{
#' # Write a tiny dummy Qualtrics-style CSV to a temp file
#' tmp <- tempfile(fileext = ".csv")
#' readr::write_csv(
#'   data.frame(Q1 = c("Choice Text", "{\"ImportId\":\"QID1\"}", "A", "B")),
#'   tmp
#' )
#' # Read it back in
#' df <- read_Qualtrics(tmp)
#' head(df)
#' }
#'
#' @export
read_Qualtrics <- function(.file, .metadata_rows = NULL) {
  if (!is.character(.file) || length(.file) != 1L || is.na(.file)) {
    stop("`.file` must be a single, non-missing character string.", call. = FALSE)
  }

  if (!file.exists(.file)) {
    stop("The specified file cannot be found.", call. = FALSE)
  }

  if (!is.null(.metadata_rows)) {
    valid_metadata_rows <-
      is.numeric(.metadata_rows) &&
      length(.metadata_rows) == 1L &&
      !is.na(.metadata_rows) &&
      .metadata_rows >= 0 &&
      .metadata_rows == as.integer(.metadata_rows)

    if (!valid_metadata_rows) {
      stop("`.metadata_rows` must be NULL or one non-negative integer.", call. = FALSE)
    }
  }

  column_names <- readr::read_csv(
    .file,
    n_max = 0,
    name_repair = "minimal",
    show_col_types = FALSE
  ) |>
    names()

  if (is.null(.metadata_rows)) {
    metadata <- readr::read_csv(
      .file,
      n_max = 2,
      col_types = readr::cols(.default = readr::col_character()),
      name_repair = "minimal",
      show_col_types = FALSE
    )

    second_row <- if (nrow(metadata) >= 2L) {
      unlist(metadata[2, ], use.names = FALSE)
    } else {
      character()
    }

    has_import_id_row <- any(grepl('"ImportId"', second_row, fixed = TRUE), na.rm = TRUE)
    .metadata_rows <- if (has_import_id_row) 2L else 1L
  }

  readr::read_csv(
    .file,
    skip = 1L + as.integer(.metadata_rows),
    col_names = column_names,
    name_repair = "minimal",
    show_col_types = FALSE
  )
}

write_qualtrics_fixture <- function(lines) {
  path <- tempfile(fileext = ".csv")
  writeLines(lines, path)
  path
}

test_that("read_Qualtrics reads current exports and preserves column order", {
  path <- write_qualtrics_fixture(c(
    "Q2,Q1,ResponseId",
    "Second choice,First choice,Response ID",
    '"{""ImportId"":""QID2""}","{""ImportId"":""QID1""}","{""ImportId"":""_recordId""}"',
    "Candidate B,Candidate A,R_1",
    "Candidate A,Candidate B,R_2"
  ))

  result <- read_Qualtrics(path)

  expect_named(result, c("Q2", "Q1", "ResponseId"), ignore.order = FALSE)
  expect_equal(nrow(result), 2L)
  expect_equal(result$Q2, c("Candidate B", "Candidate A"))
})

test_that("read_Qualtrics reads legacy exports", {
  path <- write_qualtrics_fixture(c(
    "ResponseId,Q1",
    "Response ID,First choice",
    "R_1,Candidate A",
    "R_2,Candidate B"
  ))

  result <- read_Qualtrics(path)

  expect_equal(nrow(result), 2L)
  expect_equal(result$ResponseId, c("R_1", "R_2"))
})

test_that("read_Qualtrics permits an explicit metadata-row override", {
  path <- write_qualtrics_fixture(c(
    "ResponseId,Q1",
    "R_1,Candidate A",
    "R_2,Candidate B"
  ))

  result <- read_Qualtrics(path, .metadata_rows = 0)

  expect_equal(nrow(result), 2L)
  expect_equal(result$Q1, c("Candidate A", "Candidate B"))
})

test_that("read_Qualtrics rejects invalid inputs", {
  expect_error(read_Qualtrics(character()), "single, non-missing character")
  expect_error(read_Qualtrics(tempfile()), "cannot be found")

  path <- write_qualtrics_fixture(c("Q1", "Question", "A"))
  expect_error(read_Qualtrics(path, .metadata_rows = -1), "non-negative integer")
  expect_error(read_Qualtrics(path, .metadata_rows = 1.5), "non-negative integer")
})

#' Estimate profile- or choice-level effects (internal)
#'
#' Core worker used by \code{\link{projoint}} to compute marginal means (MMs)
#' or average marginal component effects (AMCEs) under either the profile- or
#' choice-level structure. Handles IRR usage (estimated or fixed) and the
#' requested standard-error method.
#'
#' @param .data A \code{\link{projoint_data}} object.
#' @param .qoi Optional \code{\link{projoint_qoi}}; if \code{NULL}, estimates all
#'   MMs/AMCEs implied by the design. When supplied, overrides \code{.structure}
#'   and \code{.estimand}.
#' @param .structure Either \code{"profile_level"} or \code{"choice_level"}.
#' @param .estimand Either \code{"mm"} (marginal mean) or \code{"amce"} (average
#'   marginal component effect).
#' @param .se_method One of \code{"analytical"}, \code{"simulation"}, or
#'   \code{"bootstrap"}.
#' @param .irr \code{NULL} to estimate IRR from repeated tasks; numeric to fix IRR.
#' @param .remove_ties Logical; whether to drop tied responses (default \code{TRUE}).
#' @param .ignore_position Logical; choice-level only. Ignore left/right position?
#'   Default \code{TRUE}.
#' @param .n_sims Integer; required when \code{.se_method = "simulation"}.
#' @param .n_boot Integer; required when \code{.se_method = "bootstrap"}.
#' @param .weights_1,.clusters_1,.se_type_1 Arguments passed to
#'   \code{\link[estimatr]{lm_robust}} for IRR estimation. If \code{.se_type_1}
#'   is \code{NULL}, \emph{estimatr} defaults are used.
#' @param .weights_2,.clusters_2,.se_type_2 Arguments passed to
#'   \code{\link[estimatr]{lm_robust}} for MM/AMCE estimation. If \code{.se_type_2}
#'   is \code{NULL}, \emph{estimatr} defaults are used.
#' @param .auto_cluster Logical; if \code{TRUE} (default) and an \code{id} column
#'   is present while no clusters are provided, cluster automatically. Only applied
#'   when the corresponding \code{.se_type_*} is \code{NULL}.
#' @param .seed Optional integer; if supplied, sets a temporary RNG seed for
#'   simulation/bootstrap and restores prior state on exit.
#'
#' @return A \code{\link{projoint_results}} object containing:
#' \itemize{
#'   \item \code{estimates}: tibble of estimates (point estimate, SE, CI) with
#'         identifier columns (e.g., \code{att_level_*}).
#'   \item \code{estimand}: \code{"mm"} or \code{"amce"}.
#'   \item \code{structure}: \code{"profile_level"} or \code{"choice_level"}.
#'   \item \code{se_method}: SE computation method used.
#'   \item \code{irr}: character noting IRR usage (e.g., \code{"Estimated"} or
#'         \code{"Assumed (<value> )"}).
#'   \item \code{tau}: numeric \eqn{\tau} used to correct measurement error.
#'   \item \code{remove_ties}, \code{ignore_position}: flags echoed from inputs.
#'   \item \code{se_type_used}, \code{cluster_by}: details propagated from
#'         fitting calls (if available).
#'   \item \code{labels}, \code{data}: design labels and the analysis data
#'         passed through for downstream methods.
#' }
#'
#' @keywords internal
#' @seealso \code{\link{projoint}}, \code{\link{pj_estimate}}, \code{\link{organize_data}},
#'   \code{\link{projoint_results}}
projoint_level <- function(
    .data,
    .qoi,
    .structure,
    .estimand,
    .se_method,
    .irr,
    .remove_ties,
    .ignore_position,
    .n_sims,
    .n_boot,
    .weights_1,
    .clusters_1,
    .se_type_1,
    .weights_2,
    .clusters_2,
    .se_type_2,
    .auto_cluster = TRUE,
    .seed = NULL
){
  
  # check various settings --------------------------------------------------
  # Also see the checking in pj_estimate()
  
  structure <- rlang::arg_match0(.structure, c("choice_level", "profile_level"))
  estimand  <- rlang::arg_match0(.estimand, c("mm", "amce"))
  se_method <- rlang::arg_match0(.se_method, c("analytical", "simulation", "bootstrap"))
  
  if (!is(.data, "projoint_data")){
    stop("The .data argument must be of class `projoint_data` from the `reshape_projoint` function.")
  }
  
  if (!is.null(.qoi) && !is(.qoi, "projoint_qoi")){
    stop("The .qoi argument must be of class `projoint_qoi` from the `set_qoi` function.")
  }
  
  if (.se_method == "simulation" && is.null(.n_sims)) {
    stop("If SEs are calculated by simulation, .n_sims must be specified (not NULL).")
  }
  
  if (.se_method == "bootstrap" && is.null(.n_boot)) {
    stop("If SEs are calculated by bootstrap, .n_boot must be specified (not NULL).")
  }
  
  if (is.null(.qoi) && structure == "choice_level") {
    stop("The .qoi argument must be specified for choice-level analysis.") 
  }
  
  # ---- Optional reproducible RNG (wrapper) -----------------------------------
  if (!is.null(.seed)) {
    if (exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) {
      old_seed <- .Random.seed
      on.exit({
        if (exists("old_seed", inherits = FALSE)) .Random.seed <<- old_seed
      }, add = TRUE)
    }
    set.seed(.seed)
  }
  
  # estimate all MMs or AMCEs -----------------------------------------------
  
  if (is.null(.qoi)){
    
    attribute_levels <- .data$labels$level_id
    
    out <- NULL
    se_type_used <- NULL
    cluster_by   <- NULL

    for (i in seq_along(attribute_levels)){
      
      attribute <- stringr::str_extract(attribute_levels[i], "^.+(?=:)")
      level     <- stringr::str_extract(attribute_levels[i], "(?<=:).+$")
      
      if (estimand == "mm"){
        
        temp1 <- pj_estimate(.data = .data,
                             
                             .structure = structure,
                             .estimand = estimand,
                             
                             .att_choose = attribute,
                             .lev_choose = level,
                             .att_notchoose = NULL, 
                             .lev_notchoose = NULL,
                             .att_choose_b = NULL, 
                             .lev_choose_b = NULL,
                             .att_notchoose_b = NULL, 
                             .lev_notchoose_b = NULL,
                             
                             .se_method = .se_method,
                             .irr = .irr,
                             .remove_ties = .remove_ties,
                             .ignore_position = .ignore_position,
                             .n_sims = .n_sims,
                             .n_boot = .n_boot,
                             .weights_1 = .weights_1,
                             .clusters_1 = .clusters_1,
                             .se_type_1 = .se_type_1,
                             .weights_2 = .weights_2,
                             .clusters_2 = .clusters_2,
                             .se_type_2 = .se_type_2,
                             .auto_cluster = .auto_cluster) |> 
          dplyr::mutate(att_level_choose = stringr::str_c(stringr::str_c(attribute, level, sep = ":"), collapse = " or "))
        
      } else {
        
        temp1 <- pj_estimate(.data = .data,
                             .structure = structure,
                             .estimand = estimand,
                             
                             .att_choose = attribute,
                             .lev_choose = level,
                             .att_notchoose = NULL, 
                             .lev_notchoose = NULL,
                             .att_choose_b = attribute, 
                             .lev_choose_b = "level1", # The default baseline is "level1"
                             .att_notchoose_b = NULL, 
                             .lev_notchoose_b = NULL,
                             
                             .se_method = .se_method,
                             .irr = .irr,
                             .remove_ties = .remove_ties,
                             .ignore_position = .ignore_position,
                             .n_sims = .n_sims,
                             .n_boot = .n_boot,
                             .weights_1 = .weights_1,
                             .clusters_1 = .clusters_1,
                             .se_type_1 = .se_type_1,
                             .weights_2 = .weights_2,
                             .clusters_2 = .clusters_2,
                             .se_type_2 = .se_type_2,
                             .auto_cluster = .auto_cluster) |> 
          dplyr::mutate(att_level_choose = stringr::str_c(stringr::str_c(attribute, level, sep = ":"), collapse = " or "),
                        att_level_choose_baseline = stringr::str_c(stringr::str_c(attribute, "level1", sep = ":"), collapse = " or ")
          )
        
      }
      
      # capture once, from the first call
      if (is.null(se_type_used)) {
        se_type_used <- attr(temp1, "se_type_used", exact = TRUE)
        cluster_by   <- attr(temp1, "cluster_by",   exact = TRUE)
      }
      
      out <- dplyr::bind_rows(out, temp1)
      
    }
    
    if (estimand == "amce"){
      
      out <- out |> 
        dplyr::filter(att_level_choose != att_level_choose_baseline)
      
    }
    
  } else{
    
    attribute_of_interest  <- .qoi$attribute_of_interest
    levels_of_interest     <- .qoi$levels_of_interest
    
    attribute_of_interest_0  <- .qoi$attribute_of_interest_0
    levels_of_interest_0     <- .qoi$levels_of_interest_0
    
    attribute_of_interest_baseline <- .qoi$attribute_of_interest_baseline
    levels_of_interest_baseline     <- .qoi$levels_of_interest_baseline
    
    attribute_of_interest_0_baseline <- .qoi$attribute_of_interest_0_baseline
    levels_of_interest_0_baseline     <- .qoi$levels_of_interest_0_baseline
    
    temp <- pj_estimate(.data = .data,
                        .structure = structure,
                        .estimand = estimand,
                        
                        .att_choose = attribute_of_interest,
                        .lev_choose = levels_of_interest,
                        .att_notchoose = attribute_of_interest_0, 
                        .lev_notchoose = levels_of_interest_0,
                        .att_choose_b = attribute_of_interest_baseline, 
                        .lev_choose_b = levels_of_interest_baseline,
                        .att_notchoose_b = attribute_of_interest_0_baseline, 
                        .lev_notchoose_b = levels_of_interest_0_baseline,
                        
                        .se_method = .se_method,
                        .irr = .irr,
                        .remove_ties = .remove_ties,
                        .ignore_position = .ignore_position,
                        .n_sims = .n_sims,
                        .n_boot = .n_boot,
                        .weights_1 = .weights_1,
                        .clusters_1 = .clusters_1,
                        .se_type_1 = .se_type_1,
                        .weights_2 = .weights_2,
                        .clusters_2 = .clusters_2,
                        .se_type_2 = .se_type_2,
                        .auto_cluster = .auto_cluster)
    
    se_type_used <- attr(temp, "se_type_used", exact = TRUE)
    cluster_by   <- attr(temp, "cluster_by",   exact = TRUE)
    
    if (estimand == "mm"){
      
      out <- temp |> 
        dplyr::mutate(att_level_choose = stringr::str_c(stringr::str_c(attribute_of_interest, levels_of_interest, sep = ":"), collapse = " or "),
                      att_level_notchoose = stringr::str_c(stringr::str_c(attribute_of_interest_0, levels_of_interest_0, sep = ":"), collapse = " or "))
      
    } else{
      
      out <- temp |> 
        dplyr::mutate(att_level_choose = stringr::str_c(stringr::str_c(attribute_of_interest, levels_of_interest, sep = ":"), collapse = " or "),
                      att_level_notchoose = stringr::str_c(stringr::str_c(attribute_of_interest_0, levels_of_interest_0, sep = ":"), collapse = " or "),
                      att_level_choose_baseline = stringr::str_c(stringr::str_c(attribute_of_interest_baseline, levels_of_interest_baseline, sep = ":"), collapse = " or "),
                      att_level_notchoose_baseline = stringr::str_c(stringr::str_c(attribute_of_interest_0_baseline, levels_of_interest_0_baseline, sep = ":"), collapse = " or "))
      
    }
    
  }
  
  
  # Pull through meta from pj_estimate() result
  # se_type_used <- attr(out, "se_type_used", exact = TRUE)
  # cluster_by   <- attr(out, "cluster_by",   exact = TRUE)
  
  tau <- unique(out$tau)
  if (length(tau) != 1L) tau <- tau[1L]
  
  estimates <- out |> 
    dplyr::select(-tau) |> 
    dplyr::as_tibble()
  
  # return estimates --------------------------------------------------------
  
  if (!is.null(.irr)){
    irr <- stringr::str_c("Assumed (", .irr, ")")
  } else{
    irr <- "Estimated" 
  }
  
  if (.estimand == "mm"){
    
    if(is.null(.qoi)){
      return(
        projoint_results("estimand" = .estimand,
                         "structure" = .structure,
                         "estimates" = estimates, 
                         "se_method" = .se_method,
                         "irr" = irr,
                         "tau" = tau,
                         "remove_ties" = .remove_ties,
                         "ignore_position" = .ignore_position,
                         "attribute_of_interest" = "all",
                         "levels_of_interest" = "all",
                         "attribute_of_interest_0" = NULL,
                         "levels_of_interest_0" = NULL,
                         "attribute_of_interest_baseline" = NULL,
                         "levels_of_interest_baseline" = NULL,
                         "attribute_of_interest_0_baseline" = NULL,
                         "levels_of_interest_0_baseline" = NULL,
                         "se_type_used" = se_type_used,
                         "cluster_by"   = cluster_by,
                         labels = .data$labels,
                         data = .data$data)
      )
    } else {
      return(projoint_results("estimand" = .estimand,
                              "structure" = .structure,
                              "estimates" = estimates, 
                              "se_method" = .se_method,
                              "irr" = irr,
                              "tau" = tau,
                              "remove_ties" = .remove_ties,
                              "ignore_position" = .ignore_position,
                              "attribute_of_interest" = .qoi$attribute_of_interest,
                              "levels_of_interest" = .qoi$levels_of_interest,
                              "attribute_of_interest_0" = .qoi$attribute_of_interest_0,
                              "levels_of_interest_0" = .qoi$levels_of_interest_0,
                              "attribute_of_interest_baseline" = NULL,
                              "levels_of_interest_baseline" = NULL,
                              "attribute_of_interest_0_baseline" = NULL,
                              "levels_of_interest_0_baseline" = NULL,
                              "se_type_used" = se_type_used,
                              "cluster_by"   = cluster_by,
                              labels = .data$labels,
                              data = .data$data)
      )
    }
    
  } else {
    
    if(is.null(.qoi)){
      return(projoint_results("estimand" = .estimand,
                              "structure" = .structure,
                              "estimates" = estimates, 
                              "se_method" = .se_method,
                              "irr" = irr,
                              "tau" = tau,
                              "remove_ties" = .remove_ties,
                              "ignore_position" = .ignore_position,
                              "attribute_of_interest" = "all",
                              "levels_of_interest" = "all except level1",
                              "attribute_of_interest_0" = NULL,
                              "levels_of_interest_0" = NULL,
                              "attribute_of_interest_baseline" = "all",
                              "levels_of_interest_baseline" = "level1",
                              "attribute_of_interest_0_baseline" = NULL,
                              "levels_of_interest_0_baseline" = NULL,
                              "se_type_used" = se_type_used,
                              "cluster_by"   = cluster_by,
                              labels = .data$labels,
                              data = .data$data)
      )
      
    } else {
      return(projoint_results("estimand" = .estimand,
                              "structure" = .structure,
                              "estimates" = estimates, 
                              "se_method" = .se_method,
                              "irr" = irr,
                              "tau" = tau,
                              "remove_ties" = .remove_ties,
                              "ignore_position" = .ignore_position,
                              "attribute_of_interest" = .qoi$attribute_of_interest,
                              "levels_of_interest" = .qoi$levels_of_interest,
                              "attribute_of_interest_0" = .qoi$attribute_of_interest_0,
                              "levels_of_interest_0" = .qoi$levels_of_interest_0,
                              "attribute_of_interest_baseline" = .qoi$attribute_of_interest_baseline,
                              "levels_of_interest_baseline" = .qoi$levels_of_interest_baseline,
                              "attribute_of_interest_0_baseline" = .qoi$attribute_of_interest_0_baseline,
                              "levels_of_interest_0_baseline" = .qoi$levels_of_interest_0_baseline,
                              "se_type_used" = se_type_used,
                              "cluster_by"   = cluster_by,
                              labels = .data$labels,
                              data = .data$data)
      )
    }
    
  }
  
}

#' Reshapes survey response data for conjoint analysis
#'
#' This function takes a data frame, preferably from \dQuote{read_Qualtrics()}, and reshapes it from wide to long such that each row is a distinct conjoint task rather than a respondent.
#'
#' @import dplyr
#' @import tidyr
#' @import stringr
#' @import rlang
#' @param .data A data frame, preferably from \dQuote{read_Qualtrics()}
#' @param .idvar A character identifying the column name containing respondent IDs
#' @param .outcomes A character vector identifying the column names that contain outcomes
#' @param .alphabet The letter indicating conjoint attributes. If using Strezhnev's package (https://github.com/astrezhnev/conjointsdt) in Qualtrics, the default is "F".
#' @param .flipped TRUE if the profiles of the repeated task are flipped (recommended)
#' @return A conjoint task-level data frame (in other words, in long format) ready for conjoint analysis. See \dQuote{pj}.
#' @export
#' @examples
#' library(projoint)
#' 
#' data("exampleData1")
#' head(exampleData1)
#' reshaped_data = reshape_conjoint(exampleData1, .idvar = "ResponseId",
#'                                  .outcomes = c("Q4.1", "Q5.1", "Q6.1", "Q7.1", "Q8.1","Q9.1"),
#'                                  .alphabet ="A")

reshape_conjoint <- function(.data, .idvar, .outcomes, .alphabet = "F", .flipped = TRUE)
{

  . <- code <- x <- name <- attribute <- attribute_name <- NULL
  level_name <- task <- outcome_qnum <- outcomes <- NULL
  profile <- response <- selected <- selected_repeated <-  NULL

  idvar_quo <- rlang::enquo(.idvar)
  n_tasks <- length(.outcomes)

  # Remove if a conjoint table is empty -------------------------------------

  alphabet11 <- paste0(.alphabet, "-1-1") %>% rlang::sym()

  df <- .data %>%
    mutate(id = !!idvar_quo) %>%
    filter(!is.na(!!alphabet11))

  temp1 <- df %>%
    select(id, contains(paste0(.alphabet, "-"))) %>%
    pivot_longer(names_to = "code", values_to = "name", cols = 2:ncol(.)) %>%
    filter(str_detect(code, paste0(.alphabet, "-\\d+-\\d+$"))) %>%
    separate(code, into = c("x", "task", "attribute"), sep = "\\-") %>%
    select(-x) %>%
    rename(attribute_name = name)

  temp2 <- df %>%
    select(id, contains(paste0(.alphabet, "-"))) %>%
    pivot_longer(names_to = "code", values_to = "name", cols = 2:ncol(.)) %>%
    filter(str_detect(code, paste0(.alphabet, "-\\d+-\\d+-\\d+"))) %>%
    separate(code, into = c("x", "task", "profile", "attribute"), sep = "\\-") %>%
    select(-x) %>%
    rename(level_name = name)

  attribute_levels_long <- left_join(temp1, temp2,
                                     by = c("id", "task", "attribute")) %>%
    select(-attribute) %>%
    mutate_at(c("task", "profile"), .funs = as.numeric) %>%
    rename(attribute = attribute_name,
           level = level_name) %>%
    filter(task <= n_tasks)

  # Make a list of attributes and levels, as well as their IDs
  labels <- attribute_levels_long %>%
    arrange(attribute, level) %>%
    select(attribute, level) %>%
    distinct() %>%
    group_by(attribute) %>%
    mutate(attribute_id = cur_group_id(),
           level_id = row_number()) %>%
    ungroup() %>%
    mutate(attribute_id = str_c("att", attribute_id),
           level_id = str_c(attribute_id, ":level", level_id))

  attribute_levels_wide <- attribute_levels_long %>%
    left_join(labels) %>%
    select(-attribute, -level) %>%
    rename(attribute = attribute_id,
           level = level_id) %>%
    group_by(id, task) %>%
    pivot_wider(names_from = "attribute", values_from = "level") %>%
    ungroup()

  attribute_levels_repeated <- attribute_levels_wide %>%
    filter(task == 1)

  # Response variable
  responses <- df %>%
    select(id, all_of(.outcomes)) %>%
    pivot_longer(names_to = "outcome_qnum", values_to = "response", cols = 2:ncol(.)) %>%
    mutate(task = NA)

  for (i in 1:n_tasks) {
    responses <- responses %>%
      mutate(task = ifelse(outcome_qnum == .outcomes[i], i, task))
  }

  response_cleaned <- responses %>%
    mutate(selected = str_extract(response, ".$"),
           selected = case_when(selected %in% c("1", "A") ~ 1,
                                selected %in% c("2", "B") ~ 2)) %>%
    select(-response, -outcome_qnum)

  # Tasks to estimate AMCEs/MMs
  out1 <- attribute_levels_wide %>%
    left_join(response_cleaned %>%
                filter(task != n_tasks),
              by = c("id", "task")) %>%
    mutate(selected = ifelse(profile == selected, 1, 0))

  # Tasks to estimate ICR
  out2 <- attribute_levels_repeated %>%
    left_join(response_cleaned %>%
                filter(task == n_tasks) %>%
                mutate(task = 1),
              by = c("id", "task")) %>%
    mutate(selected = case_when(profile == selected & .flipped == FALSE ~ 1,
                                profile == selected & .flipped == TRUE  ~ 0,
                                profile != selected & .flipped == FALSE ~ 0,
                                profile != selected & .flipped == TRUE  ~ 1)) %>%
    rename(selected_repeated = selected)

  # Merge
  out_final <- left_join(out1, out2) %>%
    mutate_if(is.character, as.factor) %>%
    mutate(id = as.character(id)) %>%
    as.data.frame()

  # Return
  list(labels, out_final)

}

#' Analyze a conjoint dataset with measurement-error correction
#'
#' Computes marginal means (MMs) or average marginal component effects (AMCEs)
#' with correction for intra-respondent reliability (IRR, \eqn{\tau}). When a
#' repeated task is present, IRR is estimated unless a fixed value is supplied.
#' Results are returned in a structured object ready for plotting and summary.
#'
#' Most users will pass a \code{\link{projoint_data}} object (from
#' \code{\link{reshape_projoint}} or \code{\link{make_projoint_data}}). Advanced
#' users may specify custom quantities via \code{\link{projoint_qoi}}; if provided,
#' its \code{structure} and \code{estimand} override \code{.structure} and
#' \code{.estimand}.
#'
#' @import dplyr
#' @import rlang
#' @import estimatr
#' @importFrom MASS mvrnorm
#' @importFrom methods is
#'
#' @param .data A \code{\link{projoint_data}} created by
#'   \code{\link{reshape_projoint}} or \code{\link{make_projoint_data}}.
#' @param .qoi Optional \code{\link{projoint_qoi}} describing the quantity of
#'   interest. If supplied, its fields override \code{.structure} and \code{.estimand}.
#' @param .by_var Optional column name (character) for subgroup analysis; must be
#'   logical (TRUE/FALSE) or numeric/integer coded as 0/1. Only supported for
#'   \code{.structure == "profile_level"} (ignored otherwise).
#' @param .structure Either \code{"profile_level"} or \code{"choice_level"}
#'   (default \code{"choice_level"}). Overridden by \code{.qoi$structure} if present.
#' @param .estimand Either \code{"mm"} (marginal mean) or \code{"amce"} (average
#'   marginal component effect). Default \code{"mm"}. Overridden by \code{.qoi$estimand} if present.
#' @param .se_method Standard-error method: \code{"analytical"} (default),
#'   \code{"simulation"}, or \code{"bootstrap"}.
#' @param .irr Numeric or \code{NULL}. If \code{NULL} (default), IRR is estimated
#'   (when design allows); otherwise a fixed IRR value is used and IRR estimation
#'   is skipped.
#' @param .remove_ties Logical; remove ties in choice data before estimation?
#'   Default \code{TRUE}.
#' @param .ignore_position Logical; choice-level only. Ignore profile position
#'   (left/right)? Default \code{TRUE}.
#' @param .n_sims Integer; required when \code{.se_method = "simulation"}.
#' @param .n_boot Integer; required when \code{.se_method = "bootstrap"}.
#' @param .weights_1,.clusters_1,.se_type_1 Passed to
#'   \code{\link[estimatr]{lm_robust}} when estimating IRR.
#' @param .weights_2,.clusters_2,.se_type_2 Passed to
#'   \code{\link[estimatr]{lm_robust}} when estimating MMs/AMCEs.
#' @param .auto_cluster Logical. If \code{TRUE} (default), automatically cluster on
#'   an \code{id} column when present and no \code{.clusters_*} are supplied. Auto-clustering only
#'   occurs when the corresponding \code{.se_type_*} is \code{NULL}.
#' @param .seed Optional integer. Sets a temporary RNG seed for reproducible
#'   simulation/bootstrap inside the call; restores the previous RNG state on exit.
#'
#' @details
#' Valid \code{se_type_*} values depend on clustering:
#' \itemize{
#'   \item Without clusters: \code{"classical"}, \code{"HC0"}, \code{"HC1"}, \code{"HC2"}, \code{"HC3"}
#'   \item With clusters: \code{"CR0"}, \code{"CR1"}, \code{"CR2"}, \code{"stata"}, \code{"none"}
#' }
#' If \code{NULL}, \emph{estimatr} defaults are used (\code{HC2} when unclustered;
#' \code{CR2} when clustered).
#'
#' @return A \code{projoint_results} object (list-like) with components such as:
#' \itemize{
#'   \item \code{$estimand}, \code{$structure}, \code{$se_method}, \code{$irr}, \code{$tau}
#'   \item \code{$labels}: attribute/level mapping used in estimation
#'   \item \code{$estimates}: a data frame of estimates with columns like
#'         \code{att_level_choose}, \code{att_level_notchoose} (if choice-level),
#'         \code{estimate}, \code{se} / \code{std.error}, \code{conf.low}, \code{conf.high},
#'         and an \code{estimand} label such as \code{"mm_corrected"} or \code{"amce_uncorrected"}.
#' }
#' This object is suitable for downstream use in \code{\link{plot.projoint_results}},
#' \code{\link{summary.projoint_results}}, and related helpers.
#'
#' @seealso \code{\link{reshape_projoint}}, \code{\link{projoint_qoi}},
#'   \code{\link{plot.projoint_results}}, \code{\link{summary.projoint_results}}
#'
#' @examples
#' \donttest{
#' # Prepare example data
#' data(exampleData1)
#' outcomes <- c(paste0("choice", 1:8), "choice1_repeated_flipped")
#' pj <- reshape_projoint(exampleData1, outcomes)
#'
#' # Choice-level QoI based on pj$labels
#' att <- unique(pj$labels$attribute_id)[1]
#' lev_ids   <- pj$labels$level_id[pj$labels$attribute_id == att]
#' lev_names <- sub(".*:", "", lev_ids)
#'
#' q <- set_qoi(
#'     .structure     = "choice_level",
#'     .estimand      = "mm",
#'     .att_choose    = att,
#'     .lev_choose    = lev_names[2],
#'     .att_notchoose = att,
#'     .lev_notchoose = lev_names[1]
#'   )
#'   
#' # Choice-level, marginal means (fast example: fix IRR)
#' fit_choice <- projoint(
#'   pj,
#'   .qoi = q,
#'   .structure = "choice_level",
#'   .estimand  = "mm",
#'   .irr       = 0.80,        # skip IRR estimation for a quick example
#'   .se_method = "analytical"
#' )
#' head(summary(fit_choice))
#'
#' # Profile-level AMCEs
#' fit_profile <- projoint(
#'   pj,
#'   .structure = "profile_level",
#'   .estimand  = "amce",
#'   .se_method = "analytical"
#' )
#' # Plot using the S3 plot method
#' p <- plot(fit_profile, .estimates = "both")
#' print(p)
#' }
#'
#' @export

projoint <- function(
    .data,
    .qoi = NULL,
    .by_var = NULL,
    .structure = "choice_level",
    .estimand = "mm",
    .se_method = "analytical",
    .irr = NULL,
    .remove_ties = TRUE,
    .ignore_position = NULL,
    .n_sims = NULL,
    .n_boot = NULL,
    .weights_1 = NULL,
    .clusters_1 = NULL,
    .se_type_1 = NULL,
    .weights_2 = NULL,
    .clusters_2 = NULL,
    .se_type_2 = NULL,
    .auto_cluster = TRUE,
    .seed = NULL
) {
  
  if (!is.null(.structure) && !is.null(.qoi)) {
    .structure <- .qoi$structure
  }
  if (!is.null(.estimand) && !is.null(.qoi)) {
    .estimand <- .qoi$estimand
  }
  if (.structure == "choice_level" && is.null(.ignore_position)) {
    .ignore_position <- TRUE
  }

  if (!is.null(.by_var) && .structure != "profile_level") {
    stop(".by_var is only supported for profile-level analysis.")
  }
  
  # ---- Optional reproducible RNG (wrapper) -----------------------------------
  if (!is.null(.seed)) {
    if (exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) {
      old_seed <- .Random.seed
      on.exit({
        if (exists("old_seed", inherits = FALSE)) .Random.seed <<- old_seed
      }, add = TRUE)
    }
    set.seed(.seed)
  }

  weights1_quo  <- rlang::enquo(.weights_1)
  clusters1_quo <- rlang::enquo(.clusters_1)
  weights2_quo  <- rlang::enquo(.weights_2)
  clusters2_quo <- rlang::enquo(.clusters_2)
  
  if (is.null(.by_var)) {
    out <- projoint_level(
      .data = .data, 
      .qoi = .qoi, 
      .structure = .structure, 
      .estimand = .estimand, 
      .se_method = .se_method, 
      .irr = .irr, 
      .remove_ties = .remove_ties,
      .ignore_position = .ignore_position, 
      .n_sims = .n_sims, 
      .n_boot = .n_boot,
      .weights_1 = weights1_quo, 
      .clusters_1 = clusters1_quo, 
      .se_type_1 = .se_type_1,
      .weights_2 = weights2_quo, 
      .clusters_2 = clusters2_quo, 
      .se_type_2 = .se_type_2,
      .auto_cluster = .auto_cluster
    )
    
  } else {
    out <- projoint_diff(
      .data = .data, 
      .qoi = .qoi, 
      .by_var = .by_var, 
      .structure = .structure, 
      .estimand = .estimand, 
      .se_method = .se_method, 
      .irr = .irr, 
      .remove_ties = .remove_ties,
      .ignore_position = .ignore_position, 
      .n_sims = .n_sims, 
      .n_boot = .n_boot,
      .weights_1 = weights1_quo, 
      .clusters_1 = clusters1_quo, 
      .se_type_1 = .se_type_1,
      .weights_2 = weights2_quo, 
      .clusters_2 = clusters2_quo, 
      .se_type_2 = .se_type_2,
      .auto_cluster = .auto_cluster
    )
  }
  
  return(out)
}

test_that("profile-level MM analytical runs and returns sane output (warning not required)", {
  data(exampleData1, package = "projoint")
  pj <- reshape_projoint(exampleData1, c(paste0("choice", 1:8), "choice1_repeated_flipped"))
  
  expect_no_error({
    fit <- projoint(
      pj,
      .structure = "profile_level",
      .estimand  = "mm",
      .se_method = "analytical"
    )
    # swallow printed banner so the test runner stays quiet
    invisible(capture.output(summary(fit)))
  })
  
  sm <- summary(fit)
  expect_true(all(is.finite(sm$estimate)))
  expect_true(all(is.finite(sm$se) | is.na(sm$se)))
  used <- attr(sm, "se_type_used")
  expect_true(used %in% c("CR2","stata","HC1"))
})

#' @importFrom stats cov
#' @importFrom stats sd
#' @importFrom ggplot2 %+replace%

if (getRversion() >= "2.15.1") {
  utils::globalVariables(c(
    ".", 
    "1", 
    "2", 
    "aes",
    "agree",
    "att",
    "att_1",
    "att_2",
    "att_comb",
    "att_level",
    "att_level_choose",
    "att_level_choose_baseline",
    "att_level_labels",
    "att_level_notchoose",
    "att_level_notchoose_baseline",
    "attribute",
    "attribute_id",
    "attribute_id_arranged",
    "attribute_name",
    "baseline",
    "code",
    "conf.high",
    "conf.low",
    "cov",
    "cov_mm_tau",
    "estimate",
    "estimate_0",
    "estimate_1",
    "estimates",
    "estimand",
    "id",
    "irr",
    "level",
    "level_id",
    "level_id_arranged",
    "level_name",
    "name",
    "outcome_qnum",
    "predicted",
    "predict",
    "profile",
    "projoint_results",
    "qoi",
    "qoi_1",
    "qoi_2",
    "qoi_choose",
    "qoi_choose_1",
    "qoi_choose_2",
    "qoi_notchoose",
    "qoi_notchoose_1",
    "qoi_notchoose_2",
    "response",
    "response_order",
    "sd",
    "se",
    "se_0",
    "se_1",
    "selected",
    "selected_1",
    "selected_2",
    "selected_repeated",
    "std.error",
    "t1",
    "t2",
    "task",
    "task1_y",
    "task2_y",
    "ties",
    "value",
    "vcov",
    "x",
    "y",
    "y_same"
  ))
}

#' Calculate intra-respondent reliability (IRR) from a conjoint data set. This is an internal function.
#'
#' @param .data A conjoint data set.
#' @return A scalar indicating IRR.



calculate_tau <- function(.data){

  profile <- NULL
  selected <- NULL
  selected_repeated <- NULL

  .data %>%
    # select just one profile of a repeated task
    dplyr::filter(profile == 1 & !is.na(selected_repeated)) %>%
    # calculate tau (swapping error)
    dplyr::mutate(tau = ifelse(selected != selected_repeated, 1, 0)) %>%
    # save a vector
    dplyr::pull(tau) %>%
    # calculate a mean
    mean()

}

#' Internal Estimation Function
#'
#' Core workhorse for computing marginal means (MMs) or AMCEs from a conjoint design,
#' with optional intra-respondent reliability (IRR) correction.
#'
#' @param .data A \code{\link{projoint_data}} object created by \code{\link{reshape_projoint}} or \code{\link{make_projoint_data}}.
#' @param .structure Either \code{"profile_level"} or \code{"choice_level"}.
#' @param .estimand Either \code{"mm"} (marginal mean) or \code{"amce"} (average marginal component effect).
#' @param .att_choose Attribute for the chosen profile/feature.
#' @param .lev_choose Level(s) for the chosen profile/feature. Length 1 for \code{profile_level}; may be 1+ for \code{choice_level}.
#' @param .att_notchoose Attribute for the not-chosen profile/feature (required for \code{choice_level}).
#' @param .lev_notchoose Level(s) for the not-chosen profile/feature (required for \code{choice_level}).
#' @param .att_choose_b (AMCE only) Baseline attribute for comparison.
#' @param .lev_choose_b (AMCE only) Baseline level(s) for comparison.
#' @param .att_notchoose_b (AMCE only, choice-level only) Baseline attribute for the not-chosen profile.
#' @param .lev_notchoose_b (AMCE only, choice-level only) Baseline level(s) for the not-chosen profile.
#' @param .se_method One of \code{"analytical"}, \code{"simulation"}, or \code{"bootstrap"}.
#' @param .irr \code{NULL} (default) to estimate IRR from repeated tasks; otherwise a numeric IRR value.
#' @param .remove_ties Logical; should ties be removed before estimation? Defaults to \code{TRUE}.
#' @param .ignore_position Logical; only for \code{choice_level}. If \code{TRUE} (default), ignore profile position (left/right).
#' @param .n_sims Integer; required if \code{.se_method == "simulation"}.
#' @param .n_boot Integer; required if \code{.se_method == "bootstrap"}.
#' @param .weights_1 (Optional) Bare (unquoted) column with weights for IRR estimation; passed to \code{\link[estimatr]{lm_robust}}.
#' @param .clusters_1 (Optional) Bare (unquoted) column with clusters for IRR estimation; passed to \code{\link[estimatr]{lm_robust}}.
#' @param .se_type_1 SE type for IRR estimation; passed to \code{\link[estimatr]{lm_robust}}. If \code{NULL}, \emph{estimatr} defaults are used (HC2 when unclustered; CR2 when clustered).
#' @param .weights_2 (Optional) Bare (unquoted) column with weights for MM/AMCE estimation; passed to \code{\link[estimatr]{lm_robust}}.
#' @param .clusters_2 (Optional) Bare (unquoted) column with clusters for MM/AMCE estimation; passed to \code{\link[estimatr]{lm_robust}}.
#' @param .se_type_2 SE type for MM/AMCE estimation; passed to \code{\link[estimatr]{lm_robust}}. If \code{NULL}, \emph{estimatr} defaults are used (HC2 when unclustered; CR2 when clustered).
#' @param .auto_cluster Logical; if \code{TRUE} (default), auto-cluster on \code{id} when present and no \code{.clusters_*} are supplied; auto-clustering only occurs when the corresponding \code{.se_type_*} is \code{NULL}. See \code{\link{projoint}}.
#' @param .seed Optional integer. If supplied, sets a temporary RNG seed for reproducible simulation/bootstrap inside this call 
#'   and restores the previous RNG state on exit.
#' @return A data frame with rows for the requested estimand(s) and columns:
#'   \itemize{
#'     \item \code{estimand}: one of \code{"mm_uncorrected"}, \code{"mm_corrected"},
#'           \code{"amce_uncorrected"}, \code{"amce_corrected"}.
#'     \item \code{estimate}, \code{se}, \code{conf.low}, \code{conf.high}, \code{tau}.
#'   }
#' @seealso \code{\link[estimatr]{lm_robust}}, \code{\link{projoint}},
#'   \code{\link{projoint_level}}, \code{\link{projoint_diff}}
#' @keywords internal
#' @details
#' IRR is clipped to \code{[0.5, 1)} (with a tiny epsilon) to avoid boundary issues.
#' For choice-level MMs, ties must be removed (\code{.remove_ties = TRUE}).
#' When \code{.seed} is supplied, the previous RNG state is restored on exit.
pj_estimate <- function(
    .data,
    .structure,
    .estimand,
    .att_choose, 
    .lev_choose,
    .att_notchoose, 
    .lev_notchoose,
    .att_choose_b, 
    .lev_choose_b,
    .att_notchoose_b, 
    .lev_notchoose_b,
    .se_method,
    .irr,
    .remove_ties,
    .ignore_position,
    .n_sims,
    .n_boot,
    .weights_1,
    .clusters_1,
    .se_type_1,
    .weights_2,
    .clusters_2,
    .se_type_2,
    .auto_cluster = TRUE,
    .seed = NULL
){
  # ---- Match args -----------------------------------------------------------
  structure <- rlang::arg_match0(.structure, c("choice_level", "profile_level"))
  estimand  <- rlang::arg_match0(.estimand,  c("mm", "amce"))
  se_method <- rlang::arg_match0(.se_method, c("analytical", "simulation", "bootstrap"))
  
  if (structure == "choice_level" & is.null(.att_choose)) stop("The .att_choose argument must be specified.")
  if (structure == "choice_level" & is.null(.lev_choose)) stop("The .lev_choose argument must be specified.")
  
  # ---- Argument guards ------------------------------------------------------
  if(!is.null(.irr) & (!is.numeric(.irr) || length(.irr) != 1)) {
    stop("The .irr argument must be either a numeric scalar or NULL.")
  }
  if(!is.logical(.remove_ties)) stop("The .remove_ties argument must be either TRUE or FALSE.")
  
  if (structure == "profile_level" && length(.lev_choose) != 1L)
    stop(".lev_choose must have length 1 for profile-level estimands.")
  
  if (structure == "profile_level" & !is.null(.ignore_position)) {
    stop("The .ignore_position argument can be specified only when .structure = 'choice_level'.")
  }
  if (structure == "choice_level" & is.null(.ignore_position)) {
    stop("Specify the .ignore_position argument (TRUE/FALSE) for choice-level analysis.")
  }
  if (structure == "choice_level" & !is.logical(.ignore_position)) {
    stop("The .ignore_position argument must be either TRUE or FALSE.")
  }
  
  # after the existing .att_choose / .lev_choose checks
  if (structure == "choice_level" & is.null(.att_notchoose))
    stop("The .att_notchoose argument must be specified for choice-level analysis.")
  if (structure == "choice_level" & is.null(.lev_notchoose))
    stop("The .lev_notchoose argument must be specified for choice-level analysis.")
  
  if (estimand == "amce" & (is.null(.att_choose_b) || is.null(.lev_choose_b)))
    stop("For AMCE, .att_choose_b and .lev_choose_b (baseline) must be specified.")
  if (estimand == "amce" && structure == "choice_level" &&
      (is.null(.att_notchoose_b) || is.null(.lev_notchoose_b)))
    stop("For choice-level AMCE, .att_notchoose_b and .lev_notchoose_b must be specified.")
  
  if(!is.null(.n_sims) & (!is.numeric(.n_sims) || length(.n_sims) != 1)) {
    stop("The .n_sims argument must be either a numeric scalar or NULL.")
  }
  if(!is.null(.n_boot) & (!is.numeric(.n_boot) || length(.n_boot) != 1)) {
    stop("The .n_boot argument must be either a numeric scalar or NULL.")
  }
  if(se_method == "simulation" & is.null(.n_sims))  stop("Specify .n_sims for simulation SEs.")
  if(se_method != "simulation" & !is.null(.n_sims)) stop("You cannot specify .n_sims unless .se_method = 'simulation'.")
  if(se_method == "bootstrap"  & is.null(.n_boot))  stop("Specify .n_boot for bootstrap SEs.")
  if(se_method != "bootstrap"  & !is.null(.n_boot)) stop("You cannot specify .n_boot unless .se_method = 'bootstrap'.")
  
  if (structure == "choice_level" & estimand == "mm" & .remove_ties == FALSE) {
    stop(".remove_ties must be TRUE to estimate choice-level MMs.")
  }
  if (identical(.se_type_2, "none") && se_method == "simulation") {
    stop("Simulation SEs require finite standard errors. Choose analytical or bootstrap when se_type = 'none'.")
  }
  
  # ---- Optional reproducible RNG (low-level) --------------------------------
  if (!is.null(.seed)) {
    if (exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) {
      old_seed <- .Random.seed
      on.exit({
        if (exists("old_seed", inherits = FALSE)) .Random.seed <<- old_seed
      }, add = TRUE)
    }
    set.seed(.seed)
  }
  
  # ---- Organize data --------------------------------------------------------
  if (estimand == "mm"){
    if (structure == "choice_level"){
      attlev_choose    <- stringr::str_c(.att_choose,    ":", .lev_choose)
      attlev_notchoose <- stringr::str_c(.att_notchoose, ":", .lev_notchoose)
      
      temp1 <- organize_data(.dataframe = .data$data, .structure = structure, .estimand = estimand,
                             .remove_ties, .att_choose, .lev_choose, .att_notchoose, .lev_notchoose)
      
      if (isTRUE(.ignore_position)){
        if (.att_choose == .att_notchoose){
          temp2 <- temp1$data_for_estimand |>
            dplyr::mutate(
              selected    = ifelse(qoi_1 %in% attlev_choose, selected_1, selected_2),
              qoi_choose  = stringr::str_c(attlev_choose, collapse = ", "),
              qoi_notchoose = stringr::str_c(attlev_notchoose, collapse = ", ")
            ) |>
            dplyr::select(-matches("_\\d$"))
        } else {
          temp2 <- temp1$data_for_estimand |>
            dplyr::mutate(
              selected    = ifelse(qoi_choose_1 %in% attlev_choose, selected_1, selected_2),
              qoi_choose  = stringr::str_c(attlev_choose, collapse = ", "),
              qoi_notchoose = stringr::str_c(attlev_notchoose, collapse = ", ")
            ) |>
            dplyr::select(-matches("_\\d$"))
        }
      } else {
        if (.att_choose == .att_notchoose){
          temp2 <- temp1$data_for_estimand |>
            dplyr::filter(qoi_1 %in% attlev_notchoose, qoi_2 %in% attlev_choose) |>
            dplyr::mutate(
              selected    = selected_2, 
              qoi_choose  = stringr::str_c(attlev_choose, collapse = ", "),
              qoi_notchoose = stringr::str_c(attlev_notchoose, collapse = ", ")
            ) |>
            dplyr::select(-matches("_\\d$"))
        } else {
          temp2 <- temp1$data_for_estimand |>
            dplyr::filter(qoi_notchoose_1 %in% attlev_notchoose, qoi_choose_2 %in% attlev_choose) |>
            dplyr::mutate(
              selected    = selected_2,
              qoi_choose  = stringr::str_c(attlev_choose, collapse = ", "),
              qoi_notchoose = stringr::str_c(attlev_notchoose, collapse = ", ")
            ) |>
            dplyr::select(-matches("_\\d$"))
        }
      }
      
      data_for_irr      <- temp1$data_for_irr |> dplyr::distinct()
      data_for_estimand <- temp2
      
      # Ensure id exists for bootstrap/resampling
      if (!"id" %in% names(data_for_irr))      data_for_irr$id      <- seq_len(nrow(data_for_irr))
      if (!"id" %in% names(data_for_estimand)) data_for_estimand$id <- seq_len(nrow(data_for_estimand))
      
    } else {
      .list <- organize_data(.dataframe = .data$data, .structure = structure, .estimand = estimand,
                             .remove_ties, .att_choose, .lev_choose, .att_notchoose, .lev_notchoose)
      data_for_irr      <- .list$data_for_irr
      data_for_estimand <- .list$data_for_estimand
      
      # Ensure id exists for bootstrap/resampling
      if (!"id" %in% names(data_for_irr))      data_for_irr$id      <- seq_len(nrow(data_for_irr))
      if (!"id" %in% names(data_for_estimand)) data_for_estimand$id <- seq_len(nrow(data_for_estimand))
      
    }
  } else { # estimand == "amce"
    if (structure == "choice_level"){
      # Not baseline
      attlev_choose    <- stringr::str_c(.att_choose,    ":", .lev_choose)
      attlev_notchoose <- stringr::str_c(.att_notchoose, ":", .lev_notchoose)
      temp1 <- organize_data(.dataframe = .data$data, .structure = structure, .estimand = estimand,
                             .remove_ties, .att_choose, .lev_choose, .att_notchoose, .lev_notchoose)
      if (isTRUE(.ignore_position)){
        if (.att_choose == .att_notchoose){
          temp2 <- temp1$data_for_estimand |>
            dplyr::mutate(
              selected    = ifelse(qoi_1 %in% attlev_choose, selected_1, selected_2),
              qoi_choose  = stringr::str_c(attlev_choose, collapse = ", "),
              qoi_notchoose = stringr::str_c(attlev_notchoose, collapse = ", ")
            ) |>
            dplyr::select(-matches("_\\d$"))
        } else {
          temp2 <- temp1$data_for_estimand |>
            dplyr::mutate(
              selected    = ifelse(qoi_choose_1 %in% attlev_choose, selected_1, selected_2),
              qoi_choose  = stringr::str_c(attlev_choose, collapse = ", "),
              qoi_notchoose = stringr::str_c(attlev_notchoose, collapse = ", ")
            ) |>
            dplyr::select(-matches("_\\d$"))
        }
      } else {
        if (.att_choose == .att_notchoose){
          temp2 <- temp1$data_for_estimand |>
            dplyr::filter(qoi_1 %in% attlev_notchoose, qoi_2 %in% attlev_choose) |>
            dplyr::mutate(
              selected    = selected_2,
              qoi_choose  = stringr::str_c(attlev_choose, collapse = ", "),
              qoi_notchoose = stringr::str_c(attlev_notchoose, collapse = ", ")
            ) |>
            dplyr::select(-matches("_\\d$"))
        } else {
          temp2 <- temp1$data_for_estimand |>
            dplyr::filter(qoi_notchoose_1 %in% attlev_notchoose, qoi_choose_2 %in% attlev_choose) |>
            dplyr::mutate(
              selected    = selected_2,
              qoi_choose  = stringr::str_c(attlev_choose, collapse = ", "),
              qoi_notchoose = stringr::str_c(attlev_notchoose, collapse = ", ")
            ) |>
            dplyr::select(-matches("_\\d$"))
        }
      }
      
      # Baseline
      attlev_choose_b    <- stringr::str_c(.att_choose_b,    ":", .lev_choose_b)
      attlev_notchoose_b <- stringr::str_c(.att_notchoose_b, ":", .lev_notchoose_b)
      
      temp1_b <- organize_data(.dataframe = .data$data, .structure = structure, .estimand = estimand,
                               .remove_ties,
                               .att_choose = .att_choose_b, .lev_choose = .lev_choose_b,
                               .att_notchoose = .att_notchoose_b, .lev_notchoose = .lev_notchoose_b)
      if (isTRUE(.ignore_position)){
        if (.att_choose_b == .att_notchoose_b){
          temp2_b <- temp1_b$data_for_estimand |>
            dplyr::mutate(
              selected    = ifelse(qoi_1 %in% attlev_choose_b, selected_1, selected_2),
              qoi_choose  = stringr::str_c(attlev_choose_b, collapse = ", "),
              qoi_notchoose = stringr::str_c(attlev_notchoose_b, collapse = ", ")
            ) |>
            dplyr::select(-matches("_\\d$"))
        } else {
          temp2_b <- temp1_b$data_for_estimand |>
            dplyr::mutate(
              selected    = ifelse(qoi_choose_1 %in% attlev_choose_b, selected_1, selected_2),
              qoi_choose  = stringr::str_c(attlev_choose_b, collapse = ", "),
              qoi_notchoose = stringr::str_c(attlev_notchoose_b, collapse = ", ")
            ) |>
            dplyr::select(-matches("_\\d$"))
        }
      } else {
        if (.att_choose_b == .att_notchoose_b){
          temp2_b <- temp1_b$data_for_estimand |>
            dplyr::filter(qoi_1 %in% attlev_notchoose_b, qoi_2 %in% attlev_choose_b) |>
            dplyr::mutate(
              selected    = selected_2,
              qoi_choose  = stringr::str_c(attlev_choose_b, collapse = ", "),
              qoi_notchoose = stringr::str_c(attlev_notchoose_b, collapse = ", ")
            ) |>
            dplyr::select(-matches("_\\d$"))
        } else {
          temp2_b <- temp1_b$data_for_estimand |>
            dplyr::filter(qoi_notchoose_1 %in% attlev_notchoose_b, qoi_choose_2 %in% attlev_choose_b) |>
            dplyr::mutate(
              selected    = selected_2,
              qoi_choose  = stringr::str_c(attlev_choose_b, collapse = ", "),
              qoi_notchoose = stringr::str_c(attlev_notchoose_b, collapse = ", ")
            ) |>
            dplyr::select(-matches("_\\d$"))
        }
      }
      
      # Merge
      data_for_irr <- dplyr::bind_rows(temp1$data_for_irr, temp1_b$data_for_irr) |> dplyr::distinct()
      data_for_estimand <- dplyr::bind_rows(
        temp2   |> dplyr::mutate(x = 1),
        temp2_b |> dplyr::mutate(x = 0)
      )
      
      # Ensure id exists for bootstrap/resampling
      if (!"id" %in% names(data_for_irr))      data_for_irr$id      <- seq_len(nrow(data_for_irr))
      if (!"id" %in% names(data_for_estimand)) data_for_estimand$id <- seq_len(nrow(data_for_estimand))
      
      
    } else {
      temp1   <- organize_data(.dataframe = .data$data, .structure = structure, .estimand = estimand,
                               .remove_ties, .att_choose, .lev_choose, .att_notchoose, .lev_notchoose)
      temp1_b <- organize_data(.dataframe = .data$data, .structure = structure, .estimand = estimand,
                               .remove_ties,
                               .att_choose = .att_choose_b, .lev_choose = .lev_choose_b,
                               .att_notchoose = .att_notchoose_b, .lev_notchoose = .lev_notchoose_b)
      data_for_irr <- dplyr::bind_rows(temp1$data_for_irr, temp1_b$data_for_irr) |> dplyr::distinct()
      data_for_estimand <- dplyr::bind_rows(
        temp1$data_for_estimand   |> dplyr::mutate(x = 1),
        temp1_b$data_for_estimand |> dplyr::mutate(x = 0)
      )
      
      if (!"id" %in% names(data_for_irr))      data_for_irr$id      <- seq_len(nrow(data_for_irr))
      if (!"id" %in% names(data_for_estimand)) data_for_estimand$id <- seq_len(nrow(data_for_estimand))
      
    }
  }
  
  if (!"id" %in% names(data_for_irr))      data_for_irr$id      <- dplyr::row_number()
  if (!"id" %in% names(data_for_estimand)) data_for_estimand$id <- dplyr::row_number()
  
  # ---- Tidy-eval (quosures) -------------------------------------------------
  # Accept either a bare name (we'll enquo it) or a pre-made quosure
  clusters1_quo <- if (rlang::is_quosure(.clusters_1)) .clusters_1 else rlang::enquo(.clusters_1)
  weights1_quo  <- if (rlang::is_quosure(.weights_1))  .weights_1  else rlang::enquo(.weights_1)
  clusters2_quo <- if (rlang::is_quosure(.clusters_2)) .clusters_2 else rlang::enquo(.clusters_2)
  weights2_quo  <- if (rlang::is_quosure(.weights_2))  .weights_2  else rlang::enquo(.weights_2)
  
  
  # Auto-cluster only if allowed and sensible
  if (is.null(.irr) &&
      .auto_cluster &&
      rlang::quo_is_null(clusters1_quo) &&
      "id" %in% names(data_for_irr) &&
      is.null(.se_type_1)) {
    clusters1_quo <- rlang::quo(id)
  }
  if (.auto_cluster &&
      rlang::quo_is_null(clusters2_quo) &&
      "id" %in% names(data_for_estimand) &&
      is.null(.se_type_2)) {
    clusters2_quo <- rlang::quo(id)
  }
  
  # Figure out whether each stage is clustered
  is_clustered_1 <- if (is.null(.irr)) !rlang::quo_is_null(clusters1_quo) else FALSE
  is_clustered_2 <- !rlang::quo_is_null(clusters2_quo)
  
  # Define allowed se_type values once, before validation
  ok_uncl <- c("classical","HC0","HC1","HC2","HC3","stata","none")
  ok_cl   <- c("CR0","CR2","stata","none")
  
  # And gate se_type_1 validation:
  if (is.null(.irr) && !is.null(.se_type_1)) {
    if (is_clustered_1 && !(.se_type_1 %in% ok_cl)) {
      stop("`.se_type_1` must be one of ", paste(ok_cl, collapse=", "),
           " when clusters are supplied or auto-detected.")
    }
    if (!is_clustered_1 && !(.se_type_1 %in% ok_uncl)) {
      stop("`.se_type_1` must be one of ", paste(ok_uncl, collapse=", "),
           " when no clusters are used.")
    }
  }
  
  
  if (!is.null(.se_type_2)) {
    if (is_clustered_2 && !(.se_type_2 %in% ok_cl)) {
      stop("`.se_type_2` must be one of ", paste(ok_cl, collapse=", "),
           " when clusters are supplied or auto-detected.")
    }
    if (!is_clustered_2 && !(.se_type_2 %in% ok_uncl)) {
      stop("`.se_type_2` must be one of ", paste(ok_uncl, collapse=", "),
           " when no clusters are used.")
    }
  }
  
  # ---- Decide actual se_type passed to lm_robust() ---------------------------
  se_type2_final <- if (is.null(.se_type_2)) {
    if (is_clustered_2) "CR2" else "HC2"
  } else {
    .se_type_2
  }
  
  # Use this name for the label (NULL/NA if no clustering)
  cluster_by2 <- if (is_clustered_2) {
    # try to get a nice name for the quosure
    rlang::as_name(clusters2_quo)
  } else {
    NA_character_
  }
  
  # ---- Estimate IRR / tau ---------------------------------------------------
  
  if (nrow(data_for_estimand) == 0L) {
    stop("No rows match the specified attribute/level combination(s).")
  }
  if (is.null(.irr) && nrow(data_for_irr) == 0L) {
    stop("No rows available to estimate IRR/tau. Consider supplying .irr directly.")
  }
  # if (nrow(data_for_irr) == 0L) {
  #   stop("No rows available to estimate IRR/tau.")
  # }
  
  critical_from_ci <- function(est, se, lo) {
    ct <- abs((lo - est) / se)
    if (!is.finite(ct)) 1.96 else ct
  }
  
  eps <- 1e-8
  
  if (is.null(.irr)) {
    reg_irr <- estimatr::lm_robust(
      agree ~ 1,
      weights  = if (!rlang::quo_is_null(weights1_quo))  rlang::eval_tidy(weights1_quo,  data_for_irr) else NULL,
      clusters = if (!rlang::quo_is_null(clusters1_quo)) rlang::eval_tidy(clusters1_quo, data_for_irr) else NULL,
      se_type = .se_type_1,
      data = data_for_irr
    ) |> estimatr::tidy()
    
    irr_raw <- reg_irr$estimate[1]
    if (is.na(irr_raw)) {
      warning("Estimated IRR is NA. Using fallback IRR = 0.75.")
      irr <- 0.75
      var_irr <- 0   # avoid NA propagation in var_tau
    } else {
      if (irr_raw < 0.5 || irr_raw > 1) {
        warning(sprintf("Estimated IRR was %.3f and clipped to [0.5, 1].", irr_raw))
      }
      irr <- irr_raw
      se_irr  <- reg_irr$std.error[1]
      var_irr <- if (is.na(se_irr)) 0 else se_irr^2
    }
    
    # Nudge off boundaries for stability
    irr <- min(max(irr, 0.5 + eps), 1 - eps)
    
    # With interior IRR, no special-casing needed
    tau     <- (1 - sqrt(1 - 2 * (1 - irr))) / 2
    var_tau <- 0.25 * (2 * irr - 1)^(-1) * var_irr
    
  } else if (!is.numeric(.irr) || length(.irr) != 1L || is.na(.irr) || !is.finite(.irr)) {
    stop(".irr should be a finite numeric scalar.")
    
  } else {
    # user-supplied IRR: warn if outside [0.5,1], then always nudge/clip
    if (.irr < 0.5 || .irr > 1) {
      warning(sprintf("Supplied IRR %.3f clipped to [0.5, 1].", .irr))
    }
    irr <- min(max(.irr, 0.5 + eps), 1 - eps)
    
    tau     <- (1 - sqrt(1 - 2 * (1 - irr))) / 2
    var_tau <- 0   # user-supplied IRR treated as fixed
  }
  
  
  # ---- Uncorrected estimate -------------------------------------------------
  if (estimand == "mm") {
    # reg_mm <- estimatr::lm_robust(
    #   selected ~ 1,
    #   weights  = if (!rlang::quo_is_null(weights2_quo))  rlang::eval_tidy(weights2_quo,  data_for_estimand) else NULL,
    #   clusters = if (!rlang::quo_is_null(clusters2_quo)) rlang::eval_tidy(clusters2_quo, data_for_estimand) else NULL,
    #   se_type  = se_type2_final,
    #   data     = data_for_estimand
    # ) |> estimatr::tidy()
    
    # Fixed on Sep 10, 2025     ###############################################
    
    # cache once
    w2  <- if (!rlang::quo_is_null(weights2_quo))  rlang::eval_tidy(weights2_quo,  data_for_estimand) else NULL
    cl2 <- if (!rlang::quo_is_null(clusters2_quo)) rlang::eval_tidy(clusters2_quo, data_for_estimand) else NULL
    
    # 1) fit with configured se_type2_final (often "CR2")
    reg_mm_fit <- suppressWarnings(
      estimatr::lm_robust(
        selected ~ 1,
        weights  = w2,
        clusters = cl2,
        se_type  = se_type2_final,
        data     = data_for_estimand
      )
    )
    
    V <- reg_mm_fit$vcov
    d <- diag(V)
    bad_vcov <- any(!is.finite(d) | d < -1e-10)
    bad_se   <- any(!is.finite(summary(reg_mm_fit)$coefficients[, "Std. Error"]))
    
    # 2) fallback if NA/Inf or negative diag
    if (bad_vcov || bad_se) {
      reg_mm_fit <- suppressWarnings(
        estimatr::lm_robust(
          selected ~ 1,
          weights  = w2,
          clusters = cl2,
          se_type  = "stata",
          data     = data_for_estimand
        )
      )
      
      V <- reg_mm_fit$vcov
      d <- diag(V)
      bad_vcov <- any(!is.finite(d) | d < -1e-10)
      bad_se   <- any(!is.finite(summary(reg_mm_fit)$coefficients[, "Std. Error"]))
      
      # optional last resort: drop clusters + HC1
      if (bad_vcov || bad_se) {
        reg_mm_fit <- estimatr::lm_robust(
          selected ~ 1,
          weights  = w2,
          clusters = NULL,
          se_type  = "HC1",
          data     = data_for_estimand
        )
        V <- reg_mm_fit$vcov
        d <- diag(V)
      }
      
      warning("MM analytical SEs: CR2 produced non-PSD/NA variances; fell back to se_type='stata' (then HC1 if needed).")
    }
    
    # (optional) record what actually got used
    se_type2_used_mm <- reg_mm_fit$se_type
    # meta$main_se$se_type_used_mm <- se_type2_used_mm
    
    # proceed
    reg_mm <- estimatr::tidy(reg_mm_fit)
    
    ###############################################
    
    critical_t <- critical_from_ci(reg_mm$estimate[1], reg_mm$std.error[1], reg_mm$conf.low[1])
    
    mm_uncorrected     <- reg_mm$estimate[1]
    var_uncorrected_mm <- reg_mm$std.error[1]^2
    
    if (se_method == "simulation" && is.na(reg_mm$std.error[1])) {
      warning("Simulation SEs require finite standard errors; e.g., with se_type = 'none' results may be NA.")
    }
  }
  
  if (estimand == "amce") {
    
    
    # reg_amce <- estimatr::lm_robust(
    #   selected ~ x,
    #   weights  = if (!rlang::quo_is_null(weights2_quo))  rlang::eval_tidy(weights2_quo,  data_for_estimand) else NULL,
    #   clusters = if (!rlang::quo_is_null(clusters2_quo)) rlang::eval_tidy(clusters2_quo, data_for_estimand) else NULL,
    #   se_type  = se_type2_final,
    #   data     = data_for_estimand
    # ) |> estimatr::tidy()
    
    # Fixed on Sep 10, 2025     ###############################################
    
    # cache weights/clusters once
    w2  <- if (!rlang::quo_is_null(weights2_quo))  rlang::eval_tidy(weights2_quo,  data_for_estimand) else NULL
    cl2 <- if (!rlang::quo_is_null(clusters2_quo)) rlang::eval_tidy(clusters2_quo, data_for_estimand) else NULL
    
    # 1) fit with your configured se_type2_final (likely "CR2")
    reg_amce_fit <- suppressWarnings(
      estimatr::lm_robust(
        selected ~ x,
        weights  = w2,
        clusters = cl2,
        se_type  = se_type2_final,
        data     = data_for_estimand
      )
    )
    
    V <- reg_amce_fit$vcov
    d <- diag(V)
    bad_vcov <- any(!is.finite(d) | d < -1e-10)
    bad_se   <- any(!is.finite(summary(reg_amce_fit)$coefficients[, "Std. Error"]))

    # 2) fallback if vcov is NA/Inf or non-PSD (negative diag)
    if (bad_vcov || bad_se) {
      reg_amce_fit <- suppressWarnings(
        estimatr::lm_robust(
          selected ~ x,
          weights  = w2,
          clusters = cl2,
          se_type  = "stata"   # HC1/Stata-style; typically PSD
          ,
          data     = data_for_estimand
        )
      )
      
      V <- reg_amce_fit$vcov
      d <- diag(V)
      bad_vcov <- any(!is.finite(d) | d < -1e-10)
      bad_se   <- any(!is.finite(summary(reg_amce_fit)$coefficients[, "Std. Error"]))
      
      # optional last resort: drop clustering if still bad
      if (bad_vcov || bad_se) {
        reg_amce_fit <- suppressWarnings(
          estimatr::lm_robust(
            selected ~ x,
            weights  = w2,
            clusters = NULL,
            se_type  = "HC1",
            data     = data_for_estimand
          )
        )
        V <- reg_amce_fit$vcov
        d <- diag(V)
      }
      
      warning("AMCE analytical SEs: CR2 produced non-PSD/NA variances; fell back to se_type='stata' (then HC1 if needed).")
    }
    
    # (optional) record what actually got used
    se_type2_used <- reg_amce_fit$se_type
    # e.g., stash in your result/meta if you have one:
    # meta$main_se$se_type_used <- se_type2_used
    
    # proceed as before
    reg_amce <- estimatr::tidy(reg_amce_fit)
    
    ###############################################
    
    
    
    critical_t <- critical_from_ci(reg_amce$estimate[2], reg_amce$std.error[2], reg_amce$conf.low[2])
    
    amce_uncorrected     <- reg_amce$estimate[2]
    var_uncorrected_amce <- reg_amce$std.error[2]^2
    
    if (se_method == "simulation" && is.na(reg_amce$std.error[2])) {
      warning("Simulation SEs require finite standard errors; e.g., with se_type = 'none' results may be NA.")
    }
  }
  
  # ---- Covariance pieces ----------------------------------------------------
  has_agree <- "agree" %in% names(data_for_estimand)
  data_for_cov <- if (has_agree) dplyr::filter(data_for_estimand, !is.na(agree))
  else data_for_estimand[0, , drop = FALSE]
  nz <- has_agree && nrow(data_for_cov) > 0L
  
  if (estimand == "mm"){
    if (is.null(.irr) && nz) {
      cov_mm_irr <- if (nrow(data_for_cov) > 1L)
        stats::cov(data_for_cov$selected, data_for_cov$agree) / nrow(data_for_irr)
      else 0
    } else {
      cov_mm_irr <- 0
    }
    cov_mm_tau <- -0.5 * (2 * irr - 1)^(-1/2) * cov_mm_irr
  }
  
  if (estimand == "amce"){
    d_cov0 <- data_for_cov |> dplyr::filter(x == 0)
    d_cov1 <- data_for_cov |> dplyr::filter(x == 1)
    n0 <- if (nz) nrow(d_cov0) else 0L
    n1 <- if (nz) nrow(d_cov1) else 0L
    
    if (is.null(.irr) && nz) {
      cov_mm0_irr <- if (n0 > 1L) stats::cov(d_cov0$selected, d_cov0$agree) / nrow(data_for_irr) else 0
      cov_mm1_irr <- if (n1 > 1L) stats::cov(d_cov1$selected, d_cov1$agree) / nrow(data_for_irr) else 0
    } else {
      cov_mm0_irr <- 0
      cov_mm1_irr <- 0
    }
    cov_mm0_tau  <- -0.5 * (2 * irr - 1)^(-1/2) * cov_mm0_irr
    cov_mm1_tau  <- -0.5 * (2 * irr - 1)^(-1/2) * cov_mm1_irr
    cov_amce_tau <- cov_mm1_tau - cov_mm0_tau
  }
  
  # ---- Corrected estimates & SEs -------------------------------------------
  if (estimand == "mm"){
    if (se_method == "analytical"){
      mm_corrected <- (mm_uncorrected - tau) / (1 - 2 * tau)
      # var_corrected_mm <- (mm_corrected^2 / (1 - (2 * tau))^2) *
      #   ( ((var_uncorrected_mm + var_tau - 2 * cov_mm_tau) / mm_corrected^2)
      #     + 4 * (cov_mm_tau - var_tau) / mm_corrected
      #     + 4 * var_tau )
      
      # Use delta-method. Avoid blowing up the variance when the corrected estimate is near 0.
      dp  <- 1 / (1 - 2 * tau)
      dt  <- (2 * mm_uncorrected - 1) / (1 - 2 * tau)^2
      var_corrected_mm <- dp^2 * var_uncorrected_mm + dt^2 * var_tau + 2 * dp * dt * cov_mm_tau
      
      output <- data.frame(
        estimand = c("mm_uncorrected", "mm_corrected"),
        estimate = c(mm_uncorrected, mm_corrected),
        se       = c(sqrt(pmax(var_uncorrected_mm, 0)), sqrt(pmax(var_corrected_mm, 0)))
      ) |>
        dplyr::mutate(conf.low  = estimate - critical_t * se,
                      conf.high = estimate + critical_t * se,
                      tau = tau)
    }  
    
    if (se_method == "simulation"){
      if ((var_tau <= 0 || !is.finite(var_tau)) && (cov_mm_tau == 0)) {
        mm_draws  <- stats::rnorm(.n_sims, mean = mm_uncorrected, sd = sqrt(var_uncorrected_mm))
        tau_draws <- rep(tau, .n_sims)
        sim_mm <- data.frame(mm_uncorrected = mm_draws, tau = tau_draws) |>
          dplyr::mutate(mm_corrected = (mm_uncorrected - tau) / (1 - 2 * tau))
      } else {
        vcov_mm_tau <- matrix(c(var_uncorrected_mm, cov_mm_tau,
                                cov_mm_tau,        var_tau), nrow = 2)
        sim_mm <- as.data.frame(MASS::mvrnorm(.n_sims, c(mm_uncorrected, tau), vcov_mm_tau)) |>
          rlang::set_names(c("mm_uncorrected", "tau")) |>
          dplyr::mutate(mm_corrected = (mm_uncorrected - tau) / (1 - 2 * tau))
      }
      
      output <- data.frame(
        estimand = c("mm_uncorrected", "mm_corrected"),
        estimate = c(mm_uncorrected, mean(sim_mm$mm_corrected)),
        se       = c(sqrt(pmax(var_uncorrected_mm, 0)), stats::sd(sim_mm$mm_corrected))
      ) |>
        dplyr::mutate(conf.low  = estimate - critical_t * se,
                      conf.high = estimate + critical_t * se,
                      tau = tau)
    }
    
    if (se_method == "bootstrap"){
      # Only define IDs_1 if we’ll estimate IRR
      if (is.null(.irr)) {
        IDs_1 <- unique(stats::na.omit(data_for_irr$id))
        if (length(IDs_1) == 0L) stop("No rows available to bootstrap IRR/tau. Consider supplying .irr directly.")
      }
      IDs_2 <- unique(stats::na.omit(data_for_estimand$id))
      out <- NULL
      for (i in seq_len(.n_boot)) {
        
        # Always bootstrap the estimand stage
        id_2 <- sample(IDs_2, length(IDs_2), replace = TRUE)
        bs_sample_2 <- data.frame(id = id_2) |>
          dplyr::left_join(data_for_estimand, by = "id", relationship = "many-to-many")
        
        # Bootstrap (and fit) IRR only when needed
        if (is.null(.irr)) {
          id_1 <- sample(IDs_1, length(IDs_1), replace = TRUE)
          bs_sample_1 <- data.frame(id = id_1) |>
            dplyr::left_join(data_for_irr, by = "id", relationship = "many-to-many")
          
          reg_irr <- estimatr::lm_robust(
            agree ~ 1,
            weights  = if (!rlang::quo_is_null(weights1_quo))  rlang::eval_tidy(weights1_quo,  bs_sample_1) else NULL,
            clusters = if (!rlang::quo_is_null(clusters1_quo)) rlang::eval_tidy(clusters1_quo, bs_sample_1) else NULL,
            se_type  = .se_type_1,
            data     = bs_sample_1
          ) |> estimatr::tidy()
          
          irr_bs <- reg_irr$estimate[1]
          tau_bs <- (1 - sqrt(1 - 2 * (1 - irr_bs))) / 2
        } else {
          tau_bs <- (1 - sqrt(1 - 2 * (1 - .irr))) / 2
        }
        
        reg_mm <- estimatr::lm_robust(
          selected ~ 1,
          weights  = if (!rlang::quo_is_null(weights2_quo))  rlang::eval_tidy(weights2_quo,  bs_sample_2) else NULL,
          clusters = if (!rlang::quo_is_null(clusters2_quo)) rlang::eval_tidy(clusters2_quo, bs_sample_2) else NULL,
          se_type = se_type2_final,
          data = bs_sample_2
        ) |> estimatr::tidy()
        
        mm_uncorrected_bs <- reg_mm$estimate[1]
        mm_corrected_bs <- (mm_uncorrected_bs - tau_bs) / (1 - 2 * tau_bs)
        
        out <- dplyr::bind_rows(out,
                                data.frame(estimand = c("mm_corrected","mm_uncorrected"),
                                           estimate = c(mm_corrected_bs, mm_uncorrected_bs)))
      }
      
      output <- data.frame(
        estimand = c("mm_uncorrected","mm_corrected"),
        estimate = c(
          mean(out |> dplyr::filter(estimand == "mm_uncorrected") |> dplyr::pull(estimate)),
          mean(out |> dplyr::filter(estimand == "mm_corrected")   |> dplyr::pull(estimate))
        ),
        se = c(
          stats::sd(out |> dplyr::filter(estimand == "mm_uncorrected") |> dplyr::pull(estimate)),
          stats::sd(out |> dplyr::filter(estimand == "mm_corrected")   |> dplyr::pull(estimate))
        )
      ) |>
        dplyr::mutate(conf.low = estimate - critical_t * se,
                      conf.high = estimate + critical_t * se,
                      tau = tau)
    }
  }
  
  if (estimand == "amce"){
    if (se_method == "analytical"){
      amce_corrected <- amce_uncorrected / (1 - 2 * tau)
      
      # var_corrected_amce <- (amce_corrected^2 / (1 - (2 * tau))^2) *
      #   ( (var_uncorrected_amce / amce_corrected^2)
      #     + 4 * cov_amce_tau / amce_corrected
      #     + 4 * var_tau )
      
      # Use delta-method. Avoid blowing up the variance when the corrected estimate is near 0.
      db  <- 1 / (1 - 2 * tau)
      dt  <- 2 * amce_uncorrected / (1 - 2 * tau)^2
      var_corrected_amce <- db^2 * var_uncorrected_amce + dt^2 * var_tau + 2 * db * dt * cov_amce_tau
      
      output <- data.frame(
        estimand = c("amce_uncorrected","amce_corrected"),
        estimate = c(amce_uncorrected, amce_corrected),
        se       = c(sqrt(pmax(var_uncorrected_amce, 0)), sqrt(pmax(var_corrected_amce, 0)))
      ) |>
        dplyr::mutate(conf.low  = estimate - critical_t * se,
                      conf.high = estimate + critical_t * se,
                      tau = tau)
    }
    
    if (se_method == "simulation"){
      if ((var_tau <= 0 || !is.finite(var_tau)) && (cov_amce_tau == 0)) {
        amce_draws <- stats::rnorm(.n_sims, mean = amce_uncorrected, sd = sqrt(var_uncorrected_amce))
        tau_draws  <- rep(tau, .n_sims)
        sim_amce <- data.frame(amce_uncorrected = amce_draws, tau = tau_draws) |>
          dplyr::mutate(amce_corrected = amce_uncorrected / (1 - 2 * tau))
      } else {
        vcov_amce_tau <- matrix(c(var_uncorrected_amce, cov_amce_tau,
                                  cov_amce_tau,        var_tau), nrow = 2)
        sim_amce <- as.data.frame(MASS::mvrnorm(.n_sims, c(amce_uncorrected, tau), vcov_amce_tau)) |>
          rlang::set_names(c("amce_uncorrected","tau")) |>
          dplyr::mutate(amce_corrected = amce_uncorrected / (1 - 2 * tau))
      }
      
      output <- data.frame(
        estimand = c("amce_uncorrected","amce_corrected"),
        estimate = c(amce_uncorrected, mean(sim_amce$amce_corrected)),
        se       = c(sqrt(pmax(var_uncorrected_amce, 0)), stats::sd(sim_amce$amce_corrected))
      ) |>
        dplyr::mutate(conf.low  = estimate - critical_t * se,
                      conf.high = estimate + critical_t * se,
                      tau = tau)
    }
    
    if (se_method == "bootstrap"){
      
      IDs_2 <- unique(stats::na.omit(data_for_estimand$id))
      # Only define IDs_1 if we’ll estimate IRR
      if (is.null(.irr)) {
        IDs_1 <- unique(stats::na.omit(data_for_irr$id))
        if (length(IDs_1) == 0L) stop("No rows available to bootstrap IRR/tau. Consider supplying .irr directly.")
      }
      
      out <- NULL
      for (i in seq_len(.n_boot)) {
        
        # Always bootstrap the estimand stage
        id_2 <- sample(IDs_2, length(IDs_2), replace = TRUE)
        bs_sample_2 <- data.frame(id = id_2) |>
          dplyr::left_join(data_for_estimand, by = "id", relationship = "many-to-many")
        
        # Bootstrap (and fit) IRR only when needed
        if (is.null(.irr)) {
          id_1 <- sample(IDs_1, length(IDs_1), replace = TRUE)
          bs_sample_1 <- data.frame(id = id_1) |>
            dplyr::left_join(data_for_irr, by = "id", relationship = "many-to-many")
          
          reg_irr <- estimatr::lm_robust(
            agree ~ 1,
            weights  = if (!rlang::quo_is_null(weights1_quo))  rlang::eval_tidy(weights1_quo,  bs_sample_1) else NULL,
            clusters = if (!rlang::quo_is_null(clusters1_quo)) rlang::eval_tidy(clusters1_quo, bs_sample_1) else NULL,
            se_type  = .se_type_1,
            data     = bs_sample_1
          ) |> estimatr::tidy()
          
          irr_bs <- reg_irr$estimate[1]
          tau_bs <- (1 - sqrt(1 - 2 * (1 - irr_bs))) / 2
        } else {
          tau_bs <- (1 - sqrt(1 - 2 * (1 - .irr))) / 2
        }
        
        reg_amce <- estimatr::lm_robust(
          selected ~ x,
          weights  = if (!rlang::quo_is_null(weights2_quo))  rlang::eval_tidy(weights2_quo,  bs_sample_2) else NULL,
          clusters = if (!rlang::quo_is_null(clusters2_quo)) rlang::eval_tidy(clusters2_quo, bs_sample_2) else NULL,
          se_type = se_type2_final,
          data = bs_sample_2
        ) |> estimatr::tidy()
        
        amce_uncorrected_bs <- reg_amce$estimate[2]
        amce_corrected_bs <- amce_uncorrected_bs / (1 - 2 * tau_bs)
        
        out <- dplyr::bind_rows(out,
                                data.frame(estimand = c("amce_corrected","amce_uncorrected"),
                                           estimate = c(amce_corrected_bs, amce_uncorrected_bs)))
      }
      
      output <- data.frame(
        estimand = c("amce_uncorrected","amce_corrected"),
        estimate = c(
          mean(out |> dplyr::filter(estimand == "amce_uncorrected") |> dplyr::pull(estimate)),
          mean(out |> dplyr::filter(estimand == "amce_corrected")   |> dplyr::pull(estimate))
        ),
        se = c(
          stats::sd(out |> dplyr::filter(estimand == "amce_uncorrected") |> dplyr::pull(estimate)),
          stats::sd(out |> dplyr::filter(estimand == "amce_corrected")   |> dplyr::pull(estimate))
        )
      ) |>
        dplyr::mutate(conf.low = estimate - critical_t * se,
                      conf.high = estimate + critical_t * se,
                      tau = tau)
    }
  }
  
  if (any(!is.finite(output$se))) {
    warning("Some standard errors are NA/Inf after corrections. ",
            "Consider se_type='stata' or bootstrap SEs.")
  }
  
  # Prefer the se_type captured from the finalized fit (AMCE first, then MM).
  se_type_used_final <-
    if (exists("se_type2_used", inherits = FALSE)) {
      se_type2_used
    } else if (exists("se_type2_used_mm", inherits = FALSE)) {
      se_type2_used_mm
    } else {
      se_type2_final  # fallback: what was requested
    }
  
  attr(output, "se_type_used") <- se_type_used_final
  attr(output, "cluster_by")   <- cluster_by2
  
  return(output)
}

#' Summary method for \code{projoint_results}
#'
#' Creates a concise tabular summary of a \code{projoint_results} object,
#' including the chosen estimand, analysis structure, standard-error settings,
#' and a data frame of estimates.
#'
#' @param object An object of class \code{projoint_results}.
#' @param ... Additional arguments (ignored).
#'
#' @return A data frame (often a tibble) summarizing the estimated effects.
#'   At minimum, it contains the columns produced in \code{object$estimates}
#'   (e.g., attribute/level identifiers and the point estimate with its
#'   standard error and confidence interval in columns such as
#'   \code{estimate}, \code{std.error}, \code{conf.low}, \code{conf.high}).
#'   This table is suitable for further processing or printing.
#'
#' @export
#'
#' @examples
#' \donttest{
#'   data(exampleData1)
#'
#'   # Reshape data for two base tasks + repeated (for IRR estimation)
#'   dat <- reshape_projoint(
#'     exampleData1,
#'     .outcomes = c("choice1", "choice2", "choice1_repeated_flipped")
#'   )
#'
#'   # Build a valid choice-level QoI
#'   att <- unique(dat$labels$attribute_id)[1]
#'   lev_ids   <- dat$labels$level_id[dat$labels$attribute_id == att]
#'   lev_names <- sub(".*:", "", lev_ids)
#'
#'   q <- set_qoi(
#'     .structure     = "choice_level",
#'     .estimand      = "mm",
#'     .att_choose    = att,
#'     .lev_choose    = lev_names[2],
#'     .att_notchoose = att,
#'     .lev_notchoose = lev_names[1]
#'   )
#'
#'   # Fit model
#'   fit <- projoint(dat, .qoi = q)
#'
#'   # Get the tabular summary of estimates
#'   tab <- summary(fit)
#'   head(tab)
#' }
summary.projoint_results <- function(object, ...) {
  
  se_line <- if (is.null(object$se_type_used)) NA_character_ else object$se_type_used
  by_line <- if (is.null(object$cluster_by))   NA_character_ else object$cluster_by
  
  estimates_summary <- object$estimates
  
  cat("\nSummary of Projoint Estimates\n")
  cat("------------------------------\n")
  cat("Estimand: ", object$estimand, "\n", sep = "")
  cat("Structure: ", object$structure, "\n", sep = "")
  cat("Standard error method: ", object$se_method, "\n", sep = "")
  
  if (!is.na(se_line)) {
    if (!is.na(by_line)) {
      cat("SE type (lm_robust):   ", se_line, " (clustered by ", by_line, ")\n", sep = "")
    } else {
      cat("SE type (lm_robust):   ", se_line, "\n", sep = "")
    }
  }
  
  cat("IRR: ", object$irr, "\n", sep = "")
  cat("Tau: ", round(object$tau, 3), "\n", sep = "")
  cat("\n")
  
  return(estimates_summary)
}

#' Reshape survey response data for conjoint analysis (single task set)
#'
#' Takes a wide survey data frame (e.g., from \code{\link{read_Qualtrics}}) and reshapes
#' it so that each row corresponds to a single respondent–task–profile. Supports arbitrary
#' ordering of base tasks and a single repeated task per respondent. The repeated base task
#' is inferred from the first base outcome in \code{.outcomes}, and the repeated outcome
#' must be the last element of \code{.outcomes}.
#'
#' @details
#' \strong{Scope and assumptions}
#' \itemize{
#'   \item One set of conjoint tasks with exactly two profiles per task (profiles 1 and 2).
#'   \item For multi-set designs, call \code{reshape_projoint()} once per set and bind the results.
#' }
#'
#' \strong{Expected input (Qualtrics K-codes)}
#' \itemize{
#'   \item Wide columns named \code{K-<task>-<attribute>} (attribute names) and
#'         \code{K-<task>-<profile>-<attribute>} (level names), with \code{<task>} in \code{1..n}
#'         and \code{<profile>} in \code{1,2}.
#'   \item Rows with missing \code{K-1-1} are dropped as empty tables (server hiccup safeguard).
#' }
#'
#' \strong{Outcome columns (\code{.outcomes})}
#' \itemize{
#'   \item List all choice variables in the \emph{order asked}. If you include a repeated task,
#'         its outcome must be the \emph{last element}.
#'   \item For base tasks (all but the last element), the function extracts the base task id by
#'         reading the \emph{digits} in each outcome name (e.g., \code{"choice4"}, \code{"Q4"},
#'         \code{"task04"} -> task 4).
#'   \item The set of base task ids extracted from \code{.outcomes} must exactly match the set of
#'         task ids present in the K-codes; otherwise an error is thrown.
#'   \item The repeated base task is inferred as the digits in the \emph{first base outcome}
#'         (i.e., the first element of \code{.outcomes}, excluding the final repeated outcome).
#' }
#'
#' \strong{Choice parsing}
#' \itemize{
#'   \item The selected profile is parsed from the end of each outcome string and
#'         matched to \code{.choice_labels} or \code{.choice_map}. Matching is
#'         case-sensitive and uses the full supplied label, so multi-character
#'         labels are supported.
#'   \item The package cannot infer whether a Qualtrics code or displayed label
#'         refers to profile 1 or profile 2. Analysts must verify that mapping
#'         from the survey instrument or QSF file. Use \code{.choice_map} to make
#'         the verified mapping explicit and auditable.
#'   \item For clarity, use the complete response strings stored in the outcome
#'         columns as the names of \code{.choice_map}. Shorter suffixes are
#'         supported, but the supplied strings must match the ends of all
#'         non-missing outcome values exactly, including case and whitespace.
#'   \item Unrecognized, ambiguous, or whitespace-altered choice values cause an
#'         error. Missing choices also cause an error unless
#'         \code{.allow_missing_choices = TRUE} is supplied explicitly.
#' }
#'
#' \strong{Output}
#' \itemize{
#'   \item A \code{projoint_data} object with:
#'     \itemize{
#'       \item \code{$labels}: map from human-readable \code{attribute}/\code{level} to stable ids
#'             (\code{attribute_id = "att1","att2",...}, \code{level_id = "attX:levelY"}).
#'       \item \code{$data}: tibble with one row per \code{id}–\code{task}–\code{profile}, attribute
#'             columns (named \code{att*}) storing \code{level_id}, \code{selected} (1 if that profile
#'             was chosen; 0 otherwise), \code{agree} (1/0/NA for repeated-task agreement after flip logic),
#'             and any \code{.covariates}. \code{id} is coerced to character; attribute columns are factors.
#'     }
#' }
#'
#' \strong{Filling agreement}
#' \itemize{
#'   \item If \code{.fill = TRUE}, \code{agree} is filled within respondent across tasks in task order,
#'         propagating the observed repeated-task agreement to all tasks for that respondent. This assumes
#'         IRR is respondent-specific and independent of table content.
#' }
#'
#' \strong{Diagnostics}
#' \itemize{
#'   \item \code{dplyr::count(reshaped$data, task, profile)} should show exactly two rows per task.
#'   \item If \code{pj_estimate()} later reports “No rows match the specified attribute/level”, construct QoIs
#'         from \code{reshaped$labels} (use the exact \code{attX:levelY} ids).
#' }
#'
#' @param .dataframe A data frame, preferably from \code{\link{read_Qualtrics}}.
#' @param .outcomes Character vector of outcome column names in the \emph{asked order}.
#'   If a repeated task is used, its outcome must be the \emph{last element}.
#' @param .choice_labels Character vector (default \code{c("A","B")}) giving the two labels that
#'   appear at the end of the outcome strings. Position defines profile identity:
#'   the first label means profile 1 and the second means profile 2. Ignored when
#'   \code{.choice_map} is supplied.
#' @param .choice_map Optional named numeric vector that explicitly maps response
#'   values to Qualtrics profile positions, for example
#'   \code{c("Community A" = 1, "Community B" = 2)}. The names should preferably
#'   be the complete strings stored in the outcome columns; shorter suffixes are
#'   also supported. The numbers refer to the profile positions encoded by
#'   columns such as \code{K-1-1-*} and \code{K-1-2-*}, not to new profile names.
#'   The vector must map exactly two unique labels to profiles 1 and 2. Verify
#'   the mapping against the survey instrument or QSF file because it cannot be
#'   inferred from the CSV alone.
#' @param .alphabet Single character (default \code{"K"}) indicating the Qualtrics prefix.
#' @param .idvar Character (default \code{"ResponseId"}) indicating the respondent id column.
#' @param .repeated Logical (default \code{TRUE}) indicating whether a repeated task is present.
#' @param .flipped Logical (default \code{TRUE}) indicating whether the repeated task flips profiles
#'   before agreement is computed.
#' @param .covariates Optional character vector of respondent-level covariate column names to carry through.
#' @param .fill Logical (default \code{FALSE}). If \code{TRUE}, fills \code{agree} within respondent
#'   across tasks as described under “Filling agreement”.
#' @param .allow_missing_choices Logical (default \code{FALSE}). If \code{FALSE},
#'   any missing outcome stops with an error. If \code{TRUE}, missing choices are
#'   retained as \code{NA} for both profiles in the affected task.
#'
#' @return A \code{projoint_data} object with elements \code{$labels} and \code{$data}; see Details.
#'
#' @seealso \code{\link{make_projoint_data}}, \code{\link{projoint}}
#'
#' @import dplyr
#' @import tidyr
#' @import stringr
#' @import rlang
#' @import tidyselect
#' @importFrom readr parse_number
#'
#' @examples
#' \donttest{
#' # Base tasks asked in numeric order; repeated task corresponds to task 1
#' data(exampleData1)
#' outcomes <- c(paste0("choice", 1:8), "choice1_repeated_flipped")
#' reshaped <- reshape_projoint(
#'   exampleData1,
#'   outcomes,
#'   .choice_map = c("Community A" = 1, "Community B" = 2)
#' )
#' dplyr::count(reshaped$data, task, profile) # should be 2 per task
#' }
#'
#' @export
reshape_projoint <- function(
  .dataframe,
  .outcomes,
  .choice_labels = c("A", "B"),
  .choice_map = NULL,
  .alphabet = "K",
  .idvar = "ResponseId",
  .repeated = TRUE,
  .flipped = TRUE,
  .covariates = NULL,
  .fill = FALSE,
  .allow_missing_choices = FALSE
) {
  # Validate arguments -----------------------------------------------------

  if (!is.data.frame(.dataframe)) {
    stop("`.dataframe` must be a data frame.", call. = FALSE)
  }

  if (!is.character(.outcomes) || length(.outcomes) == 0L ||
      anyNA(.outcomes) || any(.outcomes == "") || anyDuplicated(.outcomes)) {
    stop("`.outcomes` must contain unique, non-missing column names.", call. = FALSE)
  }

  missing_outcomes <- setdiff(.outcomes, names(.dataframe))
  if (length(missing_outcomes) > 0L) {
    stop(
      sprintf(
        "Outcome column(s) not found: %s.",
        paste(missing_outcomes, collapse = ", ")
      ),
      call. = FALSE
    )
  }

  if (!is.character(.idvar) || length(.idvar) != 1L ||
      is.na(.idvar) || !nzchar(.idvar) || !.idvar %in% names(.dataframe)) {
    stop("`.idvar` must name one respondent identifier column.", call. = FALSE)
  }

  respondent_ids <- .dataframe[[.idvar]]
  if (anyNA(respondent_ids)) {
    stop("The respondent identifier contains missing values.", call. = FALSE)
  }
  if (anyDuplicated(respondent_ids)) {
    stop("The respondent identifier must be unique; duplicate IDs were found.", call. = FALSE)
  }

  if (!is.character(.alphabet) || length(.alphabet) != 1L ||
      is.na(.alphabet) || !nzchar(.alphabet)) {
    stop("`.alphabet` must be one non-missing character string.", call. = FALSE)
  }

  logical_arguments <- list(
    .repeated = .repeated,
    .flipped = .flipped,
    .fill = .fill,
    .allow_missing_choices = .allow_missing_choices
  )
  invalid_logical <- vapply(
    logical_arguments,
    function(x) !is.logical(x) || length(x) != 1L || is.na(x),
    logical(1)
  )
  if (any(invalid_logical)) {
    stop(
      sprintf(
        "%s must be either TRUE or FALSE.",
        names(logical_arguments)[which(invalid_logical)[1]]
      ),
      call. = FALSE
    )
  }

  if (.repeated && length(.outcomes) < 2L) {
    stop("A repeated design requires at least one base outcome and one repeated outcome.", call. = FALSE)
  }

  if (!is.null(.covariates)) {
    if (!is.character(.covariates) || anyNA(.covariates) || anyDuplicated(.covariates)) {
      stop("`.covariates` must contain unique, non-missing column names.", call. = FALSE)
    }
    missing_covariates <- setdiff(.covariates, names(.dataframe))
    if (length(missing_covariates) > 0L) {
      stop(
        sprintf(
          "Covariate column(s) not found: %s.",
          paste(missing_covariates, collapse = ", ")
        ),
        call. = FALSE
      )
    }
  }

  if (is.null(.choice_map)) {
    if (!is.character(.choice_labels) || length(.choice_labels) != 2L ||
        anyNA(.choice_labels) || any(!nzchar(.choice_labels)) ||
        anyDuplicated(.choice_labels)) {
      stop("`.choice_labels` must contain exactly two unique, non-missing labels.", call. = FALSE)
    }
    choice_tokens <- .choice_labels
    choice_profiles <- c(1L, 2L)
  } else {
    valid_choice_map <-
      is.numeric(.choice_map) &&
      length(.choice_map) == 2L &&
      !is.null(names(.choice_map)) &&
      !anyNA(.choice_map) &&
      !anyNA(names(.choice_map)) &&
      all(nzchar(names(.choice_map))) &&
      !anyDuplicated(names(.choice_map)) &&
      setequal(as.integer(.choice_map), c(1L, 2L)) &&
      all(.choice_map == as.integer(.choice_map))

    if (!valid_choice_map) {
      stop(
        "`.choice_map` must be a named numeric vector mapping two unique labels to profiles 1 and 2.",
        call. = FALSE
      )
    }
    choice_tokens <- names(.choice_map)
    choice_profiles <- as.integer(.choice_map)
  }

  if (any(trimws(choice_tokens) != choice_tokens)) {
    stop("Choice labels must not contain leading or trailing whitespace.", call. = FALSE)
  }

  ambiguous_tokens <- outer(
    choice_tokens,
    choice_tokens,
    Vectorize(function(x, y) x != y && endsWith(x, y))
  )
  if (any(ambiguous_tokens)) {
    stop("Choice labels must not be suffixes of one another.", call. = FALSE)
  }

  # number of tasks (including the repeated task)
  n_tasks_all <- length(.outcomes)

  # number of tasks (excluding the repeated task)
  if (.repeated == TRUE) {
    n_tasks <- n_tasks_all - 1
  } else {
    n_tasks <- n_tasks_all
  }

  # initial data cleaning
  df <- .dataframe |>
    # Rename the respondent identifier "id"
    dplyr::rename("id" = all_of(.idvar)) |>
    # Sometimes empty conjoint tables are generated due to server problems.
    # The following line removes tasks with no information (using "-1-1")
    # assuming that all contents are empty if "-1-1" is NA.
    dplyr::filter(!is.na(!!rlang::sym(paste0(.alphabet, "-1-1"))))

  # data frame that only includes ID and conjoint-related variables
  temp0 <- df |>
    dplyr::select(id, tidyselect::contains(paste0(.alphabet, "-")))

  # number of columns in temp0
  n_col <- ncol(temp0)

  # c("id", "task", "attribute", "attribute_name")
  # "task" is extracted from the "code."
  temp1 <- temp0 |>
    tidyr::pivot_longer(names_to = "code", values_to = "name", cols = all_of(2:n_col)) |>
    dplyr::filter(stringr::str_detect(code, paste0(.alphabet, "-\\d+-\\d+$"))) |>
    tidyr::separate(code, into = c("x", "task", "attribute"), sep = "\\-") |>
    dplyr::select(-all_of("x")) |>
    rlang::set_names(c("id", "task", "attribute", "attribute_name"))

  # c("id", "task", "profile", "attribute", "level_name")
  # "task" is extracted from the "code."
  temp2 <- temp0 |>
    tidyr::pivot_longer(names_to = "code", values_to = "name", cols = all_of(2:n_col)) |>
    dplyr::filter(stringr::str_detect(code, paste0(.alphabet, "-\\d+-\\d+-\\d+"))) |>
    tidyr::separate(code, into = c("x", "task", "profile", "attribute"), sep = "\\-") |>
    dplyr::select(-all_of("x")) |>
    rlang::set_names(c("id", "task", "profile", "attribute", "level_name"))

  # merge temp1 and temp2 and do further wrangling
  attribute_levels_long <- left_join(temp1, temp2,
    by = c("id", "task", "attribute"),
    multiple = "all"
  ) |>
    dplyr::select(-all_of("attribute")) |>
    dplyr::mutate(across(c(task, profile), as.numeric)) |>
    dplyr::rename(
      "attribute" = attribute_name,
      "level" = level_name
    ) |>
    dplyr::filter(task <= n_tasks)

  # make a list of attributes and levels, as well as their IDs
  labels <- attribute_levels_long |>
    dplyr::arrange(attribute, level) |>
    dplyr::select(attribute, level) |>
    dplyr::distinct() |>
    dplyr::group_by(attribute) |>
    dplyr::mutate(
      attribute_id = dplyr::cur_group_id(),
      level_id = dplyr::row_number()
    ) |>
    dplyr::ungroup() |>
    dplyr::mutate(
      attribute_id = stringr::str_c("att", attribute_id),
      level_id = stringr::str_c(attribute_id, ":level", level_id)
    )

  # make a wide-form data frame with the attribute and level IDs
  attribute_levels_wide <- attribute_levels_long |>
    dplyr::left_join(labels, by = c("attribute", "level")) |>
    dplyr::select(-attribute, -level) |>
    dplyr::rename(
      "attribute" = attribute_id,
      "level" = level_id
    ) |>
    dplyr::group_by(id, task, profile) |>
    tidyr::pivot_wider(names_from = "attribute", values_from = "level") |>
    dplyr::ungroup()

  # Ensure exactly two profiles per base task exist in the design
  chk <- attribute_levels_wide |>
    dplyr::group_by(task) |>
    dplyr::summarise(has_both = setequal(sort(unique(profile)), c(1, 2)), .groups = "drop")

  if (!all(chk$has_both)) {
    bad_tasks <- paste(chk$task[!chk$has_both], collapse = ", ")
    stop(sprintf("Expected profiles 1 and 2 for every task in K-codes; missing for task(s): %s.", bad_tasks))
  }


  ###########################################################################

  # The outcome names (excluding the repeated task) must include task IDs.
  # Example: if tasks presented are {2, 1, 3, 4, 5}, then outcomes might be
  # "Q2","Q1","Q3","Q4","Q5","Q2_repeated". The repeated outcome must be last.

  taskIDs_in_outcomes <- readr::parse_number(.outcomes)

  if (.repeated == TRUE) {
    taskIDs_in_outcomes <- taskIDs_in_outcomes[1:(length(taskIDs_in_outcomes) - 1)]
  }

  # check all numbers in task_order_in_outcomes exist in a sequence of number (tasks)
  # extracted directly from the code.

  all_tasks <- unique(attribute_levels_long$task)

  if (!isTRUE(identical(
    sort(taskIDs_in_outcomes),
    sort(all_tasks)
  ))) {
    stop("The outcome names (excluding the repeated question) should contain the corresponding task IDs.")
  }


  if (.repeated) {
    repeated_task <- taskIDs_in_outcomes[1]

    # keep the repeated task ID
    attribute_levels_repeated <- attribute_levels_wide |>
      dplyr::filter(task == repeated_task)

    # wrangle the response variables
    responses <- df |>
      dplyr::select(id, all_of(.outcomes)) |>
      tidyr::pivot_longer(
        names_to = "outcome_qnum",
        values_to = "response",
        cols = 2:(n_tasks_all + 1)
      ) |>
      dplyr::mutate(task = readr::parse_number(outcome_qnum)) |>
      # tentatively add the order of responses
      dplyr::group_by(id) |>
      dplyr::mutate(
        response_order = row_number(),
        task = ifelse(response_order == max(response_order), repeated_task, task)
      ) |>
      dplyr::ungroup()

    # Ensure each respondent has the expected number of outcomes
    bad <- responses |>
      dplyr::count(id) |>
      dplyr::filter(n != n_tasks_all)
    if (nrow(bad) > 0) stop("Some respondents are missing the repeated outcome.")
  } else {
    # wrangle the response variables
    responses <- df |>
      dplyr::select(id, all_of(.outcomes)) |>
      tidyr::pivot_longer(
        names_to = "outcome_qnum",
        values_to = "response",
        cols = 2:(n_tasks_all + 1)
      ) |>
      dplyr::mutate(task = readr::parse_number(outcome_qnum))
  }


  ###########################################################################

  # # assign the response numbers
  # for (i in 1:n_tasks_all) {
  #   responses <- responses |>
  #     dplyr::mutate(task = ifelse(outcome_qnum == .outcomes[i], i, task))
  # }

  # Parse and validate the profile mapping ---------------------------------

  response_values <- as.character(responses$response)
  missing_choice <- is.na(response_values)

  choice_matches <- vapply(
    choice_tokens,
    function(token) {
      matched <- endsWith(response_values, token)
      matched[is.na(matched)] <- FALSE
      matched
    },
    logical(length(response_values))
  )
  if (is.null(dim(choice_matches))) {
    choice_matches <- matrix(choice_matches, ncol = length(choice_tokens))
  }

  match_count <- rowSums(choice_matches)
  invalid_choice <- !missing_choice & match_count != 1L

  if (any(invalid_choice)) {
    invalid_values <- unique(response_values[invalid_choice])
    shown_values <- utils::head(invalid_values, 5L)
    stop(
      sprintf(
        paste0(
          "Outcome values must end with exactly one verified choice label (%s). ",
          "Invalid value(s): %s%s. Check case and trailing whitespace."
        ),
        paste(sprintf("'%s'", choice_tokens), collapse = ", "),
        paste(sprintf("'%s'", shown_values), collapse = ", "),
        if (length(invalid_values) > length(shown_values)) " (additional values omitted)" else ""
      ),
      call. = FALSE
    )
  }

  if (any(missing_choice) && !.allow_missing_choices) {
    missing_columns <- unique(responses$outcome_qnum[missing_choice])
    stop(
      sprintf(
        paste0(
          "Missing choices were found in %s row(s), including outcome column(s): %s. ",
          "Set `.allow_missing_choices = TRUE` only after confirming that retaining them is appropriate."
        ),
        sum(missing_choice),
        paste(missing_columns, collapse = ", ")
      ),
      call. = FALSE
    )
  }

  selected_profile <- rep(NA_integer_, length(response_values))
  valid_choice <- !missing_choice & match_count == 1L
  selected_profile[valid_choice] <- choice_profiles[
    max.col(choice_matches[valid_choice, , drop = FALSE], ties.method = "first")
  ]

  response_cleaned <- responses |>
    dplyr::mutate(selected = selected_profile) |>
    dplyr::select(-response, -outcome_qnum)

  if (.repeated) {
    # data frame excluding the repeated task
    out1 <- attribute_levels_wide |>
      left_join(
        response_cleaned |>
          dplyr::filter(response_order <= n_tasks),
        by = c("id", "task")
      ) |>
      dplyr::select(-response_order) |>
      dplyr::mutate(selected = ifelse(profile == selected, 1, 0))
  } else {
    # data frame excluding the repeated task
    out1 <- attribute_levels_wide |>
      left_join(
        response_cleaned,
        by = c("id", "task")
      ) |>
      dplyr::mutate(selected = ifelse(profile == selected, 1, 0))
  }


  # add the repeated task (if any)
  if (.repeated == TRUE) {
    # tasks to estimate IRR
    out2 <- attribute_levels_repeated |>
      left_join(
        response_cleaned |>
          dplyr::filter(response_order > n_tasks),
        by = c("id", "task")
      ) |>
      dplyr::select(-response_order) |>
      dplyr::mutate(selected = dplyr::case_when(
        profile == selected & .flipped == FALSE ~ 1,
        profile == selected & .flipped == TRUE ~ 0,
        profile != selected & .flipped == FALSE ~ 0,
        profile != selected & .flipped == TRUE ~ 1
      )) |>
      dplyr::rename("selected_repeated" = selected)

    # merge
    suppressMessages(
      out <- dplyr::left_join(out1, out2) |>
        dplyr::mutate(agree = ifelse(selected == selected_repeated, 1, 0))
    )
  } else if (.repeated == FALSE) {
    out <- out1 |>
      dplyr::mutate(agree = NA)
  }

  # Verify one selected profile per observed choice ------------------------

  choice_integrity <- out |>
    dplyr::group_by(id, task) |>
    dplyr::summarise(
      n_profiles = dplyr::n_distinct(profile),
      n_observed = sum(!is.na(selected)),
      n_selected = sum(selected, na.rm = TRUE),
      .groups = "drop"
    ) |>
    dplyr::filter(
      .data$n_profiles != 2L |
        !(.data$n_observed == 0L |
          (.data$n_observed == 2L & .data$n_selected == 1L))
    )

  if (nrow(choice_integrity) > 0L) {
    stop(
      "Choice integrity check failed: each observed respondent-task must have exactly one selected profile.",
      call. = FALSE
    )
  }

  if (.repeated) {
    repeated_integrity <- out |>
      dplyr::group_by(id, task) |>
      dplyr::summarise(
        n_observed = sum(!is.na(selected_repeated)),
        n_selected = sum(selected_repeated, na.rm = TRUE),
        .groups = "drop"
      ) |>
      dplyr::filter(
        !(.data$n_observed == 0L |
          (.data$n_observed == 2L & .data$n_selected == 1L))
      )

    if (nrow(repeated_integrity) > 0L) {
      stop(
        "Repeated-choice integrity check failed: each observed repeated task must have exactly one selected profile.",
        call. = FALSE
      )
    }
  }

  # Make a data frame that includes respondent covariates -------------------

  covariates <- .dataframe |>
    # Rename the respondent identifier "id"
    dplyr::rename("id" = all_of(.idvar)) |>
    # Select variables needed
    dplyr::select(all_of(c("id", .covariates)))


  # Make a final data frame -------------------------------------------------

  out_final_before_fill <- out |>
    dplyr::left_join(covariates, by = "id") |>
    dplyr::mutate(across(where(is.character), as.factor)) |>
    dplyr::mutate(id = as.character(id)) |>
    dplyr::as_tibble()

  if (.fill == TRUE) {
    out_final <- out_final_before_fill |>
      dplyr::arrange(id, task) |>
      dplyr::group_by(id) |>
      tidyr::fill(agree, .direction = "downup") |>
      dplyr::ungroup()
  } else {
    out_final <- out_final_before_fill |>
      dplyr::arrange(id, task, agree)
  }


  # return the data frame and the variable labels as a list
  return(projoint_data(
    "labels" = labels,
    "data" = out_final
  ))
}

data(exampleData3, package = "projoint")

test_that("reshape_projoint preserves declared task mapping after column permutation", {
  outcomes <- paste0("choice", 1:8)
  permuted_data <- exampleData3[c(
    setdiff(names(exampleData3), outcomes),
    rev(outcomes)
  )]

  canonical <- reshape_projoint(
    exampleData3,
    outcomes,
    .repeated = FALSE
  )
  permuted <- reshape_projoint(
    permuted_data,
    outcomes,
    .repeated = FALSE
  )

  expect_equal(permuted$data, canonical$data)
})

test_that("reshape_projoint supports an explicit choice-to-profile map", {
  outcomes <- paste0("choice", 1:8)

  positional <- reshape_projoint(
    exampleData3,
    outcomes,
    .repeated = FALSE
  )
  explicit <- reshape_projoint(
    exampleData3,
    outcomes,
    .choice_map = c("Community A" = 1, "Community B" = 2),
    .repeated = FALSE
  )
  reversed <- reshape_projoint(
    exampleData3,
    outcomes,
    .choice_map = c("Community A" = 2, "Community B" = 1),
    .repeated = FALSE
  )

  expect_equal(explicit$data, positional$data)
  expect_equal(reversed$data$selected, 1 - positional$data$selected)
})

test_that("reshape_projoint matches full multi-character suffixes", {
  outcomes <- paste0("choice", 1:8)
  labelled_data <- exampleData3
  labelled_data[outcomes] <- lapply(labelled_data[outcomes], function(x) {
    x <- sub("A$", "Profile One", x)
    sub("B$", "Profile Two", x)
  })

  result <- reshape_projoint(
    labelled_data,
    outcomes,
    .choice_map = c("Profile One" = 1, "Profile Two" = 2),
    .repeated = FALSE
  )

  expect_true(all(result$data$selected %in% c(0, 1)))
})

test_that("reshape_projoint rejects unverified or malformed choices", {
  outcomes <- paste0("choice", 1:8)

  invalid <- exampleData3
  invalid$choice1[1] <- "Community Z"
  expect_error(
    reshape_projoint(invalid, outcomes, .repeated = FALSE),
    "Invalid value"
  )

  whitespace <- exampleData3
  whitespace$choice1[1] <- paste0(whitespace$choice1[1], " ")
  expect_error(
    reshape_projoint(whitespace, outcomes, .repeated = FALSE),
    "trailing whitespace"
  )

  expect_error(
    reshape_projoint(
      exampleData3,
      outcomes,
      .choice_labels = c("A", "BA"),
      .repeated = FALSE
    ),
    "suffixes of one another"
  )
})

test_that("reshape_projoint requires explicit approval for missing choices", {
  outcomes <- paste0("choice", 1:8)
  missing <- exampleData3
  affected_id <- missing$ResponseId[1]
  missing$choice1[1] <- NA_character_

  expect_error(
    reshape_projoint(missing, outcomes, .repeated = FALSE),
    "Missing choices"
  )

  result <- reshape_projoint(
    missing,
    outcomes,
    .repeated = FALSE,
    .allow_missing_choices = TRUE
  )
  affected <- result$data[
    result$data$id == affected_id & result$data$task == 1,
  ]

  expect_equal(nrow(affected), 2L)
  expect_true(all(is.na(affected$selected)))
})

test_that("reshape_projoint validates identifiers and mapping arguments", {
  outcomes <- paste0("choice", 1:8)
  duplicated <- dplyr::bind_rows(exampleData3, exampleData3[1, ])

  expect_error(
    reshape_projoint(duplicated, outcomes, .repeated = FALSE),
    "duplicate IDs"
  )
  expect_error(
    reshape_projoint(
      exampleData3,
      outcomes,
      .choice_map = c("A" = 1, "B" = 1),
      .repeated = FALSE
    ),
    "profiles 1 and 2"
  )
  expect_error(
    reshape_projoint(
      exampleData3,
      outcomes,
      .choice_labels = c("A ", "B"),
      .repeated = FALSE
    ),
    "whitespace"
  )
})

#' Save attribute and level labels to a CSV file
#'
#' Saves the attributes and levels (and their order) from a \code{\link{projoint_data}}
#' object, as generated by \code{\link{reshape_projoint}}, to a CSV file. This
#' enables manual reordering and later re-import via \code{\link{read_labels}}.
#'
#' @import readr
#' @import dplyr
#'
#' @param .data A \code{\link{projoint_data}} object.
#' @param .filename A character string giving the name of a CSV file to be written.
#'
#' @return No return value, called for side effects (writes a CSV file).
#'
#' @seealso \code{\link{read_labels}}, \code{\link{reshape_projoint}}
#'
#' @examples
#' \donttest{
#' library(projoint)
#' data(exampleData1)
#' reshaped <- reshape_projoint(
#'   exampleData1,
#'   .outcomes = c(paste0("choice", 1:8), "choice1_repeated_flipped")
#' )
#' tmpfile <- tempfile(fileext = ".csv")
#' save_labels(reshaped, tmpfile)
#' readLines(tmpfile, n = 5)  # show first few lines
#' }
#'
#' @export
save_labels <- function(
    .data,
    .filename
){
  
  (.data$labels |> 
      dplyr::select(level_id, attribute, level) |> 
      dplyr::arrange(level_id) |> 
      dplyr::mutate(order = row_number())
  )|>
    readr::write_csv(.filename)
}  

#' Print method for projoint_tau objects
#'
#' Custom print method for objects of class \code{projoint_tau}, typically created
#' by \code{\link{projoint}} or related functions.
#'
#' @param x An object of class \code{projoint_tau}.
#' @param ... Additional arguments (currently unused).
#'
#' @return No return value; called for its side effect of printing a summary of
#'   the estimated intra-respondent reliability (\eqn{\tau}).
#'
#' @export
#'
#' @examples
#' toy_tau <- structure(
#'   list(irr = data.frame(predicted = 0.413, se = 0.02, n = 200)),
#'   class = "projoint_tau"
#' )
#' print(toy_tau)
print.projoint_tau <- function(x, ...) {
  irr <- x$irr
  if (!is.data.frame(irr) || !"predicted" %in% names(irr) || nrow(irr) < 1) {
    message("Tau: <unavailable>")
    return(invisible(x))
  }
  val <- irr$predicted[1]
  message("Tau estimated using the extrapolation method: ", round(val, 3))
  invisible(x)
}

#' Summary for \code{projoint_data}
#'
#' Custom summary method for objects of class \code{projoint_data}. Prints a brief
#' overview of the main data and attribute-level labels contained in the object.
#'
#' @param object A \code{projoint_data} object.
#' @param ... Additional arguments (currently unused).
#'
#' @return No return value, called for its side effect of printing a summary of the
#'   \code{projoint_data} object to the console.
#'
#' @export
#'
#' @examples
#' \donttest{
#' data(exampleData1)
#' dat <- reshape_projoint(
#'   exampleData1,
#'   .outcomes = c("choice1", "choice2")
#' )
#' summary(dat)
#' }
summary.projoint_data <- function(object, ...) {
  cat("<Summary of projoint_data>\n\n")
  
  if (!is.null(object$data)) {
    cat("Main Data:\n")
    cat("- Number of rows:", nrow(object$data), "\n")
    cat("- Number of columns:", ncol(object$data), "\n")
    cat("- Sample columns:\n")
    print(utils::head(names(object$data), 5))
  } else {
    cat("- Main data is NULL\n")
  }
  
  cat("\n")
  
  if (!is.null(object$labels)) {
    cat("Attribute-Level Labels:\n")
    cat("- Number of levels:", nrow(object$labels), "\n")
    cat("- Attributes found:\n")
    print(unique(object$labels$attribute))
  } else {
    cat("- Labels data is NULL\n")
  }
  
  invisible(object)
}

#' Reads and reformats a Qualtrics csv.
#'
#' @param .file A filename corresponding to a csv downloaded from Qualtrics.
#' @return A data frame suitable for \texttt{reshape_conjoint}
#' @export



read_Qualtrics <- function(.file){

  if(!file.exists(.file)){
    stop("Error: The specified file cannot be found.")
  }

  colnames <- .file %>%
    readr::read_csv(show_col_types = FALSE) %>%
    names()

  qualtrics <- readr::read_csv(.file, skip = 2, show_col_types = FALSE)
  colnames(qualtrics) <- colnames

  return(qualtrics)

}

test_that("the package citation identifies the current CRAN release", {
  package_citation <- citation("projoint")[[1]]

  expect_match(
    paste(toBibtex(package_citation), collapse = "\n"),
    "@Manual\\{projoint,"
  )
  expect_equal(
    package_citation$title,
    "projoint: Conjoint Analysis with Reliability Correction and Visualization"
  )
  expect_equal(package_citation$year, "2026")
  expect_equal(
    package_citation$note,
    paste("R package version", as.character(packageVersion("projoint")))
  )
  expect_equal(package_citation$organization, "Comprehensive R Archive Network (CRAN)")
  expect_equal(package_citation$doi, "10.32614/CRAN.package.projoint")
  expect_equal(package_citation$url, "https://CRAN.R-project.org/package=projoint")
})

---
title: "Choice-Level Analysis"
output: rmarkdown::html_vignette
description: >
  Why choice-level analysis offers deeper insights compared to standard profile-level analysis.
vignette: >
  %\VignetteIndexEntry{Choice-Level Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

## **Why Choice-Level Analysis?**

💡 **Choice-level analysis is simpler, easier, and more powerful than profile-level analysis.**

- 🧠 Conjoint designs began in market research and psychology, where respondents **rated** two profiles (e.g., products). Each rating was treated as an **independent observation**, creating the **profile-level design** with `2 × n` rows for `n` respondents. 
- 🔍 Social scientists later used conjoint designs for **choices** instead of ratings but kept the same structure with a **single binary outcome**. This shift introduced **statistical and conceptual issues**.

---

### ⚠️ **Problems with Profile-Level Analysis**

🚫 Profile-level analysis forces researchers to correct a dependence that they created themselves.

- 🔁 **Redundant structure:** Each choice task generates **two rows** per task for each respondent even though there is only **one independent choice**.  
- 🔗 **Mechanical dependence:** Selecting one profile **necessarily implies rejecting the other**, violating the independence assumption.  
- 🧩 **Artificial complexity:** Analysts must correct for this dependence using **complicated statistical adjustments**, even though it arises solely from how the data were organized—not from respondents’ behavior.


---

### ✅ **Advantages of Choice-Level Analysis**

In contrast, **choice-level analysis** organizes data by respondent *decisions* rather than profiles.

- 📋 Each unit of observation represents **a choice** for a given task per respondent.  
- ⚡ The outcome variable reflects **which profile was chosen** in that task.  
- 🧠 This data structure allows researchers to model the choice **conditional on the full comparison**.

🎯 **Choice-level analysis** directly models the respondent’s decision between two (or more) alternatives,  
capturing the true structure of the conjoint task.

---

## **Key Issues and Applications**

- Profile-level estimands like **AMCEs** assume that each profile is generated independently and ignores how respondents evaluate one profile *relative* to another.  This limits the types of questions researchers can ask.

- Choice-level analysis allows researchers to explore questions that **explicitly depend on the comparison between profiles**, such as:

<details>
<summary><strong>Examples of Choice-Level Research Questions</strong></summary>

- 🗳️ Do voters choose a **white** candidate over a **non-white** candidate?  
  (The levels—white vs. Asian, Black, Hispanic—always differ between profiles.)

- 🌐 Do **Asian Democrat respondents** prefer an **Asian Republican** over a **white Democrat**?  
  (Profiles are intentionally designed with multiple correlated attributes.)

- 📊 Do voters care about **electability**?  
  (The two percentages representing win probability must sum to 100.)

- ⚖️ Do voters prefer the **status quo** over a **policy proposal**?  
  (One profile is fixed while the other varies across tasks.)

- 🧭 How much do voters prefer **extreme left-leaning** or **extreme right-leaning** policies?  
  (Attributes are consistently positioned on the ideological spectrum.)

</details>

Furthermore, when individuals compare profiles side-by-side, their evaluations are often **psychologically influenced by the alternative**, such as through **assimilation or contrast effects**  
(see *Horiuchi and Johnson 2025*).

---

## **Why Move to Choice-Level Analysis?**

🔍 **Choice-level analysis models the decision *between* two profiles, not the evaluation of a single profile.**

This structure more closely mirrors:

- 🧠 **Real-world decision-making**, where people choose among competing alternatives.  
- 🔄 **Comparative cognition**, where evaluations depend on the context of available options.  
- 🎛️ **Tradeoff reasoning**, where respondents weigh attribute combinations jointly.

Hence, rather than estimating the probability of selecting an isolated profile, choice-level analysis estimates the **probability of choosing one profile over another**, conditional on all attributes involved.

✅ Mirrors real-world behavior  
✅ Captures comparative judgment and psychological context  
✅ Reveals authentic tradeoffs and priorities  

---

## **Summary**

| Profile-Level Analysis | Choice-Level Analysis |
|:-----------------------|:----------------------|
| Treats profiles as independent | Models the decision *between* profiles |
| Ignores comparative context | Captures mutual influence of options |
| May blur or bias tradeoffs | Highlights actual tradeoffs |
| Can misstate uncertainty | Produces more interpretable estimates |
| Requires complex correction methods | Works with simple and transparent models |

---

## **Key Takeaway**

🚀 If your conjoint design presents respondents with **two or more profiles for comparison**,  
then **choice-level analysis is essential for valid, interpretable, and psychologically realistic inference**.

It provides:

- **Deeper insights into human decision-making**  
- **Cleaner estimation procedures**  
- **Closer correspondence to real-world behavior**

---

## 📚 **References**

- **Clayton, Horiuchi, Kaufman, King, & Komisarchik (Forthcoming).**  
  *Correcting Measurement Error Bias in Conjoint Survey Experiments.*  
  _Forthcoming, American Journal of Political Science._  
  [Preprint available](https://gking.harvard.edu/conjointE)

- **Horiuchi & Johnson (2025).**  
  *Advancing Conjoint Analysis: Delving Further into Profile Comparisons.*  
  _Work in progress._

---
title: "Read and Wrangle Your Data"
output: rmarkdown::html_vignette
description: >
  Read your Qualtrics data and wrangle it in R.
vignette: >
  %\VignetteIndexEntry{Read and Wrangle Your Data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
library(projoint)
```

## 📥 Read Your Data

Before you can reshape or analyze your conjoint survey data, you first need to **import it into R**.  In **projoint**, use the `read_Qualtrics()` function to quickly read properly formatted Qualtrics files.

---

## 🚀 Read Workflow

<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;">**1. Export your survey responses from Qualtrics**</summary>

When exporting from Qualtrics:

- Click **"Download Data"**.
- Choose **CSV** format.
- Critically, select **"Use choice text"** rather than coded values.

⚡ If you skip selecting "Use choice text," your conjoint data may fail to load properly!

</details>

<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;">**2. Load essential packages**</summary>

```{r, warning=FALSE, message=FALSE}
library(tidyverse)
library(projoint)
```
</details>

<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;">**3. Read your CSV file into R using `read_Qualtrics()`**</summary>

```r
# Example: If your file is located in a "data" folder
data <- read_Qualtrics("data/your_file.csv")
```

`read_Qualtrics()` automatically removes the question-text and `ImportId`
metadata rows used by current Qualtrics exports. It also recognizes legacy
exports with only a question-text row while preserving the original variable
names and column order.

Or, if using an example bundled with **projoint**:

```{r, eval=TRUE, echo=FALSE}
data <- read_Qualtrics(
  system.file("extdata", "mummolo_nall_replication.csv", package = "projoint")
)
```

```{r}
# Inspect the imported data:
data
```
</details>

---

```{r fig-setup, include=FALSE}
# Global default settings for all figures
knitr::opts_chunk$set(
  fig.width = 7,
  fig.height = 5,
  fig.align = "center",
  dpi = 300  # Optional: high-resolution plots
)

# Helper functions for special figure sizes
narrow_fig <- function() list(fig.width = 5, fig.height = 4)
wide_fig <- function() list(fig.width = 8, fig.height = 5)
tall_fig <- function() list(fig.width = 6, fig.height = 7)

# Load libraries
library(projoint)
data(exampleData1, package = "projoint")
data(exampleData2, package = "projoint")
data(exampleData3, package = "projoint")
data(exampleData1_labelled_tibble, package = "projoint")
data(out1_arranged, package = "projoint")

```

## 🛠️ Wrangle Your Data

Preparing your data correctly is one of the most important steps in conjoint analysis. Fortunately, the `reshape_projoint()` function in **projoint** makes this easy.

---

## 🚀 Wrangle Workflow

<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;">**1. Reshape Your Data**</summary>
<div>

> **Outcome naming & order (important)**
> 
> - List `.outcomes` in the **order questions were asked**.  
> - If you have a repeated task, its outcome must be the **last element**.  
> - For base tasks (all but last), the function reads the **digits** in each name as the task id (e.g., `"choice4"`, `"Q4"`, `"task04"` → task 4).  
> - The **repeated base task** is inferred from the **first base outcome’s digits**. The repeated outcome itself **need not** contain digits—only its position (last) matters.  
> - Outcome strings must end with verified choice labels. For an auditable
>   mapping, use `.choice_map = c("A" = 1, "B" = 2)` (or the corresponding
>   mapping from your instrument). **projoint cannot infer whether an exported
>   label or numeric recode refers to profile 1 or profile 2.**
> - Invalid labels, trailing whitespace, and missing choices now stop with an
>   informative error. Retain missing choices only after review by setting
>   `.allow_missing_choices = TRUE`.


### Example (Flipped Repeated Task)

```{r, error=TRUE}
outcomes <- paste0("choice", 1:8)
outcomes1 <- c(outcomes, "choice1_repeated_flipped")

out1 <- reshape_projoint(
  .dataframe = exampleData1,
  .outcomes = outcomes1,
  .choice_map = c("A" = 1, "B" = 2),
  .alphabet = "K",
  .idvar = "ResponseId",
  .repeated = TRUE,
  .flipped = TRUE
)
```

**Key Arguments**:

- `.outcomes`: Outcome columns (include repeated task last)
- `.choice_map`: Verified mapping from response labels to profiles
- `.idvar`: Respondent ID variable
- `.alphabet`: Variable prefix ("K")
- `.repeated`, `.flipped`: If repeated task exists and is flipped

</div>
</details>

<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;">**2. Variations: Repeated vs. Non-Repeated**</summary>

**Not-Flipped Repeated Task**

```{r}
outcomes <- paste0("choice", 1:8)
outcomes2 <- c(outcomes, "choice1_repeated_notflipped")
out2 <- reshape_projoint(
  .dataframe = exampleData2,
  .outcomes = outcomes2,
  .repeated = TRUE,
  .flipped = FALSE
)
```

**No Repeated Task**

```{r}
outcomes <- paste0("choice", 1:8)
out3 <- reshape_projoint(
  .dataframe = exampleData3,
  .outcomes = outcomes,
  .repeated = FALSE
)
```

</details>

<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;">**3. The `.fill` Argument: Should You Use It?**</summary>

Use `.fill = TRUE` to "fill" missing values based on IRR agreement.

```{r}
fill_FALSE <- reshape_projoint(
  .dataframe = exampleData1,
  .outcomes = outcomes1,
  .fill = FALSE
)

fill_TRUE <- reshape_projoint(
  .dataframe = exampleData1,
  .outcomes = outcomes1,
  .fill = TRUE
)
```

Compare:

```{r}
selected_vars <- c("id", "task", "profile", "selected", "selected_repeated", "agree")
fill_FALSE$data[selected_vars]
fill_TRUE$data[selected_vars]
```

**Tip:**  
- Use `.fill = TRUE` for small-sample or subgroup analysis (helps increase power).  
- Use `.fill = FALSE` (default) when in doubt for safer estimates.

</details>

<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;">**4. What If Your Data Is Already Clean?**</summary>

If you already have a clean dataset, use `make_projoint_data()`:

```{r}
out4 <- make_projoint_data(
  .dataframe = exampleData1_labelled_tibble,
  .attribute_vars = c(
    "School Quality", "Violent Crime Rate (Vs National Rate)",
    "Racial Composition", "Housing Cost",
    "Presidential Vote (2020)", "Total Daily Driving Time for Commuting and Errands",
    "Type of Place"
  ),
  .id_var = "id",
  .task_var = "task",
  .profile_var = "profile",
  .selected_var = "selected",
  .selected_repeated_var = "selected_repeated",
  .fill = TRUE
)
```

Preview:

```{r}
out4
```

</details>

<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;">**5. Arranging Attribute and Level Labels**</summary>

To reorder or relabel attributes:

1. Save labels:

```{r, eval=FALSE}
save_labels(out1, "temp/labels_original.csv")
```

2. Edit the CSV (change `order`, label columns; leave `level_id` untouched)

3. Save it as "labels_arranged.csv" or something else.

4. Reload labels:

```{r, eval=FALSE}
out1_arranged <- read_labels(out1, "temp/labels_arranged.csv")
```

```{r, eval=TRUE, eco=FALSE}
data(out1_arranged, package = "projoint")
```

Compare using our example:

```{r}
mm <- projoint(out1, .structure = "profile_level", .estimand = "mm")
plot(mm)
```

```{r}
mm <- projoint(out1_arranged, .structure = "profile_level", .estimand = "mm")
plot(mm)
```

</details>

---

🏠 **Home:** [Home](https://yhoriuchi.github.io/projoint/index.html)

---
title: "Correct Measurement-Error Bias"
output: rmarkdown::html_vignette
description: >
  Why you should identify and correct measurement error bias in conjoint analysis
vignette: >
  %\VignetteIndexEntry{Correct Measurement-Error Bias}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

## **Why Correct for Measurement Error?**

- In conjoint experiments, **measurement error** is pervasive but often overlooked.  
- Respondents may:
  - Misread attributes
  - Misunderstand levels
  - Click randomly
  - Forget information across tasks
- As a result, respondents' recorded choices often refelect random noise.   
- This **random error** leads to attenuate **true prefernces** (or true effects).  

---

## **The Consequences of Ignoring Measurement Error**

| Without Correction             | With Correction                  |
|:--------------------------------|:----------------------------------|
| Underestimates true preferences | Provides more accurate preferences |
| Falsely suggests indifference | Recovers meaningful trade-offs |
| Misleads theory building and application | Provides accurate, unbiased insights |

---

## **How projoint Corrects for Measurement Error**

- **Estimates the intra-respondent reliability (IRR)** based on responses to a repeated task
- **Adjusts marginal means (MMs) and average marginal component effects (AMCEs)** accordingly
- **Provides corrected estimates** that better reflect respondents’ true preferences
- Corrected estimates reveal the true magnitude of effects, **improving both theoretical and applied inferences** in political science, marketing, and other fields.

✅ No additional respondent burden (just one repeated task)  
✅ Minimal survey design changes  
✅ Massive improvements in accuracy

---

## **Key Takeaway**

> 🧠 Measurement error systematically *biases* results.  
> 
> 🔥 Correcting for measurement error reveals true preferences, sharper trade-offs, and prevents misleading inferences.

---

## 📚 Key Reference

- **Clayton, Horiuchi, Kaufman, King, Komisarchik (Forthcoming).**  
  "Correcting Measurement Error Bias in Conjoint Survey Experiments."  
  _Forthcoming, American Journal of Political Science._  
  [Pre-Print Available](https://gking.harvard.edu/conjointE)

---

---
title: "FAQ Page"
output: rmarkdown::html_vignette
description: >
  Frequently Asked Questions
vignette: >
  %\VignetteIndexEntry{FAQ Page}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
library(projoint)
library(dplyr)
library(ggplot2)
library(patchwork)
```


## What's the difference between "profile-level" and "choice-level" conjoint analysis??

<details>
<summary>Choice-level analysis is simpler, easier, and more powerful than profile-level analysis.</summary>

Conjoint designs originated in market research and psychology where each respondent is asked to rate each of two different profiles (e.g., two products). Each of the two ratings provided separate information, and the two are analyzed as separate observations. Researchers with this <em>profile-level</em> design find it convenient to arrange their data with one profile per row, and thus twice as many rows as respondents.

<p>Unfortunately, when social scientists adopted the conjoint survey design, they kept the same profile-level design but changed the outcome measure from separate ratings to a single choice between the two profiles (e.g., to reflect a voter choice between two candidates). In this situation, respondents asked to make one choice between the two profiles that are exactly dependent, as choosing one necessarily meant not choosing the other (e.g., in a two-candidate partisan election, one observation would be "Democrat" and the other would be "not the Republican"). Using this profile-level design with 2*n rows but only n independent observations requires the introduction of complicated statistical procedures to correct for the dependence induced solely by the researcher's decision to organize the data in this complicated way.</p>

<p>We recommend the much simpler and more powerful <em>choice-level</em> design.  The idea is to arrange data at the level of the respondent's choice, so that each row in the data matrix includes information about one choice (and both profiles together, with n observations and n rows). Our <a href="https://gking.harvard.edu/conjointE" target="_blank" class="external-link"><em>AJPS</em> article</a> clarifies this point, shows how this choice-level analysis vastly simplifies the notation, statistical analysis procedures, and intuition, and greatly expands the substantive questions conjoint analysis be used to answer.</p>
</details>

## What if I have profile-level data?

<details><summary>Transform your data to choice-level, which is simpler and more powerful. To replicate old analyses, you can also use our profile-level tools</summary>
  <details><summary><b>Profile-Level MMs (All Levels)</b></summary>
  
```{r, echo = FALSE}
data("exampleData1")
outcomes <- paste0("choice", 1:8)
outcomes <- c(outcomes, "choice1_repeated_flipped")
out1 <- reshape_projoint(exampleData1, outcomes)
```

```{r}
mm0 <- projoint(out1, .structure = "profile_level", .estimand = "mm")
print(mm0)
summary(mm0)
```
  
  </details>

  <details><summary><b>Profile-Level MMs (Specific Level)</b></summary>

```{r}
qoi_1 <- set_qoi(
  .structure = "profile_level",
  .estimand = "mm",
  .att_choose = "att1",
  .lev_choose = "level1"
)

mm1 <- projoint(out1, .qoi = qoi_1)
print(mm1)
summary(mm1)
```

  </details>
  
  <details><summary><b>Profile-Level MMs (Specific Level, Manual IRR)</b></summary>

```{r}
mm1b <- projoint(out1, .qoi = qoi_1, .irr = 0.75)
print(mm1b)
summary(mm1b)
```

  </details>
  
  <details><summary><b>Profile-Level AMCEs (All Levels)</b></summary>

```{r}
amce0 <- projoint(out1, .structure = "profile_level", .estimand = "amce")
print(amce0)
summary(amce0)
```

</details>

<details><summary><b>Profile-Level AMCEs (Specific Level)</b></summary>

```{r}
qoi_3 <- set_qoi(
  .structure = "profile_level",
  .estimand = "amce",
  .att_choose = "att1",
  .lev_choose = "level3",
  .att_choose_b = "att1",
  .lev_choose_b = "level1"
)

amce1 <- projoint(out1, .qoi = qoi_3)
print(amce1)
summary(amce1)
```

</details>

<details><summary><b>Profile-Level AMCEs (Specific Level, Manual IRR)</b></summary>

```{r}
amce1b <- projoint(out1, .qoi = qoi_3, .irr = 0.75)
print(amce1b)
summary(amce1b)
```

</details>

  <details>
    <summary>💡 Tip: When to Use <code>.by_var</code><a class="anchor" aria-label="anchor" href="#tip-when-to-use--by_var"></a></summary>
    Use <code>.by_var</code> <strong>only</strong> when comparing
    <strong>profile-level MMs</strong> between two groups (e.g., Democrats
vs. Republicans).

For AMCEs or choice-level quantities, <code>.by_var</code> is not
currently supported.
  </details>
  
  
### 📈 Profile-Level MMs and AMCEs

<details>
<summary><b>Estimate</b></summary>

```{r}
data("out1_arranged")
mm   <- projoint(out1_arranged, .structure = "profile_level") 
amce <- projoint(out1_arranged, .structure = "profile_level", .estimand = "amce")
```

</details>

<details>
<summary><b>Visualize</b></summary>

* Marginal Means

```{r, fig.width = 6, fig.height = 6}
plot(mm)
```

* AMCEs

```{r, fig.width = 6, fig.height = 6}
plot(amce)
```
  
</details>

<details><summary><b>Profile-Level Subgroup Comparison: White vs. Non-White Respondents</b></summary>

```{r, fig.height = 6.5, fig.width = 7}
outcomes <- c(paste0("choice", 1:8), "choice1_repeated_flipped")

df <- exampleData1 |> mutate(white = ifelse(race == "White", 1, 0))

df_0 <- df |> filter(white == 0) |> reshape_projoint(outcomes)
df_1 <- df |> filter(white == 1) |> reshape_projoint(outcomes)
df_d <- df |> reshape_projoint(outcomes, .covariates = "white")

data_file <- system.file("extdata", "labels_arranged.csv", package = "projoint")
if (data_file == "") stop("File not found!")

df_0 <- read_labels(df_0, data_file)
df_1 <- read_labels(df_1, data_file)
df_d <- read_labels(df_d, data_file)

out_0 <- projoint(df_0, .structure = "profile_level")
out_1 <- projoint(df_1, .structure = "profile_level")
out_d <- projoint(df_d, .structure = "profile_level", .by_var = "white")

plot_0 <- plot(out_0)
plot_1 <- plot(out_1)
plot_d <- plot(out_d, .by_var = TRUE)

plot_0 +
  coord_cartesian(xlim = c(0.2, 0.8)) +
  labs(title = "Non-white", x = "AMCE") +
  theme(plot.title = element_text(hjust = 0.5)) +

plot_1 +
  coord_cartesian(xlim = c(0.2, 0.8)) +
  labs(title = "White", x = "AMCE") +
  theme(axis.text.y = element_blank(), plot.title = element_text(hjust = 0.5)) +

plot_d +
  coord_cartesian(xlim = c(-0.4, 0.4)) +
  labs(title = "Difference", x = "Difference") +
  theme(axis.text.y = element_blank(), plot.title = element_text(hjust = 0.5))
```

</details>

</details>


## Is there other conjoint software??

<details>
  <summary>See Anton Strezhnev's <strong>Conjoint Survey Design Tool</strong> (Link: <a href="https://github.com/astrezhnev/conjointsdt/" class="external-link" target="_blank">conjointSDT</a>)</summary>
  
  ### 1. Generate a JavaScript or PHP randomizer

Anton Strezhnev's **Conjoint Survey Design Tool** (Link: [conjointSDT](https://github.com/astrezhnev/conjointsdt/)) produces a JavaScript or PHP randomizer.

#### JavaScript

The JavaScript randomizer can be inserted into the first screen of your Qualtrics survey using **Edit Question JavaScript**. Example screenshot:  

![](https://yhoriuchi.github.io/projoint/man/figures/embedded_js.png){#id .class width=80% height=80%}

- Example JavaScript: [Download here](https://raw.githubusercontent.com/yhoriuchi/projoint/master/data-raw/example.js)

The JavaScript runs internally within Qualtrics and generates *embedded fields* for each conjoint task.  
For example:

- `"K-1-1-7"` = value for the 7th attribute, first profile, first task
- `"K-5-2-5"` = value for the 5th attribute, second profile, fifth task

#### PHP

Alternatively, the PHP randomizer must be hosted externally.

Example hosted on our server:  
https://www.horiuchi.org/php/ACHR_Modified_2.php  
(PHP file [here](https://raw.githubusercontent.com/yhoriuchi/projoint/master/data-raw/ACHR_Modified_2.php))

This method was used in:  
[Agadjanian, Carey, Horiuchi, and Ryan (2023)](https://www.nowpublishers.com/article/Details/QJPS-21119)

---

### 2. Modify your JavaScript or PHP randomizer

You may want to add constraints — for example, **prevent ties** between profiles.  
To do this, you can manually modify your JavaScript or PHP.

In the future, **projoint** will offer easier ways to add constraints!  
Until then, resources like [OpenAI’s GPT-4](https://openai.com/gpt-4) can help you edit scripts.

Example PHP snippet ensuring racial balance between profiles:

```php
$treat_profile_one = "B-" . (string)$p . "-1-" . (string)$treat_number;
$treat_profile_two = "B-" . (string)$p . "-2-" . (string)$treat_number;
$cond1 = $returnarray[$treat_profile_one] == "White" && $returnarray[$treat_profile_two] == $type;
$cond2 = $returnarray[$treat_profile_two] == "White" && $returnarray[$treat_profile_one] == $type;

if ($cond1 or $cond2) {
  $complete = True;
}
```

If you have good examples of manual constraints, please email [Yusaku Horiuchi](mailto:yusaku.horiuchi@gmail.com)!

---

### 3. Add conjoint tables with embedded fields in Qualtrics

After generating the randomizer, you must create **HTML tables** displaying embedded fields for each task.

Example of the first task:

![](https://yhoriuchi.github.io/projoint/man/figures/screenshot_first.png){#id .class width=80% height=80%}

- Example HTML file: [task_first.html](https://raw.githubusercontent.com/yhoriuchi/projoint/master/data-raw/task_first.html)

Each conjoint study typically includes **5-10 tasks**.

The embedded fields update across tasks:  
e.g., `"K-1..."` for Task 1, `"K-2..."` for Task 2, and so on.

---

#### Adding a repeated task (recommended!)

It’s easy to create a **repeated task** for intra-respondent reliability (IRR) estimation:

- Copy the HTML for Task 1 later into the survey (e.g., after Task 5)
- **Flip** Profile 1 and Profile 2 (swap the embedded field digits)

Example repeated task:

![](https://yhoriuchi.github.io/projoint/man/figures/screenshot_last.png){#id .class width=80% height=80%}

- Example HTML: [task_repeated.html](https://raw.githubusercontent.com/yhoriuchi/projoint/master/data-raw/task_repeated.html)

---

### 4. Sample Qualtrics survey template

We provide a full **Qualtrics QSF file** ready to use!

```{r, echo=FALSE, warning = F}
library(downloadthis)
download_link(
  link = "https://raw.githubusercontent.com/yhoriuchi/projoint/master/data-raw/CHKKK_Mummolo_and_Nall_full_replication_W1_Lucid_-_August_2021.qsf",
  button_label = "Download QSF file",
  button_type = "danger",
  has_icon = TRUE,
  icon = "fa fa-save",
  self_contained = FALSE
)
```

<br>

This file recreates a standard conjoint survey setup based on Mummolo and Nall (2017).
  
</details>

---
title: "Analyze and Visualize Important QOIs"
output: rmarkdown::html_vignette
description: >
  How to estimate and correct marginal means (MMs) or average marginal component effects (AMCEs), including predicting IRR if necessary. Visualize these and other QOIs of interest.
vignette: >
  %\VignetteIndexEntry{Analyze and Visualize Important QOIs}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
library(projoint)
library(dplyr)
library(ggplot2)
library(patchwork)
```

## 🎯 Estimate Corrected MMs or AMCEs

In conjoint analysis, default MMs and AMCEs can be **biased** due to measurement error from intra-respondent variability.

**projoint** corrects for this bias automatically.

The following instructions apply to choice-level data. What if you have profile-level data?<sup><a href="faq.html#what-if-i-have-profile-level-data" target="_blank" style="text-decoration: none;">&#9432;</a></sup> Our <a href="faq.html" target="_blank">FAQ Page</a> has instructions to estimate and visualize profile-level QOIs.

---

## 📦 Prepare Example Data

> **Outcome naming & order (important)**
> 
> - List `.outcomes` in the **order questions were asked**.  
> - If you have a repeated task, its outcome must be the **last element**.  
> - For base tasks (all but last), the function reads the **digits** in each name as the task id (e.g., `"choice4"`, `"Q4"`, `"task04"` → task 4).  
> - The **repeated base task** is inferred from the **first base outcome’s digits**. The repeated outcome itself **need not** contain digits—only its position (last) matters.  
> - Outcome strings should end with your choice labels; by default we parse the **last character** and expect `"A"`/`"B"`. If your survey uses `"1"`/`"2"` (or other endings), set `.choice_labels` accordingly.

**Examples**

```{r}
# Standard order; repeated = task 1
data("exampleData1")
outcomes <- c(paste0("choice", 1:8), "choice1_repeated_flipped")
out1 <- reshape_projoint(exampleData1, outcomes)
```

---

## 🛠️ Why Use IDs (e.g., `att1`, `level1`)?

Before estimating quantities, it’s important to understand how attribute and level IDs work inside projoint.

<details>
We recommend working with **attribute IDs** rather than actual text labels because:

- Safer against special characters, languages, or typos
- Allows **multiple attributes** to have identical labels (e.g., "High" for both "Teaching Quality" and "Research Quality")

Check attribute-level mappings:

```{r}
out1$labels
```

You can also save these labels for easier editing:

```r
save_labels(out1, "labels.csv")
```
</details>
---

## 📈 Estimate Marginal Means (MMs)

<details><summary><b>Choice-Level MMs (Specific Level)</b></summary>

Suppose you want to estimate, within a given profile pair, the probability of choosing a profile that includes "40% of pre-tax income" (`level3`) for Housing Cost (`att1`) rather than one that includes "15% of pre-tax income" (`level1`) for the same attribute, averaging over all combinations of the other attributes and across respondents; then use the following code:

```{r}
qoi <- set_qoi(
  .structure = "choice_level",
  .att_choose = "att1",
  .lev_choose = "level3",
  .att_notchoose = "att1",
  .lev_notchoose = "level1"
)

mm2 <- projoint(out1, qoi)
print(mm2)
summary(mm2)
```

</details>
---

## 📉 Estimate AMCEs

<details><summary><b>Choice-Level AMCEs (Specific Level)</b></summary>

Suppose you want to quantify how the choice probability **changes** between the following profile pairs:

- choosing a profile that includes "40% of pre-tax income" (`level3`) for Housing Cost (`att1`) versus one that includes "15% of pre-tax income" (`level1`) for Housing Cost (`att1`); **and**
- [baseline] choosing a profile that includes "30% of pre-tax income" (`level2`) for Housing Cost (`att1`) versus one that includes "15% of pre-tax income" (`level1`) for Housing Cost (`att1`); 

averaging over all combinations of the other attributes and across respondents. Then write the following code:

```{r}
qoi <- set_qoi(
  .structure = "choice_level",
  .estimand = "amce",
  .att_choose = "att1",
  .lev_choose = "level3",
  .att_notchoose = "att1",
  .lev_notchoose = "level1",
  .att_choose_b = "att1",
  .lev_choose_b = "level2",
  .att_notchoose_b = "att1",
  .lev_notchoose_b = "level1"
)

amce2 <- projoint(out1, qoi)
print(amce2)
summary(amce2)
```

</details>
---

## 🔎 Predict Intra-Respondent Reliability (IRR)

If your design **does not include** a repeated task, you can **predict** IRR using predict_tau(), based on observed respondent behavior.

<details><summary><b>Predict IRR Using `predict_tau()`</b></summary>

```{r}
data(out1_arranged)
predicted_irr <- predict_tau(out1_arranged)

print(predicted_irr)
summary(predicted_irr)
plot(predicted_irr)
```

</details>
---

```{r fig-setup, include=FALSE}
# Global default settings for all figures
knitr::opts_chunk$set(
  fig.width = 6,
  fig.height = 3,
  fig.align = "center",
  dpi = 300  # Optional: high-resolution plots
)

# Helper functions for special figure sizes
narrow_fig <- function() list(fig.width = 5, fig.height = 4)
wide_fig <- function() list(fig.width = 8, fig.height = 5)
tall_fig <- function() list(fig.width = 6, fig.height = 7)

```

```{r, echo=FALSE}
library(projoint)
library(ggplot2)

data(out1_arranged, package = "projoint")
```

## 🎨 Visualize MMs or AMCEs

The **projoint** package provides ready-to-publish plotting tools for conjoint analysis results.

<span style="color:DarkRed"><b>Note:</b></span> The current version of **projoint** supports plotting choice-level MMs only.  
Support for **choice-level AMCEs** will be available in future updates!

---

### ⚖️ Choice-Level Analysis

<details>
<summary><b>Estimate</b></summary>

- Specify your quantity of interest:
```{r}
qoi_mm <- set_qoi(
  .structure = "choice_level", # default
  .att_choose = "att1", 
  .lev_choose = "level1", 
  .att_notchoose = "att1", 
  .lev_notchoose = "level3"
)
```

- Estimate
```{r}
choice_mm <- projoint(
  .data = out1_arranged, 
  .qoi = qoi_mm, 
  .ignore_position = TRUE
)
```
</details>

<details>
<summary><b>Visualize (Levels) </b></summary>

```{r, fig.width = 6, fig.height = 3}
plot(choice_mm)
```
</details>

<details>
<summary><b>Visualize (Differences)</b></summary>

```{r, fig.width = 8, fig.height = 2}
plot(choice_mm, .type = "pointrange")
```
</details>

---

🏠 **Home:** [Home](https://yhoriuchi.github.io/projoint/index.html)

---
title: "Explore and Compare Further"
output: rmarkdown::html_vignette
description: >
  Advanced choice-level analyses using **projoint**: direct trade-offs, level collapsing,
  and subgroup comparisons (MMs). Designed to complement the main "Analyze and Visualize Important QOIs" vignette.
vignette: >
  %\VignetteIndexEntry{Explore and Compare Further}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
library(projoint)
library(dplyr)
library(ggplot2)
library(patchwork)
```

## 🌟 Explore and Compare Further

Choice-level analysis opens the door to many **new research questions** that traditional profile-level analysis often overlooks. Below, we demonstrate how to estimate **deeper quantities** and **compare subgroups** effectively.

Depending on your objectives, you may want to reorganize the data in a `projoint_data` object. The helper function below is internal to the package, but you can call it explicitly in your script.
```{r}
projoint_data <- function(labels, data) {
  structure(
    list(labels = labels, data = data),
    class = "projoint_data"
  )
}
```

---

## 📦 Setup

We use the already wrangled and cleaned data `out1_arranged`.

```{r}
data("out1_arranged")
out1_arranged$labels
```

---

## ⚖️ Explore: Compare Trade-offs Directly

<details><summary><b>Example: Low Housing Costs vs. Low Crime Rates</b></summary>

**Goal.** Compare choices between two *joint* profiles:

- **Low housing cost** but **high violent‑crime rate**, versus
- **High housing cost** but **low violent‑crime rate**.

```{r, message=FALSE, warning=FALSE}
# 1) Data: keep only the two joint profiles of interest
data("out1_arranged")
d1 <- out1_arranged$data
d2 <- d1 |>
  mutate(y1 = case_when(
    # Low housing cost, high crime
    att1 == "att1:level1" & att6 == "att6:level2" ~ 1,
    TRUE ~ 0
  ),
  y0 = case_when(
    # High housing cost, low crime
    att1 == "att1:level3" & att6 == "att6:level1" ~ 1,
    TRUE ~ 0
  )) |>
  filter(y1 == 1 | y0 == 1)

# 2) Labels: rename only the two att1 levels to reflect the joint trade-offs
labels1 <- out1_arranged$labels
labels2 <- labels1 |> 
  mutate(level = case_when(level_id == "att1:level1" ~ "Housing Cost (Low)\nCrime Rate(High)", 
                           level_id == "att1:level3" ~ "Housing Cost (High)\nCrime Rate(Low)", 
                           TRUE ~ level_id))
```


**(Optional) Sanity checks**
```{r}
d1 |> count(att1, att6)
d2 |> count(att1, att6) # only the two joint profiles remain
labels1 |> filter(attribute_id == "att1")
labels2 |> filter(attribute_id == "att1" & level_id %in% c("att1:level1", "att1:level3"))
```

**Recreate a `projoint_data` object, set the QOI, and plot.**
```{r, fig.width = 6, fig.height = 3}
# 3) Build a new projoint_data object
pj_data_wrangled <- projoint_data("labels" = labels2, 
                                  "data" = d2)

# 4) Quantity of interest: Low vs High housing cost under the specified crime conditions (choice-level MM)
qoi <- set_qoi(
  .att_choose = "att1", 
  .lev_choose = "level1", # Low housing cost (with high crime in this subset)
  .att_notchoose = "att1", 
  .lev_notchoose = "level3" # High housing cost (with low crime in this subset)
)

# 5) Estimate and plot (horizontal layout)
out <- projoint(pj_data_wrangled, qoi)
plot(out)
```

</details>

---

## 🧩 Explore: Compare Multiple Levels Simultaneously

<details><summary><b>Example: Urban vs. Suburban Preferences</b></summary>

**Goal.** Collapse `att7` into two buckets—City (levels 1–2) vs. Suburban (levels 5–6)—then re‑estimate and plot.

```{r, message=FALSE, warning=FALSE}
# 1) Data: collapse levels for att7
d1 <- out1_arranged$data
d2 <- d1 |>
  mutate(
    att7 = case_when(
      att7 %in% c("att7:level1", "att7:level2") ~ "att7:level7",  # City
      att7 %in% c("att7:level5", "att7:level6") ~ "att7:level8",  # Suburban
      TRUE ~ att7
    )
  )

# 2) Labels: create matching level IDs and readable names
labels1 <- out1_arranged$labels
labels2 <- labels1 |>
  mutate(
    level_id = case_when(
      level_id %in% c("att7:level1", "att7:level2") ~ "att7:level7",
      level_id %in% c("att7:level5", "att7:level6") ~ "att7:level8",
      TRUE ~ level_id
    ),
    level = case_when(
      level_id == "att7:level7" ~ "City",
      level_id == "att7:level8" ~ "Suburban",
      TRUE ~ level
    )
  ) |>
  distinct()
```


**(Optional) Sanity checks**
```{r}
d1 |> count(att7)
d2 |> count(att7)
labels1 |> filter(attribute_id == "att7")
labels2 |> filter(attribute_id == "att7")
```

**Recreate a `projoint_data` object, set the QOI, and plot.**
```{r, fig.width = 6, fig.height = 3}
# 3) Build a new projoint_data object
pj_data_wrangled <- projoint_data("labels" = labels2, 
                                  "data" = d2)

# 4) Quantity of interest: City vs. Suburban (choice-level MM)
qoi <- set_qoi(
  .structure     = "choice_level",
  .att_choose    = "att7",
  .lev_choose    = "level7",  # City
  .att_notchoose = "att7",
  .lev_notchoose = "level8"   # Suburban
)

# 5) Estimate and plot (horizontal layout)
out <- projoint(pj_data_wrangled, qoi)
plot(out)
```

</details>

---

## 📊 Compare: Subgroup Differences

<details><summary><b>Choice-Level Subgroup Comparison: Party Differences</b></summary>

```{r, fig.width = 6, fig.height = 3}
data("exampleData1")

outcomes <- c(paste0("choice", 1:8), "choice1_repeated_flipped")

df_D <- exampleData1 |> filter(party_1 == "Democrat") |> reshape_projoint(outcomes)
df_R <- exampleData1 |> filter(party_1 == "Republican") |> reshape_projoint(outcomes)
df_0 <- exampleData1 |> filter(party_1 %in% c("Something else", "Independent")) |> reshape_projoint(outcomes)

qoi <- set_qoi(
  .structure = "choice_level",
  .estimand = "mm",
  .att_choose = "att2",
  .lev_choose = "level3",
  .att_notchoose = "att2",
  .lev_notchoose = "level1"
)

out_D <- projoint(df_D, qoi)
out_R <- projoint(df_R, qoi)
out_0 <- projoint(df_0, qoi)

out_merged <- bind_rows(
  out_D$estimates |> mutate(party = "Democrat"),
  out_R$estimates |> mutate(party = "Republican"),
  out_0$estimates |> mutate(party = "Independent")
) |> filter(estimand == "mm_corrected")

# Plot

ggplot(out_merged, aes(y = party, x = estimate)) +
  geom_vline(xintercept = 0.5, linetype = "dashed", color = "gray") +
  geom_pointrange(aes(xmin = conf.low, xmax = conf.high)) +
  geom_text(aes(label = format(round(estimate, 2), nsmall = 2)), vjust = -1) +
  labs(y = NULL, x = "Choice-level Marginal Mean",
       title = "Preference for Democratic-majority areas") +
  theme_classic()
```

</details>

---

🏠 **Home:** [Home](https://yhoriuchi.github.io/projoint/index.html)

---
title: "Design Your Survey"
output: rmarkdown::html_vignette
description: >
  Learn how to set up your conjoint survey in Qualtrics, with or without the Projoint Survey Design Tool.
vignette: >
  %\VignetteIndexEntry{Design Your Survey}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

## Set Up Your Survey Using the Projoint Survey Design Tool

Using our <a href="https://projoint.aaronrkaufman.com/" target="_blank">Projoint Survey Design Tool</a>, you can easily: 

- Set up attributes and levels
- Randomize attributes and levels with weights
- Add cross-attributes *and* cross-profile constraints
- Automatically create a repeated task for intra-respondent reliability (IRR) estimation
- Generate test responses for checking your design
- 🚀 <span style="color:DarkOrange"><b>Export your survey directly for Qualtrics (in QSF format)</b></span> ✅

---

## How to Use the Projoint Survey Design Tool

The following instructions are taken directly from the Projoint Survey Design Tool's <a href="https://projoint.aaronrkaufman.com/tutorials/00_tutorial.md" target="_blank">Tutorial section</a>.

<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;">**1. Setting the Attributes**</summary>
The most important part of a discrete choice experiment is setting the attributes and levels: these are the features that are getting randomized. Add attributes by clicking on the "Add Attribute" button, and add levels by clicking on the "Add Level" button.

There are two advanced options on the page where users add attributes and levels. The first controls the order in which attributes appear to respondents. By clicking on the padlock icon to the right of each attribute, the user can control where they appear. A locked icon will always appear in the indicated position, so if the second attribute is locked, that attribute will always appear second in the list of attributes shown to respondents.

The second advanced option controls randomization weights, and can be accesed by clicking on the "Edit Weights" button. This allows researchers to set how frequently each level should appear. Most choice experiments set each level to equal frequency, but this is not necessarily preferred.

After you have set the attributes and levels, we recommend going to the Preview screen to preview your study. This produces a single task, which can be refreshed, so the researcher can see what the respondents will see.

</details>
<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;">**2. Additional Settings**</summary>
* <b>Number of profiles</b>: This defaults to 2, a binary comparison, but can be any positive integer. We recommend no more than 5.
* <b>Number of tasks</b>: This is the number of distinct comparison tasks the respondent sees. More tasks produces more data, but also potentially fatigues the respondent.
* <b>Repeated task</b>: This toggles whether one of the tasks respondents see is an exact duplicate of a previous task. By comparing the rate at which respondents give the same answer to identical questions, it is possible to measure and correct for a certain type of measurement error in conjoint studies. We strongly recommend turning this on.
* <b>Repeated task is shuffled</b>: Option to either flip columns for repeated tasks.
* <b>Which task to repeat?</b>: Which of the tasks should be repeated? This can range from 1 to N where N is the number of tasks.
* <b>Where to repeat?</b>: Where should the repeated task go? This can range from K where K is the index of the task to be repeated to N+1 where N is the number of tasks.
* <b>Ordering of attributes</b>: How should the order of attributes be randomized? The three options are Non random, Participant randomized, and Task randomized. Non-random says that every respondent and every task will have the same attribute order: the one specified by you. Participant-randomized says that each participant sees a different attribute order, but that attribute order is the same for all of that participant's tasks. In other words, each participant sees one ordering of attributes. Task-randomized says that every task has an independently-randomized attribute order. We recommend Participant-randomized as it is less confusing for participants.

</details>
<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;">**3. Restrictions**</summary>
There are two types of restrictions we discuss: <b>cross-attribute restrictions</b> and <b>cross-profile restrictions</b>. Cross-attribute restrictions prevent a profile from being logically inconsistent. For example, in a conjoint study comparing potential immigrants, if an immigrant has only a middle school education, their career cannot be neurosurgeon. Cross-profile restrictions produce dependencies across the different profiles of one task. For example, in a conjoint study comparing political candidates in a two-party, if one candidate is polling at 60%, the other must be polling at 40% or less.

</details>
<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;">**4. Exporting**</summary>
There are four options for exporting a survey, and we recommend users do at least three of them:

1. Exporting to <b>CSV</b> produces a user-specified number of test data indicating the attributes and levels of a number of random tasks. We recommend using this to double-check that all restrictions and randomization weights were successfully applied, as well as to conduct a power analysis.
2. Exporting to <b>JSON</b> produces a file containing all the information the user set defining the survey. We recommend saving this so that you can recreate the survey settings if you clear your cookies. This JSON file can be imported to this web tool, automating this process.
3. Exporting to <b>JavaScript (JS)</b> produces a file that can be uploaded to Qualtrics to perform randomization.
4. Exporting to <b>Qualtrics</b> produces a .QSF file that can be loaded into Qualtrics, automatically populating a survey with appropriate randomization and the correct question tasks.

</details>

---

## Other Options

If you are interested in how researchers previously set up their conjoint surveys, see the <a href="https://yhoriuchi.github.io/projoint/articles/faq.html" target="_blank">FAQ Page</a> instructions for using alternative tools<sup><a href="https://yhoriuchi.github.io/projoint/articles/faq.html#how-did-researchers-design-conjoint-surveys-previously" target="_blank" style="text-decoration: none;">&#9432;</a></sup>

---

🏠 **Home:** [Home](https://yhoriuchi.github.io/projoint/index.html)

---
title: "Read and Wrangle Your Data"
output: rmarkdown::html_vignette
description: >
  Read your Qualtrics data and wrangle it in R.
vignette: >
  %\VignetteIndexEntry{Read and Wrangle Your Data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
library(projoint)
```

## 📥 Read Your Data

Before you can reshape or analyze your conjoint survey data, you first need to **import it into R**.  In **projoint**, use the `read_Qualtrics()` function to quickly read properly formatted Qualtrics files.

---

## 🚀 Read Workflow

<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;">**1. Export your survey responses from Qualtrics**</summary>

When exporting from Qualtrics:

- Click **"Download Data"**.
- Choose **CSV** format.
- Critically, select **"Use choice text"** rather than coded values.

⚡ If you skip selecting "Use choice text," your conjoint data may fail to load properly!

</details>

<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;">**2. Load essential packages**</summary>

```{r, warning=FALSE, message=FALSE}
library(tidyverse)
library(projoint)
```
</details>

<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;">**3. Read your CSV file into R using `read_Qualtrics()`**</summary>

```r
# Example: If your file is located in a "data" folder
data <- read_Qualtrics("data/your_file.csv")
```

`read_Qualtrics()` automatically removes the question-text and `ImportId`
metadata rows used by current Qualtrics exports. It also recognizes legacy
exports with only a question-text row while preserving the original variable
names and column order.

Or, if using an example bundled with **projoint**:

```{r, eval=TRUE, echo=FALSE}
data <- read_Qualtrics(
  system.file("extdata", "mummolo_nall_replication.csv", package = "projoint")
)
```

```{r}
# Inspect the imported data:
data
```
</details>

---

```{r fig-setup, include=FALSE}
# Global default settings for all figures
knitr::opts_chunk$set(
  fig.width = 7,
  fig.height = 5,
  fig.align = "center",
  dpi = 300  # Optional: high-resolution plots
)

# Helper functions for special figure sizes
narrow_fig <- function() list(fig.width = 5, fig.height = 4)
wide_fig <- function() list(fig.width = 8, fig.height = 5)
tall_fig <- function() list(fig.width = 6, fig.height = 7)

# Load libraries
library(projoint)
data(exampleData1, package = "projoint")
data(exampleData2, package = "projoint")
data(exampleData3, package = "projoint")
data(exampleData1_labelled_tibble, package = "projoint")
data(out1_arranged, package = "projoint")

```

## 🛠️ Wrangle Your Data

Preparing your data correctly is one of the most important steps in conjoint analysis. Fortunately, the `reshape_projoint()` function in **projoint** makes this easy.

---

## 🚀 Wrangle Workflow

<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;">**1. Reshape Your Data**</summary>
<div>

> **Outcome naming & order (important)**
> 
> - List `.outcomes` in the **order questions were asked**.  
> - If you have a repeated task, its outcome must be the **last element**.  
> - For base tasks (all but last), the function reads the **digits** in each name as the task id (e.g., `"choice4"`, `"Q4"`, `"task04"` → task 4).  
> - The **repeated base task** is inferred from the **first base outcome’s digits**. The repeated outcome itself **need not** contain digits—only its position (last) matters.  
> - Specify the two exported response values with `.choice_map`. Its **names**
>   are the response strings stored in the outcome columns, and its values are
>   the corresponding Qualtrics profile positions (`1` or `2`). For example,
>   use `.choice_map = c("Community A" = 1, "Community B" = 2)` when those are
>   the exported choices and the instrument confirms that Community A is
>   profile 1. **projoint cannot infer this mapping from the CSV.**
> - Invalid labels, trailing whitespace, and missing choices now stop with an
>   informative error. Retain missing choices only after review by setting
>   `.allow_missing_choices = TRUE`.

### Specify response labels and profile positions

First inspect the response values that actually appear in your outcome
columns. For example:

```{r}
outcomes <- paste0("choice", 1:8)
sort(unique(unlist(exampleData1[outcomes], use.names = FALSE)))
```

Then verify the relationship between those response values and the two profile
positions using the Qualtrics instrument or QSF file. Write the verified
relationship as:

```r
.choice_map = c(
  "exact response value for profile 1" = 1,
  "exact response value for profile 2" = 2
)
```

The strings on the left are not new profile names created by
`reshape_projoint()`. They must match the ends of the values stored in every
outcome column, including capitalization and whitespace. The numbers on the
right refer to the profile positions encoded in columns such as `K-1-1-*` and
`K-1-2-*`. Although shorter suffixes such as `"A"` and `"B"` are supported,
using the complete exported response strings is clearer and more auditable.


### Example (Flipped Repeated Task)

```{r, error=TRUE}
outcomes <- paste0("choice", 1:8)
outcomes1 <- c(outcomes, "choice1_repeated_flipped")

out1 <- reshape_projoint(
  .dataframe = exampleData1,
  .outcomes = outcomes1,
  .choice_map = c("Community A" = 1, "Community B" = 2),
  .alphabet = "K",
  .idvar = "ResponseId",
  .repeated = TRUE,
  .flipped = TRUE
)
```

**Key Arguments**:

- `.outcomes`: Outcome columns (include repeated task last)
- `.choice_map`: Verified mapping from exact exported response values to
  Qualtrics profile positions 1 and 2
- `.idvar`: Respondent ID variable
- `.alphabet`: Variable prefix ("K")
- `.repeated`, `.flipped`: If repeated task exists and is flipped

</div>
</details>

<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;">**2. Variations: Repeated vs. Non-Repeated**</summary>

**Not-Flipped Repeated Task**

```{r}
outcomes <- paste0("choice", 1:8)
outcomes2 <- c(outcomes, "choice1_repeated_notflipped")
out2 <- reshape_projoint(
  .dataframe = exampleData2,
  .outcomes = outcomes2,
  .repeated = TRUE,
  .flipped = FALSE
)
```

**No Repeated Task**

```{r}
outcomes <- paste0("choice", 1:8)
out3 <- reshape_projoint(
  .dataframe = exampleData3,
  .outcomes = outcomes,
  .repeated = FALSE
)
```

</details>

<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;">**3. The `.fill` Argument: Should You Use It?**</summary>

Use `.fill = TRUE` to "fill" missing values based on IRR agreement.

```{r}
fill_FALSE <- reshape_projoint(
  .dataframe = exampleData1,
  .outcomes = outcomes1,
  .fill = FALSE
)

fill_TRUE <- reshape_projoint(
  .dataframe = exampleData1,
  .outcomes = outcomes1,
  .fill = TRUE
)
```

Compare:

```{r}
selected_vars <- c("id", "task", "profile", "selected", "selected_repeated", "agree")
fill_FALSE$data[selected_vars]
fill_TRUE$data[selected_vars]
```

**Tip:**  
- Use `.fill = TRUE` for small-sample or subgroup analysis (helps increase power).  
- Use `.fill = FALSE` (default) when in doubt for safer estimates.

</details>

<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;">**4. What If Your Data Is Already Clean?**</summary>

If you already have a clean dataset, use `make_projoint_data()`:

```{r}
out4 <- make_projoint_data(
  .dataframe = exampleData1_labelled_tibble,
  .attribute_vars = c(
    "School Quality", "Violent Crime Rate (Vs National Rate)",
    "Racial Composition", "Housing Cost",
    "Presidential Vote (2020)", "Total Daily Driving Time for Commuting and Errands",
    "Type of Place"
  ),
  .id_var = "id",
  .task_var = "task",
  .profile_var = "profile",
  .selected_var = "selected",
  .selected_repeated_var = "selected_repeated",
  .fill = TRUE
)
```

Preview:

```{r}
out4
```

</details>

<details style="margin-left: 25px; margin-bottom: -10px">
<summary style="font-size: 18px;">**5. Arranging Attribute and Level Labels**</summary>

To reorder or relabel attributes:

1. Save labels:

```{r, eval=FALSE}
save_labels(out1, "temp/labels_original.csv")
```

2. Edit the CSV (change `order`, label columns; leave `level_id` untouched)

3. Save it as "labels_arranged.csv" or something else.

4. Reload labels:

```{r, eval=FALSE}
out1_arranged <- read_labels(out1, "temp/labels_arranged.csv")
```

```{r, eval=TRUE, eco=FALSE}
data(out1_arranged, package = "projoint")
```

Compare using our example:

```{r}
mm <- projoint(out1, .structure = "profile_level", .estimand = "mm")
plot(mm)
```

```{r}
mm <- projoint(out1_arranged, .structure = "profile_level", .estimand = "mm")
plot(mm)
```

</details>

---

🏠 **Home:** [Home](https://yhoriuchi.github.io/projoint/index.html)

---
title: "Analyze and Visualize Important QOIs"
output: rmarkdown::html_vignette
description: >
  How to estimate and correct marginal means (MMs) or average marginal component effects (AMCEs), including predicting IRR if necessary. Visualize these and other QOIs of interest.
vignette: >
  %\VignetteIndexEntry{Analyze and Visualize Important QOIs}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
library(projoint)
library(dplyr)
library(ggplot2)
library(patchwork)
```

## 🎯 Estimate Corrected MMs or AMCEs

In conjoint analysis, default MMs and AMCEs can be **biased** due to measurement error from intra-respondent variability.

**projoint** corrects for this bias automatically.

The following instructions apply to choice-level data. What if you have profile-level data?<sup><a href="faq.html#what-if-i-have-profile-level-data" target="_blank" style="text-decoration: none;">&#9432;</a></sup> Our <a href="faq.html" target="_blank">FAQ Page</a> has instructions to estimate and visualize profile-level QOIs.

---

## 📦 Prepare Example Data

> **Outcome naming & order (important)**
> 
> - List `.outcomes` in the **order questions were asked**.  
> - If you have a repeated task, its outcome must be the **last element**.  
> - For base tasks (all but last), the function reads the **digits** in each name as the task id (e.g., `"choice4"`, `"Q4"`, `"task04"` → task 4).  
> - The **repeated base task** is inferred from the **first base outcome’s digits**. The repeated outcome itself **need not** contain digits—only its position (last) matters.  
> - Outcome strings must end with verified choice labels. Prefer an explicit
>   map using the complete exported response values, such as
>   `.choice_map = c("Community A" = 1, "Community B" = 2)`. The names are the
>   response values; `1` and `2` are the Qualtrics profile positions. Verify
>   this relationship from the survey instrument or QSF file because it cannot
>   be inferred from the CSV.
> - Invalid labels, trailing whitespace, and missing choices stop with an
>   informative error. Use `.allow_missing_choices = TRUE` only after review.

**Examples**

```{r}
# Standard order; repeated = task 1
data("exampleData1")
outcomes <- c(paste0("choice", 1:8), "choice1_repeated_flipped")
out1 <- reshape_projoint(
  exampleData1,
  outcomes,
  .choice_map = c("Community A" = 1, "Community B" = 2)
)
```

---

## 🛠️ Why Use IDs (e.g., `att1`, `level1`)?

Before estimating quantities, it’s important to understand how attribute and level IDs work inside projoint.

<details>
We recommend working with **attribute IDs** rather than actual text labels because:

- Safer against special characters, languages, or typos
- Allows **multiple attributes** to have identical labels (e.g., "High" for both "Teaching Quality" and "Research Quality")

Check attribute-level mappings:

```{r}
out1$labels
```

You can also save these labels for easier editing:

```r
save_labels(out1, "labels.csv")
```
</details>
---

## 📈 Estimate Marginal Means (MMs)

<details><summary><b>Choice-Level MMs (Specific Level)</b></summary>

Suppose you want to estimate, within a given profile pair, the probability of choosing a profile that includes "40% of pre-tax income" (`level3`) for Housing Cost (`att1`) rather than one that includes "15% of pre-tax income" (`level1`) for the same attribute, averaging over all combinations of the other attributes and across respondents; then use the following code:

```{r}
qoi <- set_qoi(
  .structure = "choice_level",
  .att_choose = "att1",
  .lev_choose = "level3",
  .att_notchoose = "att1",
  .lev_notchoose = "level1"
)

mm2 <- projoint(out1, qoi)
print(mm2)
summary(mm2)
```

</details>
---

## 📉 Estimate AMCEs

<details><summary><b>Choice-Level AMCEs (Specific Level)</b></summary>

Suppose you want to quantify how the choice probability **changes** between the following profile pairs:

- choosing a profile that includes "40% of pre-tax income" (`level3`) for Housing Cost (`att1`) versus one that includes "15% of pre-tax income" (`level1`) for Housing Cost (`att1`); **and**
- [baseline] choosing a profile that includes "30% of pre-tax income" (`level2`) for Housing Cost (`att1`) versus one that includes "15% of pre-tax income" (`level1`) for Housing Cost (`att1`); 

averaging over all combinations of the other attributes and across respondents. Then write the following code:

```{r}
qoi <- set_qoi(
  .structure = "choice_level",
  .estimand = "amce",
  .att_choose = "att1",
  .lev_choose = "level3",
  .att_notchoose = "att1",
  .lev_notchoose = "level1",
  .att_choose_b = "att1",
  .lev_choose_b = "level2",
  .att_notchoose_b = "att1",
  .lev_notchoose_b = "level1"
)

amce2 <- projoint(out1, qoi)
print(amce2)
summary(amce2)
```

</details>
---

## 🔎 Predict Intra-Respondent Reliability (IRR)

If your design **does not include** a repeated task, you can **predict** IRR using predict_tau(), based on observed respondent behavior.

<details><summary><b>Predict IRR Using `predict_tau()`</b></summary>

```{r}
data(out1_arranged)
predicted_irr <- predict_tau(out1_arranged)

print(predicted_irr)
summary(predicted_irr)
plot(predicted_irr)
```

</details>
---

```{r fig-setup, include=FALSE}
# Global default settings for all figures
knitr::opts_chunk$set(
  fig.width = 6,
  fig.height = 3,
  fig.align = "center",
  dpi = 300  # Optional: high-resolution plots
)

# Helper functions for special figure sizes
narrow_fig <- function() list(fig.width = 5, fig.height = 4)
wide_fig <- function() list(fig.width = 8, fig.height = 5)
tall_fig <- function() list(fig.width = 6, fig.height = 7)

```

```{r, echo=FALSE}
library(projoint)
library(ggplot2)

data(out1_arranged, package = "projoint")
```

## 🎨 Visualize MMs or AMCEs

The **projoint** package provides ready-to-publish plotting tools for conjoint analysis results.

<span style="color:DarkRed"><b>Note:</b></span> The current version of **projoint** supports plotting choice-level MMs only.  
Support for **choice-level AMCEs** will be available in future updates!

---

### ⚖️ Choice-Level Analysis

<details>
<summary><b>Estimate</b></summary>

- Specify your quantity of interest:
```{r}
qoi_mm <- set_qoi(
  .structure = "choice_level", # default
  .att_choose = "att1", 
  .lev_choose = "level1", 
  .att_notchoose = "att1", 
  .lev_notchoose = "level3"
)
```

- Estimate
```{r}
choice_mm <- projoint(
  .data = out1_arranged, 
  .qoi = qoi_mm, 
  .ignore_position = TRUE
)
```
</details>

<details>
<summary><b>Visualize (Levels) </b></summary>

```{r, fig.width = 6, fig.height = 3}
plot(choice_mm)
```
</details>

<details>
<summary><b>Visualize (Differences)</b></summary>

```{r, fig.width = 8, fig.height = 2}
plot(choice_mm, .type = "pointrange")
```
</details>

---

🏠 **Home:** [Home](https://yhoriuchi.github.io/projoint/index.html)

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/make_projoint_data.R
\name{make_projoint_data}
\alias{make_projoint_data}
\title{Make a \code{projoint_data} object from a labelled tibble}
\usage{
make_projoint_data(
  .dataframe,
  .attribute_vars,
  .id_var = "id",
  .task_var = "task",
  .profile_var = "profile",
  .selected_var = "selected",
  .selected_repeated_var = NULL,
  .fill = FALSE
)
}
\arguments{
\item{.dataframe}{A data frame (or tibble). One row per profile per task per respondent.}

\item{.attribute_vars}{Character vector of attribute column names.}

\item{.id_var}{Column name (character) with respondent IDs. Default \code{"id"}.}

\item{.task_var}{Column name (character) with task numbers. Default \code{"task"}.}

\item{.profile_var}{Column name (character) with profile numbers. Default \code{"profile"}.}

\item{.selected_var}{Column name (character) with the binary choice for each task
(values in \code{\{0,1\}}). Default \code{"selected"}.}

\item{.selected_repeated_var}{Optional column name (character) with the binary choice
for the repeated task. Default \code{NULL}.}

\item{.fill}{Logical. If \code{TRUE}, uses repeated-task agreement to \emph{fill}
missing agreement values for non-repeated tasks (assumes IRR is independent of
table content). If unsure, prefer the default \code{FALSE}.}
}
\value{
A \code{projoint_data} object (a list-like object containing a \code{labels}
  tibble and a \code{data} tibble) ready to pass to \code{\link{projoint}} and related
  functions.
}
\description{
Converts a labelled tibble/data frame (one column per attribute) into an object
of class \code{projoint_data} that downstream functions (e.g., \code{\link{projoint}})
can consume. The unit of observation should be each of two profiles in each task
for each respondent.
}
\examples{
\donttest{
# Example: build a projoint_data object from the labelled-tibble example
data(exampleData1_labelled_tibble)

att_cols <- c("School Quality", "Violent Crime Rate (Vs National Rate)",
              "Racial Composition", "Housing Cost",
              "Presidential Vote (2020)", 
              "Total Daily Driving Time for Commuting and Errands",
              "Type of Place")

pj_dat <- make_projoint_data(
  .dataframe             = exampleData1_labelled_tibble,
  .attribute_vars        = att_cols,
  .id_var                = "id",
  .task_var              = "task",
  .profile_var           = "profile",
  .selected_var          = "selected",
  .selected_repeated_var = "selected_repeated",
  .fill                  = FALSE
)

class(pj_dat)
# [1] "projoint_data"
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pj_estimate.R
\name{pj_estimate}
\alias{pj_estimate}
\title{Internal Estimation Function}
\usage{
pj_estimate(
  .data,
  .structure,
  .estimand,
  .att_choose,
  .lev_choose,
  .att_notchoose,
  .lev_notchoose,
  .att_choose_b,
  .lev_choose_b,
  .att_notchoose_b,
  .lev_notchoose_b,
  .se_method,
  .irr,
  .remove_ties,
  .ignore_position,
  .n_sims,
  .n_boot,
  .weights_1,
  .clusters_1,
  .se_type_1,
  .weights_2,
  .clusters_2,
  .se_type_2,
  .auto_cluster = TRUE,
  .seed = NULL
)
}
\arguments{
\item{.data}{A \code{\link{projoint_data}} object created by \code{\link{reshape_projoint}} or \code{\link{make_projoint_data}}.}

\item{.structure}{Either \code{"profile_level"} or \code{"choice_level"}.}

\item{.estimand}{Either \code{"mm"} (marginal mean) or \code{"amce"} (average marginal component effect).}

\item{.att_choose}{Attribute for the chosen profile/feature.}

\item{.lev_choose}{Level(s) for the chosen profile/feature. Length 1 for \code{profile_level}; may be 1+ for \code{choice_level}.}

\item{.att_notchoose}{Attribute for the not-chosen profile/feature (required for \code{choice_level}).}

\item{.lev_notchoose}{Level(s) for the not-chosen profile/feature (required for \code{choice_level}).}

\item{.att_choose_b}{(AMCE only) Baseline attribute for comparison.}

\item{.lev_choose_b}{(AMCE only) Baseline level(s) for comparison.}

\item{.att_notchoose_b}{(AMCE only, choice-level only) Baseline attribute for the not-chosen profile.}

\item{.lev_notchoose_b}{(AMCE only, choice-level only) Baseline level(s) for the not-chosen profile.}

\item{.se_method}{One of \code{"analytical"}, \code{"simulation"}, or \code{"bootstrap"}.}

\item{.irr}{\code{NULL} (default) to estimate IRR from repeated tasks; otherwise a numeric IRR value.}

\item{.remove_ties}{Logical; should ties be removed before estimation? Defaults to \code{TRUE}.}

\item{.ignore_position}{Logical; only for \code{choice_level}. If \code{TRUE} (default), ignore profile position (left/right).}

\item{.n_sims}{Integer; required if \code{.se_method == "simulation"}.}

\item{.n_boot}{Integer; required if \code{.se_method == "bootstrap"}.}

\item{.weights_1}{(Optional) Bare (unquoted) column with weights for IRR estimation; passed to \code{\link[estimatr]{lm_robust}}.}

\item{.clusters_1}{(Optional) Bare (unquoted) column with clusters for IRR estimation; passed to \code{\link[estimatr]{lm_robust}}.}

\item{.se_type_1}{SE type for IRR estimation; passed to \code{\link[estimatr]{lm_robust}}. If \code{NULL}, \emph{estimatr} defaults are used (HC2 when unclustered; CR2 when clustered).}

\item{.weights_2}{(Optional) Bare (unquoted) column with weights for MM/AMCE estimation; passed to \code{\link[estimatr]{lm_robust}}.}

\item{.clusters_2}{(Optional) Bare (unquoted) column with clusters for MM/AMCE estimation; passed to \code{\link[estimatr]{lm_robust}}.}

\item{.se_type_2}{SE type for MM/AMCE estimation; passed to \code{\link[estimatr]{lm_robust}}. If \code{NULL}, \emph{estimatr} defaults are used (HC2 when unclustered; CR2 when clustered).}

\item{.auto_cluster}{Logical; if \code{TRUE} (default), auto-cluster on \code{id} when present and no \code{.clusters_*} are supplied; auto-clustering only occurs when the corresponding \code{.se_type_*} is \code{NULL}. See \code{\link{projoint}}.}

\item{.seed}{Optional integer. If supplied, sets a temporary RNG seed for reproducible simulation/bootstrap inside this call 
and restores the previous RNG state on exit.}
}
\value{
A data frame with rows for the requested estimand(s) and columns:
  \itemize{
    \item \code{estimand}: one of \code{"mm_uncorrected"}, \code{"mm_corrected"},
          \code{"amce_uncorrected"}, \code{"amce_corrected"}.
    \item \code{estimate}, \code{se}, \code{conf.low}, \code{conf.high}, \code{tau}.
  }
}
\description{
Core workhorse for computing marginal means (MMs) or AMCEs from a conjoint design,
with optional intra-respondent reliability (IRR) correction.
}
\details{
IRR is clipped to \code{[0.5, 1)} (with a tiny epsilon) to avoid boundary issues.
For choice-level MMs, ties must be removed (\code{.remove_ties = TRUE}).
When \code{.seed} is supplied, the previous RNG state is restored on exit.
}
\seealso{
\code{\link[estimatr]{lm_robust}}, \code{\link{projoint}},
  \code{\link{projoint_level}}, \code{\link{projoint_diff}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.projoint_data.R
\name{summary.projoint_data}
\alias{summary.projoint_data}
\title{Summary for \code{projoint_data}}
\usage{
\method{summary}{projoint_data}(object, ...)
}
\arguments{
\item{object}{A \code{projoint_data} object.}

\item{...}{Additional arguments (currently unused).}
}
\value{
No return value, called for its side effect of printing a summary of the
  \code{projoint_data} object to the console.
}
\description{
Custom summary method for objects of class \code{projoint_data}. Prints a brief
overview of the main data and attribute-level labels contained in the object.
}
\examples{
\donttest{
data(exampleData1)
dat <- reshape_projoint(
  exampleData1,
  .outcomes = c("choice1", "choice2")
)
summary(dat)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.projoint_tau.R
\name{print.projoint_tau}
\alias{print.projoint_tau}
\title{Print method for projoint_tau objects}
\usage{
\method{print}{projoint_tau}(x, ...)
}
\arguments{
\item{x}{An object of class \code{projoint_tau}.}

\item{...}{Additional arguments (currently unused).}
}
\value{
No return value; called for its side effect of printing a summary of
  the estimated intra-respondent reliability (\eqn{\tau}).
}
\description{
Custom print method for objects of class \code{projoint_tau}, typically created
by \code{\link{projoint}} or related functions.
}
\examples{
toy_tau <- structure(
  list(irr = data.frame(predicted = 0.413, se = 0.02, n = 200)),
  class = "projoint_tau"
)
print(toy_tau)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.projoint_tau.R
\name{plot.projoint_tau}
\alias{plot.projoint_tau}
\title{Plot method for \code{projoint_tau}}
\usage{
\method{plot}{projoint_tau}(x, ...)
}
\arguments{
\item{x}{A \code{projoint_tau} object.}

\item{...}{Optional arguments (currently unused).}
}
\value{
A \code{ggplot2} object representing the IRR (\eqn{\tau}) visualization.
  The plot is drawn for its side effect and also returned (invisibly).
}
\description{
Visualizes the estimated intra-respondent reliability (\eqn{\tau}) produced
by the extrapolation method and stored in a \code{projoint_tau} object.
}
\examples{
\donttest{
# Estimate tau, then plot:
# dat <- reshape_projoint(exampleData1, .outcomes = c("choice1","choice2"))
# tau_fit <- projoint_tau(dat)  # or predict_tau(dat)
# p <- plot(tau_fit)            # also returns the ggplot object (invisibly)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.projoint_results.R
\name{summary.projoint_results}
\alias{summary.projoint_results}
\title{Summary method for \code{projoint_results}}
\usage{
\method{summary}{projoint_results}(object, ...)
}
\arguments{
\item{object}{An object of class \code{projoint_results}.}

\item{...}{Additional arguments (ignored).}
}
\value{
A data frame (often a tibble) summarizing the estimated effects.
  At minimum, it contains the columns produced in \code{object$estimates}
  (e.g., attribute/level identifiers and the point estimate with its
  standard error and confidence interval in columns such as
  \code{estimate}, \code{std.error}, \code{conf.low}, \code{conf.high}).
  This table is suitable for further processing or printing.
}
\description{
Creates a concise tabular summary of a \code{projoint_results} object,
including the chosen estimand, analysis structure, standard-error settings,
and a data frame of estimates.
}
\examples{
\donttest{
  data(exampleData1)

  # Reshape data for two base tasks + repeated (for IRR estimation)
  dat <- reshape_projoint(
    exampleData1,
    .outcomes = c("choice1", "choice2", "choice1_repeated_flipped")
  )

  # Build a valid choice-level QoI
  att <- unique(dat$labels$attribute_id)[1]
  lev_ids   <- dat$labels$level_id[dat$labels$attribute_id == att]
  lev_names <- sub(".*:", "", lev_ids)

  q <- set_qoi(
    .structure     = "choice_level",
    .estimand      = "mm",
    .att_choose    = att,
    .lev_choose    = lev_names[2],
    .att_notchoose = att,
    .lev_notchoose = lev_names[1]
  )

  # Fit model
  fit <- projoint(dat, .qoi = q)

  # Get the tabular summary of estimates
  tab <- summary(fit)
  head(tab)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/projoint.R
\name{projoint}
\alias{projoint}
\title{Analyze a conjoint dataset with measurement-error correction}
\usage{
projoint(
  .data,
  .qoi = NULL,
  .by_var = NULL,
  .structure = "choice_level",
  .estimand = "mm",
  .se_method = "analytical",
  .irr = NULL,
  .remove_ties = TRUE,
  .ignore_position = NULL,
  .n_sims = NULL,
  .n_boot = NULL,
  .weights_1 = NULL,
  .clusters_1 = NULL,
  .se_type_1 = NULL,
  .weights_2 = NULL,
  .clusters_2 = NULL,
  .se_type_2 = NULL,
  .auto_cluster = TRUE,
  .seed = NULL
)
}
\arguments{
\item{.data}{A \code{\link{projoint_data}} created by
\code{\link{reshape_projoint}} or \code{\link{make_projoint_data}}.}

\item{.qoi}{Optional \code{\link{projoint_qoi}} describing the quantity of
interest. If supplied, its fields override \code{.structure} and \code{.estimand}.}

\item{.by_var}{Optional column name (character) for subgroup analysis; must be
logical (TRUE/FALSE) or numeric/integer coded as 0/1. Only supported for
\code{.structure == "profile_level"} (ignored otherwise).}

\item{.structure}{Either \code{"profile_level"} or \code{"choice_level"}
(default \code{"choice_level"}). Overridden by \code{.qoi$structure} if present.}

\item{.estimand}{Either \code{"mm"} (marginal mean) or \code{"amce"} (average
marginal component effect). Default \code{"mm"}. Overridden by \code{.qoi$estimand} if present.}

\item{.se_method}{Standard-error method: \code{"analytical"} (default),
\code{"simulation"}, or \code{"bootstrap"}.}

\item{.irr}{Numeric or \code{NULL}. If \code{NULL} (default), IRR is estimated
(when design allows); otherwise a fixed IRR value is used and IRR estimation
is skipped.}

\item{.remove_ties}{Logical; remove ties in choice data before estimation?
Default \code{TRUE}.}

\item{.ignore_position}{Logical; choice-level only. Ignore profile position
(left/right)? Default \code{TRUE}.}

\item{.n_sims}{Integer; required when \code{.se_method = "simulation"}.}

\item{.n_boot}{Integer; required when \code{.se_method = "bootstrap"}.}

\item{.weights_1, .clusters_1, .se_type_1}{Passed to
\code{\link[estimatr]{lm_robust}} when estimating IRR.}

\item{.weights_2, .clusters_2, .se_type_2}{Passed to
\code{\link[estimatr]{lm_robust}} when estimating MMs/AMCEs.}

\item{.auto_cluster}{Logical. If \code{TRUE} (default), automatically cluster on
an \code{id} column when present and no \code{.clusters_*} are supplied. Auto-clustering only
occurs when the corresponding \code{.se_type_*} is \code{NULL}.}

\item{.seed}{Optional integer. Sets a temporary RNG seed for reproducible
simulation/bootstrap inside the call; restores the previous RNG state on exit.}
}
\value{
A \code{projoint_results} object (list-like) with components such as:
\itemize{
  \item \code{$estimand}, \code{$structure}, \code{$se_method}, \code{$irr}, \code{$tau}
  \item \code{$labels}: attribute/level mapping used in estimation
  \item \code{$estimates}: a data frame of estimates with columns like
        \code{att_level_choose}, \code{att_level_notchoose} (if choice-level),
        \code{estimate}, \code{se} / \code{std.error}, \code{conf.low}, \code{conf.high},
        and an \code{estimand} label such as \code{"mm_corrected"} or \code{"amce_uncorrected"}.
}
This object is suitable for downstream use in \code{\link{plot.projoint_results}},
\code{\link{summary.projoint_results}}, and related helpers.
}
\description{
Computes marginal means (MMs) or average marginal component effects (AMCEs)
with correction for intra-respondent reliability (IRR, \eqn{\tau}). When a
repeated task is present, IRR is estimated unless a fixed value is supplied.
Results are returned in a structured object ready for plotting and summary.
}
\details{
Most users will pass a \code{\link{projoint_data}} object (from
\code{\link{reshape_projoint}} or \code{\link{make_projoint_data}}). Advanced
users may specify custom quantities via \code{\link{projoint_qoi}}; if provided,
its \code{structure} and \code{estimand} override \code{.structure} and
\code{.estimand}.


Valid \code{se_type_*} values depend on clustering:
\itemize{
  \item Without clusters: \code{"classical"}, \code{"HC0"}, \code{"HC1"}, \code{"HC2"}, \code{"HC3"}
  \item With clusters: \code{"CR0"}, \code{"CR1"}, \code{"CR2"}, \code{"stata"}, \code{"none"}
}
If \code{NULL}, \emph{estimatr} defaults are used (\code{HC2} when unclustered;
\code{CR2} when clustered).
}
\examples{
\donttest{
# Prepare example data
data(exampleData1)
outcomes <- c(paste0("choice", 1:8), "choice1_repeated_flipped")
pj <- reshape_projoint(exampleData1, outcomes)

# Choice-level QoI based on pj$labels
att <- unique(pj$labels$attribute_id)[1]
lev_ids   <- pj$labels$level_id[pj$labels$attribute_id == att]
lev_names <- sub(".*:", "", lev_ids)

q <- set_qoi(
    .structure     = "choice_level",
    .estimand      = "mm",
    .att_choose    = att,
    .lev_choose    = lev_names[2],
    .att_notchoose = att,
    .lev_notchoose = lev_names[1]
  )
  
# Choice-level, marginal means (fast example: fix IRR)
fit_choice <- projoint(
  pj,
  .qoi = q,
  .structure = "choice_level",
  .estimand  = "mm",
  .irr       = 0.80,        # skip IRR estimation for a quick example
  .se_method = "analytical"
)
head(summary(fit_choice))

# Profile-level AMCEs
fit_profile <- projoint(
  pj,
  .structure = "profile_level",
  .estimand  = "amce",
  .se_method = "analytical"
)
# Plot using the S3 plot method
p <- plot(fit_profile, .estimates = "both")
print(p)
}

}
\seealso{
\code{\link{reshape_projoint}}, \code{\link{projoint_qoi}},
  \code{\link{plot.projoint_results}}, \code{\link{summary.projoint_results}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reshape_projoint.R
\name{reshape_projoint}
\alias{reshape_projoint}
\title{Reshape survey response data for conjoint analysis (single task set)}
\usage{
reshape_projoint(
  .dataframe,
  .outcomes,
  .choice_labels = c("A", "B"),
  .choice_map = NULL,
  .alphabet = "K",
  .idvar = "ResponseId",
  .repeated = TRUE,
  .flipped = TRUE,
  .covariates = NULL,
  .fill = FALSE,
  .allow_missing_choices = FALSE
)
}
\arguments{
\item{.dataframe}{A data frame, preferably from \code{\link{read_Qualtrics}}.}

\item{.outcomes}{Character vector of outcome column names in the \emph{asked order}.
If a repeated task is used, its outcome must be the \emph{last element}.}

\item{.choice_labels}{Character vector (default \code{c("A","B")}) giving the two labels that
appear at the end of the outcome strings. Position defines profile identity:
the first label means profile 1 and the second means profile 2. Ignored when
\code{.choice_map} is supplied.}

\item{.choice_map}{Optional named numeric vector that explicitly maps response
values to Qualtrics profile positions, for example
\code{c("Community A" = 1, "Community B" = 2)}. The names should preferably
be the complete strings stored in the outcome columns; shorter suffixes are
also supported. The numbers refer to the profile positions encoded by
columns such as \code{K-1-1-*} and \code{K-1-2-*}, not to new profile names.
The vector must map exactly two unique labels to profiles 1 and 2. Verify
the mapping against the survey instrument or QSF file because it cannot be
inferred from the CSV alone.}

\item{.alphabet}{Single character (default \code{"K"}) indicating the Qualtrics prefix.}

\item{.idvar}{Character (default \code{"ResponseId"}) indicating the respondent id column.}

\item{.repeated}{Logical (default \code{TRUE}) indicating whether a repeated task is present.}

\item{.flipped}{Logical (default \code{TRUE}) indicating whether the repeated task flips profiles
before agreement is computed.}

\item{.covariates}{Optional character vector of respondent-level covariate column names to carry through.}

\item{.fill}{Logical (default \code{FALSE}). If \code{TRUE}, fills \code{agree} within respondent
across tasks as described under “Filling agreement”.}

\item{.allow_missing_choices}{Logical (default \code{FALSE}). If \code{FALSE},
any missing outcome stops with an error. If \code{TRUE}, missing choices are
retained as \code{NA} for both profiles in the affected task.}
}
\value{
A \code{projoint_data} object with elements \code{$labels} and \code{$data}; see Details.
}
\description{
Takes a wide survey data frame (e.g., from \code{\link{read_Qualtrics}}) and reshapes
it so that each row corresponds to a single respondent–task–profile. Supports arbitrary
ordering of base tasks and a single repeated task per respondent. The repeated base task
is inferred from the first base outcome in \code{.outcomes}, and the repeated outcome
must be the last element of \code{.outcomes}.
}
\details{
\strong{Scope and assumptions}
\itemize{
  \item One set of conjoint tasks with exactly two profiles per task (profiles 1 and 2).
  \item For multi-set designs, call \code{reshape_projoint()} once per set and bind the results.
}

\strong{Expected input (Qualtrics K-codes)}
\itemize{
  \item Wide columns named \code{K-<task>-<attribute>} (attribute names) and
        \code{K-<task>-<profile>-<attribute>} (level names), with \code{<task>} in \code{1..n}
        and \code{<profile>} in \code{1,2}.
  \item Rows with missing \code{K-1-1} are dropped as empty tables (server hiccup safeguard).
}

\strong{Outcome columns (\code{.outcomes})}
\itemize{
  \item List all choice variables in the \emph{order asked}. If you include a repeated task,
        its outcome must be the \emph{last element}.
  \item For base tasks (all but the last element), the function extracts the base task id by
        reading the \emph{digits} in each outcome name (e.g., \code{"choice4"}, \code{"Q4"},
        \code{"task04"} -> task 4).
  \item The set of base task ids extracted from \code{.outcomes} must exactly match the set of
        task ids present in the K-codes; otherwise an error is thrown.
  \item The repeated base task is inferred as the digits in the \emph{first base outcome}
        (i.e., the first element of \code{.outcomes}, excluding the final repeated outcome).
}

\strong{Choice parsing}
\itemize{
  \item The selected profile is parsed from the end of each outcome string and
        matched to \code{.choice_labels} or \code{.choice_map}. Matching is
        case-sensitive and uses the full supplied label, so multi-character
        labels are supported.
  \item The package cannot infer whether a Qualtrics code or displayed label
        refers to profile 1 or profile 2. Analysts must verify that mapping
        from the survey instrument or QSF file. Use \code{.choice_map} to make
        the verified mapping explicit and auditable.
  \item For clarity, use the complete response strings stored in the outcome
        columns as the names of \code{.choice_map}. Shorter suffixes are
        supported, but the supplied strings must match the ends of all
        non-missing outcome values exactly, including case and whitespace.
  \item Unrecognized, ambiguous, or whitespace-altered choice values cause an
        error. Missing choices also cause an error unless
        \code{.allow_missing_choices = TRUE} is supplied explicitly.
}

\strong{Output}
\itemize{
  \item A \code{projoint_data} object with:
    \itemize{
      \item \code{$labels}: map from human-readable \code{attribute}/\code{level} to stable ids
            (\code{attribute_id = "att1","att2",...}, \code{level_id = "attX:levelY"}).
      \item \code{$data}: tibble with one row per \code{id}–\code{task}–\code{profile}, attribute
            columns (named \code{att*}) storing \code{level_id}, \code{selected} (1 if that profile
            was chosen; 0 otherwise), \code{agree} (1/0/NA for repeated-task agreement after flip logic),
            and any \code{.covariates}. \code{id} is coerced to character; attribute columns are factors.
    }
}

\strong{Filling agreement}
\itemize{
  \item If \code{.fill = TRUE}, \code{agree} is filled within respondent across tasks in task order,
        propagating the observed repeated-task agreement to all tasks for that respondent. This assumes
        IRR is respondent-specific and independent of table content.
}

\strong{Diagnostics}
\itemize{
  \item \code{dplyr::count(reshaped$data, task, profile)} should show exactly two rows per task.
  \item If \code{pj_estimate()} later reports “No rows match the specified attribute/level”, construct QoIs
        from \code{reshaped$labels} (use the exact \code{attX:levelY} ids).
}
}
\examples{
\donttest{
# Base tasks asked in numeric order; repeated task corresponds to task 1
data(exampleData1)
outcomes <- c(paste0("choice", 1:8), "choice1_repeated_flipped")
reshaped <- reshape_projoint(
  exampleData1,
  outcomes,
  .choice_map = c("Community A" = 1, "Community B" = 2)
)
dplyr::count(reshaped$data, task, profile) # should be 2 per task
}

}
\seealso{
\code{\link{make_projoint_data}}, \code{\link{projoint}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/projoint_diff.R
\name{projoint_diff}
\alias{projoint_diff}
\title{Estimate subgroup differences (internal)}
\usage{
projoint_diff(
  .data,
  .qoi,
  .by_var,
  .structure,
  .estimand,
  .se_method,
  .irr,
  .remove_ties,
  .ignore_position,
  .n_sims,
  .n_boot,
  .weights_1,
  .clusters_1,
  .se_type_1,
  .weights_2,
  .clusters_2,
  .se_type_2,
  .auto_cluster = TRUE,
  .seed = NULL
)
}
\arguments{
\item{.data}{A \code{\link{projoint_data}} object.}

\item{.qoi}{Optional \code{\link{projoint_qoi}}; if \code{NULL}, estimates all MMs/AMCEs.}

\item{.by_var}{Column name in \code{.data$data} defining subgroups; must be logical,
numeric/integer coded as 0/1, or factor with levels \code{"0"}/\code{"1"}.}

\item{.structure}{Must be \code{"profile_level"}.}

\item{.estimand}{Either \code{"mm"} or \code{"amce"}.}

\item{.se_method}{One of \code{"analytical"}, \code{"simulation"}, or \code{"bootstrap"}.}

\item{.irr}{\code{NULL} to estimate IRR from repeated tasks, or numeric to fix IRR.}

\item{.remove_ties}{Logical; drop ties before estimation? Default \code{TRUE}.}

\item{.ignore_position}{Ignored (subgroup analysis is profile-level only).}

\item{.n_sims}{Integer; required when \code{.se_method = "simulation"}.}

\item{.n_boot}{Integer; required when \code{.se_method = "bootstrap"}.}

\item{.weights_1, .clusters_1, .se_type_1}{Passed to \code{\link[estimatr]{lm_robust}} for IRR estimation.}

\item{.weights_2, .clusters_2, .se_type_2}{Passed to \code{\link[estimatr]{lm_robust}} for MM/AMCE estimation.}

\item{.auto_cluster}{Logical; if \code{TRUE} (default), auto-cluster on \code{id} when suitable
and no clusters are provided (applies only if the corresponding \code{.se_type_*} is \code{NULL}).}

\item{.seed}{Optional integer; sets a temporary RNG seed and restores prior state on exit.}
}
\value{
A \code{\link{projoint_results}} object containing subgroup differences with fields:
\itemize{
  \item \code{estimates}: tibble with one row per attribute/level and columns
        like \code{estimate} (group1 minus group0), \code{se}, \code{conf.low},
        \code{conf.high}, plus internal columns (\code{estimate_1}, \code{estimate_0},
        \code{se_1}, \code{se_0}) used in the diff.
  \item \code{estimand}: \code{"mm"} or \code{"amce"}.
  \item \code{structure}: \code{"profile_level"}.
  \item \code{se_method}: SE method used.
  \item \code{irr}: \code{"Estimated"} or \code{"Assumed (<value> )"}.
  \item \code{tau}: numeric reliability used (average of subgroup taus when estimated).
  \item \code{remove_ties}, \code{ignore_position}: flags echoed from inputs.
  \item \code{se_type_used}, \code{cluster_by}: details propagated from fitting calls.
  \item \code{labels}, \code{data}: design labels and the analysis data (passed through).
}
}
\description{
Worker used by \code{\link{projoint}} to compute subgroup differences
(group \code{== 1} minus group \code{== 0}) in marginal means (MMs) or
average marginal component effects (AMCEs). Supported only for
\code{.structure = "profile_level"}.
}
\seealso{
\code{\link{projoint}}, \code{\link{projoint_level}}, \code{\link{projoint_results}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/out1_arranged.R
\docType{data}
\name{out1_arranged}
\alias{out1_arranged}
\title{Example Output: Manually Rearranged Labels}
\format{
An object of class \code{projoint_data}.
  Contains two elements:
  \itemize{
    \item \code{$labels}: reordered attribute–level mapping
    \item \code{$data}: the corresponding profile-level dataset
  }
}
\usage{
data(out1_arranged)
}
\description{
A toy \code{\link{projoint_data}} object showing what happens after labels
have been manually rearranged (e.g., via \code{\link{save_labels}} /
\code{\link{read_labels}}).
}
\details{
Useful for illustrating workflows without requiring users to re-run the
reordering themselves.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/projoint_qoi.R
\name{projoint_qoi}
\alias{projoint_qoi}
\title{Create a projoint_qoi Object}
\usage{
projoint_qoi(...)
}
\arguments{
\item{...}{Named elements specifying QOI details, such as:
\itemize{
  \item \code{structure}: Analysis level (\code{"profile_level"} or \code{"choice_level"}).
  \item \code{estimand}: Quantity of interest (\code{"mm"} or \code{"amce"}).
  \item \code{attribute_of_interest}, \code{levels_of_interest}, etc.
}}
}
\value{
A list of QOI specifications with class \code{"projoint_qoi"}.
}
\description{
Internal constructor for \code{projoint_qoi} objects.  
A \code{projoint_qoi} stores the specification of custom
quantities of interest (QOIs) for conjoint estimation.
Typically called by \code{\link{set_qoi}} rather than directly.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/organize_data.R
\name{organize_data}
\alias{organize_data}
\title{Organize data for estimation (internal helper)}
\usage{
organize_data(
  .dataframe,
  .structure,
  .estimand,
  .remove_ties,
  .att_choose,
  .lev_choose,
  .att_notchoose,
  .lev_notchoose
)
}
\arguments{
\item{.dataframe}{A tibble/data frame from \code{\link{reshape_projoint}()},
containing columns like \code{id}, \code{task}, \code{profile}, \code{selected},
\code{agree} (if repeated), and attribute columns named \code{att1}, \code{att2}, ...
that store \code{level_id}s (e.g., \code{"att1:level2"}).}

\item{.structure}{Either \code{"profile_level"} or \code{"choice_level"}.}

\item{.estimand}{Either \code{"mm"} or \code{"amce"}.}

\item{.remove_ties}{Logical; if \code{TRUE} (default) remove tied responses
in profile-level setups (keeps tasks where exactly one profile is selected).}

\item{.att_choose}{Attribute ID for the “chosen” side (e.g., \code{"att3"}).}

\item{.lev_choose}{Level ID(s) for the chosen side (e.g., \code{"level2"} for
profile-level; vector of level IDs for choice-level).}

\item{.att_notchoose}{Attribute ID for the “not chosen” side (choice-level only).}

\item{.lev_notchoose}{Level ID(s) for the not-chosen side (choice-level only).}
}
\value{
A named list with two tibbles:
\itemize{
  \item \code{$data_for_estimand}: rows restricted and reshaped to the
        requested estimand/QOI. For profile-level, one row per respondent–task–profile
        where the target level is present (ties optionally removed). For choice-level,
        one row per respondent–task with paired information for profiles 1 and 2,
        restricted to the requested attribute-level pair(s).
  \item \code{$data_for_irr}: one row per respondent with columns \code{id},
        \code{agree} (if available), and any detected weight/cluster hints
        (columns matching \code{"my_weight|my_cluster|^weights?$|^clusters?$"}).
}
}
\description{
Prepares tidy inputs for MM/AMCE estimation and IRR handling.
Called inside \code{pj_estimate()} after reshaping to respondent–task–profile.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.projoint_tau.R
\name{summary.projoint_tau}
\alias{summary.projoint_tau}
\title{Summary method for projoint_tau objects}
\usage{
\method{summary}{projoint_tau}(object, ...)
}
\arguments{
\item{object}{An object of class \code{projoint_tau}.}

\item{...}{Additional arguments (currently unused).}
}
\value{
A tibble (data frame) showing IRR estimates, typically by the number
  of differing attributes, as stored in \code{object$irr}.
}
\description{
Custom summary method for objects of class \code{projoint_tau}, typically created
by \code{\link{projoint}} or related functions. Summarizes intra-respondent
reliability (IRR) estimates.
}
\examples{
toy_tau <- structure(
  list(irr = data.frame(predicted = 0.413, se = 0.02, n = 200)),
  class = "projoint_tau"
)
summary(toy_tau)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/projoint_data.R
\name{projoint_data}
\alias{projoint_data}
\title{Create a projoint_data Object}
\usage{
projoint_data(labels, data)
}
\arguments{
\item{labels}{A data frame of conjoint attribute–level metadata. Must include
attribute names, attribute IDs (e.g., \code{"att1"}), and level IDs
(e.g., \code{"att1:lev1"}).}

\item{data}{A data frame (typically a tibble) containing the reshaped conjoint
survey responses, one row per respondent–task–profile, with attribute columns,
\code{selected}, \code{agree}, and any covariates.}
}
\value{
A list of length two with class \code{"projoint_data"}, containing:
  \itemize{
    \item \code{$labels}: attribute–level mapping
    \item \code{$data}: respondent–task–profile dataset
  }
}
\description{
Internal constructor for \code{projoint_data} objects.  
Used by \code{\link{reshape_projoint}} and \code{\link{make_projoint_data}} to
bundle conjoint survey labels and response data into a consistent structure.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/projoint_results.R
\name{projoint_results}
\alias{projoint_results}
\title{Create a projoint_results Object}
\usage{
projoint_results(...)
}
\arguments{
\item{...}{Named elements, typically including:
\itemize{
  \item \code{estimates}: A data frame of estimated effects (point estimates,
    standard errors, confidence intervals).
  \item \code{labels}: Attribute/level label mappings from the design.
  \item \code{structure}: Either \code{"profile_level"} or \code{"choice_level"}.
  \item \code{estimand}: Either \code{"mm"} (marginal mean) or
    \code{"amce"} (average marginal component effect).
  \item \code{irr}: Intra-respondent reliability value used.
  \item \code{tau}: Estimated \eqn{\tau} value from IRR correction.
  \item \code{se_method}: Method used for standard errors.
}}
}
\value{
A list with class \code{"projoint_results"} containing
  estimates and associated metadata. This object is the standard
  output of \code{\link{projoint}} and is accepted by plotting
  and summary methods.
}
\description{
Internal constructor for \code{projoint_results} objects.  
A \code{projoint_results} stores the outputs of a conjoint analysis,
including estimated effects and metadata about the analysis settings.
It is normally generated by \code{\link{projoint}} and not called directly.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exampleData1_labelled_tibble.R
\docType{data}
\name{exampleData1_labelled_tibble}
\alias{exampleData1_labelled_tibble}
\title{Projoint Example Data Set 1: "Labelled Tibble"}
\format{
A tibble with 6,400 rows and 14 columns. Contains survey responses
  including outcome choices and conjoint attribute values (columns typically
  named with a `K-*-*` convention).
}
\source{
Qualtrics survey on Prolific; see Clayton et al. replication materials.
}
\usage{
data(exampleData1_labelled_tibble)
}
\description{
A cleaned tibble where each attribute corresponds to a separate column with a
descriptive attribute name. The unit of observation is each of two profiles in
each task for each respondent.
}
\details{
This dataset is intended for illustrating reading, reshaping, and
  analysis workflows in \pkg{projoint}. Column names are compatible with
  \code{reshape_projoint()}.
}
\examples{
# Load the data
data(exampleData1_labelled_tibble)

# Basic inspection (fast and always runnable)
head(exampleData1_labelled_tibble)
dim(exampleData1_labelled_tibble)

# Optional: quick structure peek (names only)
names(exampleData1_labelled_tibble)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exampleData3.R
\docType{data}
\name{exampleData3}
\alias{exampleData3}
\title{Projoint Example Data Set 3: Building Conjoint with No Repeated Task}
\format{
A data frame with 400 rows and 184 columns. Contains survey responses
  including demographic information, outcome choices, and conjoint attribute
  values identified by \code{K-*-*} variable names.
}
\usage{
data(exampleData3)
}
\description{
A cleaned Qualtrics export from a conjoint study that compares two potential
new building developments. Each respondent completed 8 standard tasks only;
no repeated tasks are included in this dataset.
}
\examples{
# Load the dataset
data(exampleData3)

# Inspect the first few rows
head(exampleData3)

# Number of rows and columns
dim(exampleData3)

# Display first 10 column names
names(exampleData3)[1:10]

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.projoint_results.R
\name{print.projoint_results}
\alias{print.projoint_results}
\title{Print method for projoint_results}
\usage{
\method{print}{projoint_results}(x, ...)
}
\arguments{
\item{x}{An object of class \code{projoint_results}.}

\item{...}{Additional arguments (ignored).}
}
\value{
No return value, called for its side effect of printing a summary of
  the \code{projoint_results} object to the console.
}
\description{
Custom print method for objects of class \code{projoint_results}.
}
\examples{
\donttest{
data(exampleData1)
dat <- reshape_projoint(
  exampleData1,
  .outcomes = c("choice1", "choice2", "choice1_repeated_flipped")
)
att <- unique(dat$labels$attribute_id)[1]
levs <- subset(dat$labels, attribute_id == att)$level_id
lev_names <- sub(".*:", "", levs)
q <- set_qoi(
  .structure     = "choice_level",
  .estimand      = "mm",
  .att_choose    = att,
  .lev_choose    = lev_names[2],
  .att_notchoose = att,
  .lev_notchoose = lev_names[1]
)
fit <- projoint(dat, .qoi = q)
print(fit)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exampleData1.R
\docType{data}
\name{exampleData1}
\alias{exampleData1}
\title{Projoint Example Data Set 1: Building Conjoint with a Repeated, Flipped Task}
\format{
A data frame with 400 rows and 185 columns. Contains survey responses
  including demographic information, outcome choices, and conjoint attribute
  values identified by \code{K-*-*} variable names.
}
\usage{
data(exampleData1)
}
\description{
A cleaned Qualtrics export from a conjoint study that compares two potential
new building developments. Each respondent completed 8 standard tasks as well
as a repeated version of the first task (flipped), which can be used to
calculate intra-respondent reliability (response instability).
}
\examples{
# Load the dataset
data(exampleData1)

# Inspect the first few rows
head(exampleData1)

# Number of rows and columns
dim(exampleData1)

# Display column names (truncated here)
names(exampleData1)[1:10]

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_projoint_choice_level_mm.R
\name{plot_projoint_choice_level_mm}
\alias{plot_projoint_choice_level_mm}
\title{Plot choice-level marginal means (MMs) (helper)}
\usage{
plot_projoint_choice_level_mm(
  x,
  .type = "pointrange",
  .estimates = "corrected",
  .labels = NULL,
  .show_attribute = TRUE,
  .remove_xaxis = FALSE,
  .xlim = c(0, 1),
  .plot.margin = c(top = 1, left = 2, bottom = 1, right = 2),
  ...
)
}
\arguments{
\item{x}{A \code{\link{projoint_results}} object produced by
\code{\link{projoint}} with \code{structure = "choice_level"} and
\code{estimand = "mm"}.}

\item{.type}{Character. Either \code{"bar"} (two bars with CIs) or
\code{"pointrange"} (horizontal estimate with CI and level labels at the
extremes). Default \code{"pointrange"}.}

\item{.estimates}{Character. Which estimate version to plot:
\code{"corrected"} (default) or \code{"uncorrected"}.}

\item{.labels}{Optional character vector of length 2 for custom level labels
(left/right). If \code{NULL}, labels are derived from \code{x$labels}.}

\item{.show_attribute}{Logical; if \code{TRUE} (default), add the attribute
name as the title when both levels are from the same attribute.}

\item{.remove_xaxis}{Logical; if \code{TRUE}, remove x-axis line, ticks, and
labels (useful when embedding). Default \code{FALSE}.}

\item{.xlim}{Numeric length-2 giving the x-axis limits. Default \code{c(0, 1)}.}

\item{.plot.margin}{Numeric vector of plot margins in cm,
\code{c(top, left, bottom, right)}. Default \code{c(1, 2, 1, 2)}.}

\item{...}{Currently unused (reserved for future extensions).}
}
\value{
A \code{ggplot2} object.
}
\description{
Internal helper used by \code{\link{plot.projoint_results}} to render
choice-level marginal means (MMs). Supports a bar chart or a horizontal
pointrange layout and optional custom level labels.
}
\details{
This helper expects that the \code{projoint_results} object \emph{already}
contains a single pair of choice-level MMs (i.e., one \code{att_level_choose}
and one \code{att_level_notchoose}). It is called internally by
\code{\link{plot.projoint_results}} when \code{x$structure == "choice_level"}
and \code{x$estimand == "mm"}.

When \code{.type = "pointrange"}, the level labels are placed just outside
\code{.xlim} to avoid overlap with the confidence interval. If the two levels
correspond to different attributes, the attribute title is omitted and a
message is emitted.
}
\examples{
data(exampleData1)
dat <- reshape_projoint(exampleData1,
  .outcomes = c("choice1", "choice2", "choice1_repeated_flipped")
)
att <- unique(dat$labels$attribute_id)[1]
levs <- subset(dat$labels, attribute_id == att)$level_id
lev_names <- sub(".*:", "", levs)
q <- set_qoi("choice_level", "mm",
  .att_choose = att, .lev_choose = lev_names[2],
  .att_notchoose = att, .lev_notchoose = lev_names[1]
)
fit <- projoint(dat, .qoi = q)
plot(fit)
}
\seealso{
\code{\link{plot.projoint_results}} for the user-facing plot method.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.projoint_results.R
\name{plot.projoint_results}
\alias{plot.projoint_results}
\title{Plot method for \code{projoint_results}}
\usage{
\method{plot}{projoint_results}(
  x,
  .estimates = "corrected",
  .by_var = FALSE,
  .labels = NULL,
  .base_size = 12,
  .base_family = "",
  .type = c("bar", "pointrange"),
  .show_attribute = TRUE,
  .remove_xaxis = FALSE,
  .xlim = c(0, 1),
  .plot.margin = c(0, 3, 0, 3),
  ...
)
}
\arguments{
\item{x}{A \code{projoint_results} object (typically from \code{\link{projoint}}).}

\item{.estimates}{Character: which estimates to plot. One of
\code{"corrected"}, \code{"uncorrected"}, or \code{"both"} (for profile-level),
and \code{"corrected"} or \code{"uncorrected"} (for choice-level). Default \code{"corrected"}.}

\item{.by_var}{Logical (profile-level only). Whether to plot subgroup differences.
Default \code{FALSE}.}

\item{.labels}{Character vector of length 2 (choice-level only). Custom x-axis
labels for bar/pointrange plots. If \code{NULL}, labels are taken from \code{x$labels}.}

\item{.base_size}{Numeric. Base font size. Default \code{12}.}

\item{.base_family}{Character. Base font family. Default \code{""} (system default).}

\item{.type}{Character (choice-level only). One of \code{"bar"} or \code{"pointrange"}.
Default \code{"bar"}.}

\item{.show_attribute}{Logical (choice-level only). Show the attribute name as the
title when both levels belong to the same attribute. Default \code{TRUE}.}

\item{.remove_xaxis}{Logical (choice-level only). Remove x-axis line, ticks, and labels.
Default \code{FALSE}.}

\item{.xlim}{Numeric length-2 vector (choice-level only). X-axis limits. Default \code{c(0, 1)}.}

\item{.plot.margin}{Numeric length-4 vector (choice-level only). Plot margins in cm:
\code{c(top, left, bottom, right)}. Default \code{c(0, 3, 0, 3)}.}

\item{...}{Additional arguments passed to downstream plotting helpers.}
}
\value{
A \code{ggplot2} object.
}
\description{
Creates publication-ready plots from a \code{projoint_results} object produced by
\code{\link{projoint}}. Supports both profile-level and choice-level analyses,
with plotting options tailored to each structure.
}
\details{
For profile-level results, only \code{.by_var}, \code{.base_size}, and \code{.base_family}
are relevant. For choice-level results, only \code{.type}, \code{.labels},
\code{.show_attribute}, \code{.remove_xaxis}, \code{.xlim}, and \code{.plot.margin}
are relevant. Irrelevant arguments are ignored with a warning.
}
\examples{
\donttest{
data(exampleData1)

# Two base tasks (1 & 2) + repeated of task 1 (last)
dat <- reshape_projoint(
  exampleData1,
  .outcomes = c("choice1", "choice2", "choice1_repeated_flipped")
)

# Build a valid QOI from the labels
att <- unique(dat$labels$attribute_id)[1]
levs <- subset(dat$labels, attribute_id == att)$level_id
lev_names <- sub(".*:", "", levs)

q <- set_qoi(
  .structure     = "choice_level",
  .estimand      = "mm",
  .att_choose    = att,
  .lev_choose    = lev_names[2],
  .att_notchoose = att,
  .lev_notchoose = lev_names[1]
)

fit <- projoint(dat, .qoi = q)

# Plot method
plot(fit)
}
}
\seealso{
\code{\link{projoint}}, \code{plot_projoint_profile_level},
  \code{\link{plot_projoint_choice_level_mm}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_labels.R
\name{read_labels}
\alias{read_labels}
\title{Read and apply a reordered attribute/level mapping}
\usage{
read_labels(.data, .filename)
}
\arguments{
\item{.data}{A \code{\link{projoint_data}} object whose labels/data should be reordered.}

\item{.filename}{Path to the revised labels CSV (originally produced from the package's labels).}
}
\value{
A \code{\link{projoint_data}} object with the same content as \code{.data} but with
  attributes and levels reordered to match the CSV. The returned object contains:
  \itemize{
    \item \code{$labels}: a tibble with new \code{attribute_id} and \code{level_id} reflecting the chosen order
    \item \code{$data}: a tibble whose \code{att*} columns have been remapped to the new \code{level_id}s
  }
}
\description{
Reads a CSV containing a revised ordering of attributes and levels and applies it
to an existing \code{\link{projoint_data}} object. Typical workflow: first save the
current labels to CSV (e.g., with \code{\link{save_labels}}), manually reorder rows
(and/or the attribute grouping) in the CSV, then call \code{read_labels()} to apply.
}
\examples{
\donttest{
# Create a projoint_data object from the example dataset
data(exampleData1)
outcomes <- c(paste0("choice", 1:8), "choice1_repeated_flipped")
pj <- reshape_projoint(exampleData1, outcomes)

# Write current labels to a temporary CSV, adding an 'order' column
tmp <- tempfile(fileext = ".csv")
pj$labels |>
  dplyr::mutate(order = dplyr::row_number()) |>
  readr::write_csv(tmp)

# (User would reorder rows in 'tmp' manually; we just read it back)
pj_reordered <- read_labels(pj, tmp)

# Inspect the updated label order
head(pj_reordered$labels)
}

}
\seealso{
\code{\link{save_labels}}, \code{\link{reshape_projoint}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_Qualtrics.R
\name{read_Qualtrics}
\alias{read_Qualtrics}
\title{Read and re-format a Qualtrics CSV (choice text)}
\usage{
read_Qualtrics(.file, .metadata_rows = NULL)
}
\arguments{
\item{.file}{A character string giving the path to a Qualtrics CSV file.}

\item{.metadata_rows}{Number of metadata rows after the CSV header. The
  default, \code{NULL}, detects current Qualtrics exports (question text plus
  an \code{ImportId} row) and legacy exports (question text only). Supply a
  non-negative integer to override detection.}
}
\value{
A tibble containing response rows only. Column names and physical
  column order are preserved from the Qualtrics export.
}
\description{
Reads a CSV file exported from Qualtrics (with "Use choice text" enabled) and
returns a data frame formatted for downstream processing with
\code{\link{reshape_projoint}}.
}
\examples{
\donttest{
# Write a tiny dummy Qualtrics-style CSV to a temp file
tmp <- tempfile(fileext = ".csv")
readr::write_csv(
  data.frame(Q1 = c("Choice Text", "{\"ImportId\":\"QID1\"}", "A", "B")),
  tmp
)
# Read it back in
df <- read_Qualtrics(tmp)
head(df)
}

}
\seealso{
\code{\link{reshape_projoint}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/projoint_level.R
\name{projoint_level}
\alias{projoint_level}
\title{Estimate profile- or choice-level effects (internal)}
\usage{
projoint_level(
  .data,
  .qoi,
  .structure,
  .estimand,
  .se_method,
  .irr,
  .remove_ties,
  .ignore_position,
  .n_sims,
  .n_boot,
  .weights_1,
  .clusters_1,
  .se_type_1,
  .weights_2,
  .clusters_2,
  .se_type_2,
  .auto_cluster = TRUE,
  .seed = NULL
)
}
\arguments{
\item{.data}{A \code{\link{projoint_data}} object.}

\item{.qoi}{Optional \code{\link{projoint_qoi}}; if \code{NULL}, estimates all
MMs/AMCEs implied by the design. When supplied, overrides \code{.structure}
and \code{.estimand}.}

\item{.structure}{Either \code{"profile_level"} or \code{"choice_level"}.}

\item{.estimand}{Either \code{"mm"} (marginal mean) or \code{"amce"} (average
marginal component effect).}

\item{.se_method}{One of \code{"analytical"}, \code{"simulation"}, or
\code{"bootstrap"}.}

\item{.irr}{\code{NULL} to estimate IRR from repeated tasks; numeric to fix IRR.}

\item{.remove_ties}{Logical; whether to drop tied responses (default \code{TRUE}).}

\item{.ignore_position}{Logical; choice-level only. Ignore left/right position?
Default \code{TRUE}.}

\item{.n_sims}{Integer; required when \code{.se_method = "simulation"}.}

\item{.n_boot}{Integer; required when \code{.se_method = "bootstrap"}.}

\item{.weights_1, .clusters_1, .se_type_1}{Arguments passed to
\code{\link[estimatr]{lm_robust}} for IRR estimation. If \code{.se_type_1}
is \code{NULL}, \emph{estimatr} defaults are used.}

\item{.weights_2, .clusters_2, .se_type_2}{Arguments passed to
\code{\link[estimatr]{lm_robust}} for MM/AMCE estimation. If \code{.se_type_2}
is \code{NULL}, \emph{estimatr} defaults are used.}

\item{.auto_cluster}{Logical; if \code{TRUE} (default) and an \code{id} column
is present while no clusters are provided, cluster automatically. Only applied
when the corresponding \code{.se_type_*} is \code{NULL}.}

\item{.seed}{Optional integer; if supplied, sets a temporary RNG seed for
simulation/bootstrap and restores prior state on exit.}
}
\value{
A \code{\link{projoint_results}} object containing:
\itemize{
  \item \code{estimates}: tibble of estimates (point estimate, SE, CI) with
        identifier columns (e.g., \code{att_level_*}).
  \item \code{estimand}: \code{"mm"} or \code{"amce"}.
  \item \code{structure}: \code{"profile_level"} or \code{"choice_level"}.
  \item \code{se_method}: SE computation method used.
  \item \code{irr}: character noting IRR usage (e.g., \code{"Estimated"} or
        \code{"Assumed (<value> )"}).
  \item \code{tau}: numeric \eqn{\tau} used to correct measurement error.
  \item \code{remove_ties}, \code{ignore_position}: flags echoed from inputs.
  \item \code{se_type_used}, \code{cluster_by}: details propagated from
        fitting calls (if available).
  \item \code{labels}, \code{data}: design labels and the analysis data
        passed through for downstream methods.
}
}
\description{
Core worker used by \code{\link{projoint}} to compute marginal means (MMs)
or average marginal component effects (AMCEs) under either the profile- or
choice-level structure. Handles IRR usage (estimated or fixed) and the
requested standard-error method.
}
\seealso{
\code{\link{projoint}}, \code{\link{pj_estimate}}, \code{\link{organize_data}},
  \code{\link{projoint_results}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/save_labels.R
\name{save_labels}
\alias{save_labels}
\title{Save attribute and level labels to a CSV file}
\usage{
save_labels(.data, .filename)
}
\arguments{
\item{.data}{A \code{\link{projoint_data}} object.}

\item{.filename}{A character string giving the name of a CSV file to be written.}
}
\value{
No return value, called for side effects (writes a CSV file).
}
\description{
Saves the attributes and levels (and their order) from a \code{\link{projoint_data}}
object, as generated by \code{\link{reshape_projoint}}, to a CSV file. This
enables manual reordering and later re-import via \code{\link{read_labels}}.
}
\examples{
\donttest{
library(projoint)
data(exampleData1)
reshaped <- reshape_projoint(
  exampleData1,
  .outcomes = c(paste0("choice", 1:8), "choice1_repeated_flipped")
)
tmpfile <- tempfile(fileext = ".csv")
save_labels(reshaped, tmpfile)
readLines(tmpfile, n = 5)  # show first few lines
}

}
\seealso{
\code{\link{read_labels}}, \code{\link{reshape_projoint}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict_tau.R
\name{predict_tau}
\alias{predict_tau}
\title{Estimate intra-respondent reliability (tau) without a repeated task}
\usage{
predict_tau(.data, .title = NULL)
}
\arguments{
\item{.data}{A \code{\link{projoint_data}} object (from \code{\link{reshape_projoint}}).}

\item{.title}{Optional character string used as the plot title prefix.}
}
\value{
A \code{projoint_tau} object (list-like) with components:
\itemize{
  \item \code{$irr}: a tibble with columns \code{x} (number of differing attributes) and
        \code{predicted} (fitted agreement), including \code{x = 0} which is the estimate
        of \eqn{\tau}.
  \item \code{$figure}: a \code{ggplot2} object visualizing observed agreement by \code{x}
        and the fitted line with the extrapolated point at \code{x = 0}.
}
}
\description{
Uses the extrapolation method to estimate intra-respondent reliability (IRR, \eqn{\tau})
when your conjoint design does not include an explicit repeated task. The input is a
\code{\link{projoint_data}} object (e.g., produced by \code{\link{reshape_projoint}}).
}
\details{
The procedure constructs pairs of base tasks within respondent, computes the proportion
of identical choices as a function of how many attributes differ between the two tasks,
fits a weighted regression of agreement on the number of differing attributes, and
extrapolates to zero differences to obtain \eqn{\hat{\tau}}.
}
\examples{
\donttest{
# Example workflow:
data(exampleData1)
outcomes <- c(paste0("choice", 1:8), "choice1_repeated_flipped")

# Even if your real study lacks a repeated task, this shows the API:
pj <- reshape_projoint(exampleData1, outcomes, .repeated = TRUE)

tau_fit <- predict_tau(pj, .title = "IRR (tau): ")
# Inspect the extrapolated tau (row where x == 0)
tau_fit$irr[tau_fit$irr$x == 0, ]

# Plot (also available via plot(tau_fit))
print(tau_fit$figure)
}

}
\seealso{
\code{\link{plot.projoint_tau}}, \code{\link{summary.projoint_tau}},
  \code{\link{reshape_projoint}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_qoi.R
\name{set_qoi}
\alias{set_qoi}
\title{Set the quantities of interest (QoIs)}
\usage{
set_qoi(
  .structure = "choice_level",
  .estimand = "mm",
  .att_choose,
  .lev_choose,
  .att_notchoose = NULL,
  .lev_notchoose = NULL,
  .att_choose_b = NULL,
  .lev_choose_b = NULL,
  .att_notchoose_b = NULL,
  .lev_notchoose_b = NULL
)
}
\arguments{
\item{.structure}{Either \code{"choice_level"} (default) or \code{"profile_level"}.}

\item{.estimand}{Either \code{"mm"} for marginal means or \code{"amce"} for
average marginal component effects.}

\item{.att_choose}{Character scalar: the attribute (column) for the level(s)
that are \emph{chosen}.}

\item{.lev_choose}{Character vector: the level id(s) for the \emph{chosen}
side. Length 1 for profile-level, \eqn{\ge}1 for choice-level.}

\item{.att_notchoose}{Character scalar: the attribute (column) for the level(s)
that are \emph{not chosen}. Only used for \code{.structure == "choice_level"}.}

\item{.lev_notchoose}{Character vector: the level id(s) for the \emph{not chosen}
side. Length 1 for profile-level, \eqn{\ge}1 for choice-level. Only used for
\code{.structure == "choice_level"}.}

\item{.att_choose_b}{Character scalar: \emph{baseline} attribute for the
chosen side when computing AMCEs.}

\item{.lev_choose_b}{Character vector: \emph{baseline} level id(s) for the
chosen side when computing AMCEs. Length 1 for profile-level, \eqn{\ge}1 for
choice-level.}

\item{.att_notchoose_b}{Character scalar: \emph{baseline} attribute for the
not-chosen side (choice-level only) when computing AMCEs.}

\item{.lev_notchoose_b}{Character vector: \emph{baseline} level id(s) for the
not-chosen side (choice-level only) when computing AMCEs.}
}
\value{
A \code{projoint_qoi} object (list-like) containing fields such as:
  \code{structure}, \code{estimand}, \code{attribute_of_interest},
  \code{levels_of_interest}, and their baseline counterparts. This object
  can be supplied to downstream estimation helpers that accept a QoI spec.
}
\description{
Constructs a quantities-of-interest (QoI) specification for \pkg{projoint}.
Use this to request specific estimands—marginal means (MMs) or average
marginal component effects (AMCEs)—at either the choice- or profile-level,
and to declare which attribute levels are compared (including baselines).
}
\examples{
# Specify a simple choice-level MM comparison for att1 levels:
q_mm <- set_qoi(
  .structure  = "choice_level",
  .estimand   = "mm",
  .att_choose = "att1",
  .lev_choose = c("att1:lev2"),
  .att_notchoose = "att1",
  .lev_notchoose = c("att1:lev1")
)
str(q_mm)

# Example AMCE with explicit baselines (profile-level):
q_amce <- set_qoi(
  .structure   = "profile_level",
  .estimand    = "amce",
  .att_choose  = "att2",
  .lev_choose  = "att2:lev3",
  .att_choose_b = "att2",
  .lev_choose_b = "att2:lev1"
)
str(q_amce)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.projoint_data.R
\name{print.projoint_data}
\alias{print.projoint_data}
\title{Print a projoint_data object}
\usage{
\method{print}{projoint_data}(x, ...)
}
\arguments{
\item{x}{A \code{projoint_data} object.}

\item{...}{Additional arguments (currently unused).}
}
\value{
No return value, called for its side effect of printing a summary of
  the \code{projoint_data} object to the console.
}
\description{
Custom print method for objects of class \code{projoint_data}.
}
\examples{
\donttest{
data(exampleData1)
dat <- reshape_projoint(
  exampleData1,
  .outcomes = c("choice1", "choice2")
)
print(dat)
}
}
